"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BDateTime = exports.BDateTimeError = void 0;
const luxon_1 = require("luxon");
const BDateTimeCount_1 = require("./BDateTimeCount");
const BDuration_1 = require("./BDuration");
const BTiming_1 = require("./BTiming");
const FrameRate_1 = require("./FrameRate");
class BDateTimeError extends Error {
    constructor(message) {
        super(message);
        this.name = "BDateTimeError";
    }
}
exports.BDateTimeError = BDateTimeError;
const PRIORITY_FLOOR_VALUE = 946684800000;
const PRIORITY_TOP_VALUE = 4102444800000;
const PRIORITY_MAX_VALUE = PRIORITY_TOP_VALUE - PRIORITY_FLOOR_VALUE;
class BDateTime extends BTiming_1.BTiming {
    /**
     * Returns a BDateTime representing the time now in UTC
     * @param frameRate - frame rate
     * @example
     * ```
     * const now = busby.bTime.BDateTime.utcNow(frameRate);
     * ```
     */
    static utcNow(frameRate) {
        let fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        const now = luxon_1.DateTime.utc();
        const fid = Math.round((now.valueOf() % (24 * 60 * 60 * 1000)) / 1000 * fr.getFramesPerSecond());
        return new BDateTime(now.set({ hour: 0, minute: 0, second: 0, millisecond: 0 }), fid, fr);
    }
    static fromTimestamp(tsFr, frTs) {
        let timestamp;
        let frameRate;
        if (typeof tsFr === "number") {
            timestamp = tsFr;
        }
        else {
            frameRate = typeof tsFr == "string" ? new FrameRate_1.FrameRate(tsFr) : tsFr;
        }
        if (typeof frTs === "number") {
            timestamp = frTs;
        }
        else {
            frameRate = typeof frTs == "string" ? new FrameRate_1.FrameRate(frTs) : frTs;
        }
        const dt = luxon_1.DateTime.fromMillis(timestamp);
        const fid = Math.round((dt.valueOf() % (24 * 60 * 60 * 1000)) / 1000 * frameRate.getFramesPerSecond());
        return new BDateTime(dt, fid, frameRate);
    }
    /**
     * Create a BDateTime from a json representation of the BDateTime
     * @param jo an object represenging a BDateTime
     * @returns A BDateTime instance from a json object representation
     */
    static fromJson(jo) {
        if (!jo || jo.time == undefined) {
            return undefined;
        }
        const frameRate = new FrameRate_1.FrameRate(jo.frameRate || "ndf25");
        const frameInDay = typeof jo.time === "string" ? BTiming_1.BTiming.decodeTcString(jo.time, frameRate, true) : jo.time;
        return new BDateTime(luxon_1.DateTime.fromISO(jo.date || "1970-01-01T00:00:00", { zone: "UTC" }), frameInDay, frameRate);
    }
    /**
     * Create a BDateTime from a date and timecode
     * @param date the date for the broadcast day
     * @param bdt the time string in timecode format (HH:mm:ss:ff)
     * @param frameRate the framerate
     * @returns A BDateTime object for the specified date and timecode string
     */
    static fromBroadcastDayDateTime(date, bdt, frameRate) {
        let fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        let hh = Number.parseInt(bdt.substr(0, 2), 10);
        if (hh > 23) { // wrap the date
            hh -= 24;
            bdt = ((hh < 10) ? "0" : "") + hh + bdt.substr(2);
            if (date instanceof luxon_1.DateTime) {
                date = date.plus({ day: 1 });
            }
            else {
                date.setDate(date.getDate() + 1);
            }
        }
        if (date instanceof luxon_1.DateTime) {
            date = date.set({ hour: 0, minute: 0, second: 0, millisecond: 0 });
        }
        let frameInDay = BTiming_1.BTiming.decodeTcString(bdt, fr, true);
        return new BDateTime(date, frameInDay, fr);
    }
    /**
     * Create a BDateTime frim an ISO date time string
     * @param iso ISO date time string
     * @param frameRate frame rate
     * @returns A BDateTime object for the specified date and time string
     * @example
     * ```
     * const iso = "2021-04-10T20:04:11.040";
     * const isoDateTime = busby.bTime.BDateTime.fromIso(iso, framerate)
     * ```
     */
    static fromIso(iso, frameRate) {
        const fullIsoWithTimezone = iso.match(/T[0-9]{2}:[0-9]{2}:[0-9]{2}(\.[0-9]{1,3})?(Z|[+-][0-9]{2}:[0-9]{2})$/);
        const timeCodeWithTimezone = iso.match(/T([0-9]{2}:[0-9]{2}:[0-9]{2})([:;][0-9]{2})(Z|[+-][0-9]{2}:[0-9]{2})$/);
        let now = fullIsoWithTimezone ? luxon_1.DateTime.fromISO(iso, { setZone: true }).setZone("UTC") : luxon_1.DateTime.fromISO(iso.substr(0, 10), { zone: "UTC" });
        let timezoneOffset = 0;
        if (timeCodeWithTimezone) {
            const inTimeZone = luxon_1.DateTime.fromISO(iso.replace(/^([^T]+T)([0-9]{2}:[0-9]{2}:[0-9]{2})([;:][0-9]{2})(.*)/, "$1$2$4"), { setZone: true });
            timezoneOffset = inTimeZone.offset;
        }
        let timePart = 0;
        let fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        if (iso.length > 10) {
            if (iso[19] === "." || fullIsoWithTimezone) {
                if (fullIsoWithTimezone) {
                    const time = now.toISOTime();
                    now = now.set({ hour: 0, minute: 0, second: 0, millisecond: 0 });
                    timePart = BTiming_1.BTiming.decodeIsoTime(time, fr, true);
                }
                else {
                    timePart = BTiming_1.BTiming.decodeIsoTime(iso.substr(11, 12), fr, true);
                }
            }
            else {
                timePart = BTiming_1.BTiming.decodeTcString(iso.substr(11, 11), fr, true);
                if (timezoneOffset !== 0) {
                    timePart -= fr.getFramesPerSecond() * 60 * timezoneOffset;
                }
                // throw new BDateTimeError(`Incorrect ISO time fromat (${iso}) should be using fromSmpte as the time is a timecode string`);
            }
        }
        if (!now.isValid || isNaN(timePart)) {
            throw new BDateTimeError(`Invalid ISO string ${iso}`);
        }
        return new BDateTime(now, timePart, fr);
    }
    /**
     * Returns a BDateTime for the specified smpte timecode string
     * @param smpte ISO date time string with time in timecode format
     * @param frameRate frame rate
     * @example
     * ```
     * const smpte = "2021-04-10T20:04:11:10";
     * const smpteDateTime = busby.bTime.BDateTime.fromSmpte(smpte, framerate);
     * ```
     */
    static fromSmpte(smpte, frameRate) {
        const now = luxon_1.DateTime.fromFormat(smpte.substr(0, 10), "yyyy-MM-dd", { zone: "UTC" });
        let timePart = 0;
        let fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        if (smpte.length > 10) {
            if (smpte[19] === "." && fr.getLabel() !== "milli") {
                throw new BDateTimeError(`Incorrect SMPTE fromat (${smpte})`);
            }
            timePart = BTiming_1.BTiming.decodeTcString(smpte.substr(11, 11), fr, true);
        }
        return new BDateTime(now, timePart, fr);
    }
    /**
     * Returns a BDateTime for the specified flex timecode string
     * @param flexTimecode ISO date time string with time in flex timecode format
     * @param frameRate frame rate
     * @example
     * ```
     * const flex = "2021-04-10T20:04:11:10";
     * const flexDateTime = busby.bTime.BDateTime.fromFlex(flexTimecode, framerate);
     * ```
     */
    static fromGvAmpp(flexTimecode, frameRate) {
        const now = new Date(0);
        const fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        const timePart = BTiming_1.BTiming.decodeGvAmppString(flexTimecode, fr, true);
        return new BDateTime(now, timePart, fr);
    }
    /**
     * Returns a BDateTime representing the timecode passed in
     * @param timecode timecode as string
     * @param frameRate {FrameRate} - frame rate
     * @returns {BDateTime}
     * @example
     * ```
     * const startBDateTime = busby.bTime.BDateTime.fromTimecode("01:00:00:00", framerate)
     * ```
     */
    static fromTimecode(timecode, frameRate) {
        let fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        return new BDateTime(new Date(0), BTiming_1.BTiming.decodeTcString(timecode, fr, true), fr);
    }
    /**
     * Returns a BDateTime representing the job priority passed in
     * @param priority the jobs priority value
     * @param frameRate frame rate
     * @returns {BDateTime}
     * @example
     * ```
     * const priorityBDateTime = busby.bTime.BDateTime.fromTimecode(busby.root.job.priority, framerate)
     * ```
     */
    static fromPriority(priority, frameRate = "milli") {
        let fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        const timeInMs = (priority * PRIORITY_MAX_VALUE) + PRIORITY_FLOOR_VALUE;
        const dt = luxon_1.DateTime.fromMillis(timeInMs, { zone: "UTC" });
        const date = dt.set({ hour: 0, minute: 0, second: 0, millisecond: 0 });
        const time = this.decodeIsoTime(dt.toISOTime(), fr, true);
        return new BDateTime(date, time, fr);
    }
    get date() {
        return this.getDateString();
    }
    get time() {
        return this._frameInDay;
    }
    /**
     * Constructs a broadcast date time object
     * @param date the date value for the object
     * @param frameInDay the frame in the day
     * @param frameRate the frame rate
     * @deprecated Should be constructed using one of the static functions
     */
    constructor(date, frameInDay, frameRate) {
        super(frameRate, true);
        if (isNaN(frameInDay)) {
            throw new BDateTimeError("Invalid time");
        }
        if (isNaN(date.valueOf())) {
            throw new BDateTimeError("Invalid date");
        }
        this._frameInDay = frameInDay;
        this._date = date instanceof Date ? luxon_1.DateTime.fromJSDate(date) : date;
        if (frameInDay < 0) {
            let nf = this._frameInDay;
            let nd = this._date;
            while (nf > this._frameRate.getFramesInDay()) {
                nf -= this._frameRate.getFramesInDay();
                nd = this._date.plus({ day: 1 });
            }
            while (nf < 0) {
                nf += this._frameRate.getFramesInDay();
                nd = this._date.minus({ day: 1 });
            }
            this._frameInDay = nf;
            this._date = nd;
        }
    }
    /**
     * Gets the BDateTime as a job priority number between 0 and 1
     */
    toPriority() {
        const date = this.toLuxon();
        const priority = (date.valueOf() - PRIORITY_FLOOR_VALUE) / PRIORITY_MAX_VALUE;
        return priority;
    }
    toJson(timeAsFrames) {
        const result = {};
        this.addToJson(result);
        if (this._date.valueOf() > 0) {
            result.date = this.getDateString();
        }
        if (timeAsFrames) {
            result.time = this._frameInDay;
        }
        else {
            result.time = this.getTimeString();
        }
        result.frameRate = this._frameRate.getFrtype();
        return result;
    }
    diff(time, ignoreDate) {
        if (ignoreDate) {
            const frames = Math.abs(time.time - this.time);
            return new BDuration_1.BDuration(frames, this._frameRate);
        }
        else {
            const frames = this.framesDifference(time);
            return new BDuration_1.BDuration(frames, this._frameRate);
        }
    }
    diffNow() {
        const frames = this.framesDifference(BDateTime.utcNow(this._frameRate));
        return new BDuration_1.BDuration(frames, this._frameRate);
    }
    framesDifference(nowTime) {
        const diff = this._date.diff(nowTime._date).as("days");
        const daysDiff = Math.floor(diff);
        return (daysDiff * this._frameRate.getFramesInDay()) + (this._frameInDay - nowTime._frameInDay);
    }
    addFrames(frames) {
        let nf = this._frameInDay + frames;
        let nd = this._date;
        while (nf >= this._frameRate.getFramesInDay()) {
            nf -= this._frameRate.getFramesInDay();
            nd = this._date.plus({ day: 1 });
        }
        while (nf < 0) {
            nf += this._frameRate.getFramesInDay();
            nd = this._date.minus({ day: 1 });
        }
        return new BDateTime(nd, nf, this._frameRate);
    }
    addDuration(dur) {
        return this.addFrames(dur.getFrames());
    }
    subtractDuration(dur) {
        const negativeDuration = dur.getNegativeDuration();
        return this.addFrames(negativeDuration.getFrames());
    }
    getTimeString(withMs) {
        return BTiming_1.BTiming.encodeTcString(this._frameInDay, this._frameRate, true, withMs);
    }
    getGvTimeString() {
        return this.getTimeString(7);
    }
    getDateString() {
        return this._date.toFormat("yyyy-MM-dd");
    }
    toString() {
        return this.getDateString() + " " + this.getTimeString();
    }
    toIsoString(withMs) {
        return this.getDateString() + "T" + this.getTimeString(withMs) + (withMs ? "Z" : "");
    }
    toGvAmppString() {
        return this.getTimeString(7);
    }
    toGvAmppIsoString() {
        return this.toIsoString(7);
    }
    equals(other) {
        return (this._frameInDay == other._frameInDay) && (this._frameRate.getFrtype() == other._frameRate.getFrtype()) && (this._date.valueOf() == other._date.valueOf());
    }
    getComponents(fodMode = this.fodMode) {
        const result = BTiming_1.BTiming.toComp(this._frameInDay, this._frameRate, fodMode);
        result.day = this._date.day;
        result.month = this._date.month;
        result.year = this._date.year;
        return result;
    }
    getAsCounter(nowTime, mode, rate, autoStop) {
        return new BDateTimeCount_1.BDateTimeCount(this, nowTime, mode, rate, autoStop);
    }
    /**
     * @returns the frame in the day
     */
    getFrames() {
        return this._frameInDay;
    }
    /**
     * @returns a JS Date which is hust the day
     */
    getDate() {
        return this._date.toJSDate();
    }
    /**
     * Converts the BDateTime to a JS Date with the time.
     * @returns a JS Date object with time
     */
    toDate() {
        return this._date.plus(luxon_1.Duration.fromMillis(this.getMillis())).toJSDate();
    }
    toLuxon() {
        return this._date.plus(luxon_1.Duration.fromMillis(this.getMillis()));
    }
    getMillis() {
        return this._frameInDay * (1000 / this._frameRate.getFramesPerSecond());
    }
    valueOf() {
        return this._date.valueOf() + this.getMillis();
    }
}
exports.BDateTime = BDateTime;
