"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BDuration = void 0;
const BTiming_1 = require("./BTiming");
const FrameRate_1 = require("./FrameRate");
const BDurationCount_1 = require("./BDurationCount");
class BDurationError extends Error {
    constructor(message) {
        super(message);
        this.name = "BDurationError";
    }
}
class BDuration extends BTiming_1.BTiming {
    get duration() {
        return this._frames;
    }
    static fromJson(jo) {
        if (!jo) {
            return undefined;
        }
        const fr = new FrameRate_1.FrameRate(jo?.frameRate || "ndf25");
        const frames = typeof jo.duration === "string" ? BTiming_1.BTiming.decodeTcString(jo.duration, fr, false) : jo.duration;
        return new BDuration(frames, fr);
    }
    static fromTcString(tcString, frameRate) {
        const fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        return new BDuration(BTiming_1.BTiming.decodeTcString(tcString, fr, false), fr);
    }
    static fromFrames(frames, frameRate) {
        const fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        return new BDuration(frames, fr);
    }
    /**
     * Returns a BDuration for the specified flex timecode string
     * @param flexTimecode ISO date time string with time in flex timecode format
     * @param frameRate frame rate
     * @example
     * ```
     * const flex = "2021-04-10T20:04:11:10";
     * const flexDateTime = busby.bTime.BDuration.fromFlex(flexTimecode, framerate);
     * ```
     */
    static fromGvAmpp(flexTimecode, frameRate) {
        const fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        const frames = BTiming_1.BTiming.decodeGvAmppString(flexTimecode, fr, false);
        return new BDuration(frames, fr);
    }
    static zero(frameRate) {
        const fr = typeof frameRate == "string" ? new FrameRate_1.FrameRate(frameRate) : frameRate;
        return new BDuration(0, fr);
    }
    constructor(frames, fr) {
        super(fr, false);
        if (isNaN(frames)) {
            throw new BDurationError("Invalid duration");
        }
        this._frames = frames;
    }
    toJson(durationAsFrames) {
        const result = {};
        this.addToJson(result);
        if (durationAsFrames) {
            result.duration = this._frames;
        }
        else {
            result.duration = this.toString();
        }
        result.frameRate = this._frameRate.getFrtype();
        return result;
    }
    addFrames(fta) {
        return new BDuration(fta + this._frames, this._frameRate);
    }
    addHours(hours) {
        const frames = this._frameRate.getFramesPerSecond() * 60 * 60 * hours;
        return this.addFrames(frames);
    }
    plus(amountOrObject, unit) {
        const fps = this._frameRate.getFramesPerSecond();
        let frames = 0;
        if (typeof amountOrObject === "number") {
            amountOrObject = { [unit]: amountOrObject };
        }
        for (const [key, value] of Object.entries(amountOrObject)) {
            switch (key) {
                case "day":
                case "days": {
                    frames += fps * 24 * 60 * 60 * value;
                    break;
                }
                case "hour":
                case "hours": {
                    frames += fps * 60 * 60 * value;
                    break;
                }
                case "minute":
                case "minutes": {
                    frames += fps * 60 * value;
                    break;
                }
                case "second":
                case "seconds": {
                    frames += fps * value;
                    break;
                }
                case "frame":
                case "frames": {
                    frames += value;
                    break;
                }
            }
        }
        return this.addFrames(Math.round(frames));
    }
    addDuration(other) {
        if (other._frameRate === this._frameRate) {
            return new BDuration(other._frames + this._frames, this._frameRate);
        }
        else {
            return new BDuration(other._frames * (this._frameRate.getFramesPerSecond() / other._frameRate.getFramesPerSecond()) + this._frames, this._frameRate);
        }
    }
    toString(withMs, withDays) {
        return BTiming_1.BTiming.encodeTcString(this._frames, this._frameRate, this.fodMode, withMs, withDays);
    }
    toGvAmppString() {
        return BTiming_1.BTiming.encodeTcString(this._frames, this._frameRate, this.fodMode, 7, true);
    }
    getAsCounter(now, mode, rate = 1) {
        return new BDurationCount_1.BDurationCount(this, now, mode, rate);
    }
    getFrames() {
        return this._frames;
    }
    getMilliseconds() {
        return this._frames / this._frameRate.getFramesPerSecond() * 1000;
    }
    getSeconds() {
        return Math.floor(this.getMilliseconds() / 1000);
    }
    getMinutes() {
        return this.getSeconds() / 60;
    }
    getHours() {
        return this.getMinutes() / 60;
    }
    equals(other) {
        return (this._frames == other.getFrames()) && (this._frameRate.getFrtype() == other._frameRate.getFrtype());
    }
    isGreaterThan(other) {
        return (this.getMilliseconds() > other.getMilliseconds());
    }
    isLessThan(other) {
        return (this.getMilliseconds() < other.getMilliseconds());
    }
    isGreaterThanOrEqual(other) {
        return (this.getMilliseconds() >= other.getMilliseconds());
    }
    isLessThanOrEqual(other) {
        return (this.getMilliseconds() <= other.getMilliseconds());
    }
    getComponents() {
        return BTiming_1.BTiming.toComp(this._frames, this._frameRate, false);
    }
    getNegativeDuration() {
        return BDuration.fromFrames(-this._frames, this._frameRate);
    }
    isNegative() {
        return this._frames < 0;
    }
}
exports.BDuration = BDuration;
