"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BTiming = exports.BTimingError = void 0;
class BTimingError extends Error {
    constructor(message) {
        super(message);
        this.name = "BTimingError";
    }
}
exports.BTimingError = BTimingError;
class BTiming {
    constructor(frameRate, fodMode) {
        this._frameRate = frameRate;
        this.fodMode = fodMode;
    }
    get frameRate() {
        return this._frameRate.getFrtype();
    }
    getFrameRate() {
        return this._frameRate;
    }
    addToJson(jo) {
        jo.frameRate = this._frameRate.getLabel();
    }
    static intDiv(a, b) {
        return Math.trunc(a / b);
    }
    static decodeTcString(tcString, frameRate, fodMode) {
        if (typeof tcString !== "string") {
            throw new TypeError(`tcString is not a string: ${typeof tcString}`);
        }
        let negative = false;
        if (tcString[0] === '-') {
            tcString = tcString.substr(1);
            negative = true;
        }
        const hh = parseInt(tcString.substring(0, 2));
        const mm = parseInt(tcString.substring(3, 5));
        const ss = parseInt(tcString.substring(6, 8));
        let ff = 0;
        if (frameRate.getFrtype() === "milli") {
            ff = tcString.length > 8 ? parseInt(tcString.substring(9, 12)) : 0;
        }
        else {
            ff = tcString.length > 8 ? parseInt(tcString.substring(9, 11)) : 0;
        }
        const calcFrame = this.calcFrame(hh, mm, ss, ff, frameRate, fodMode);
        return negative ? -calcFrame : calcFrame;
    }
    static decodeIsoTime(isoTimeString, frameRate, fodMode) {
        const hh = parseInt(isoTimeString.substring(0, 2));
        const mm = parseInt(isoTimeString.substring(3, 5));
        const ss = parseInt(isoTimeString.substring(6, 8));
        const us = isoTimeString.length > 9 ?
            parseInt(isoTimeString.substring(9, 12).padEnd(3, "0")) * 1000
            : 0;
        const ff = us / frameRate.getUsPerFrame();
        const calcFrame = this.calcFrame(hh, mm, ss, ff, frameRate, fodMode);
        return calcFrame;
    }
    static decodeGvAmppString(tcString, frameRate, fodMode) {
        if (typeof tcString !== "string") {
            throw new TypeError(`tcString is not a string: ${typeof tcString}`);
        }
        const regex = /^(?<sign>[-+]?)((?<day>[0-9]{1,4})\.)?(?<hour>[0-9]{2}):(?<minute>[0-9]{2}):(?<second>[0-9]{2})(\.(?<millisecond>[0-9]+)?)?Z?$/;
        const matches = regex.exec(tcString);
        if (!matches) {
            throw new TypeError(`invalid flex timecode string ${tcString}`);
        }
        const negative = matches.groups.sign === "-";
        const dd = parseInt(matches.groups.day ?? '0') * 24 * 60 * 60;
        const hh = parseInt(matches.groups.hour ?? '0') * 60 * 60;
        const mm = parseInt(matches.groups.minute ?? '0') * 60;
        const ss = parseInt(matches.groups.second ?? '0');
        const milli = (matches.groups.millisecond ?? '0').padEnd(7, '0');
        const ms = parseInt(milli);
        const seconds = dd + hh + mm + ss;
        const shifted = seconds * Math.pow(10, 7);
        const full = shifted + ms + 0.1;
        const fps = frameRate.getFramesPerSecond() / Math.pow(10, 7);
        const frames = Math.round((full * fps) + 0.0000001);
        return negative ? -frames : frames;
    }
    static calcFrame(hh, mm, ss, ff, frameRate, fodMode) {
        let calcFrame = 0;
        switch (frameRate.getLabel()) {
            case "ndf24":
                {
                    calcFrame = (ff + (ss * 24) + (mm * 24 * 60) + (hh * 24 * 60 * 60)) %
                        (24 * 60 * 60 * 24);
                }
                break;
            case "ndf25":
                {
                    calcFrame = (ff + (ss * 25) + (mm * 25 * 60) + (hh * 25 * 60 * 60)) %
                        (25 * 60 * 60 * 24);
                }
                break;
            case "ndf30":
                {
                    calcFrame = (ff + (ss * 30) + (mm * 30 * 60) + (hh * 30 * 60 * 60)) %
                        (30 * 60 * 60 * 24);
                }
                break;
            case "ndf50":
                {
                    calcFrame = (ff + (ss * 50) + (mm * 50 * 60) + (hh * 50 * 60 * 60)) %
                        (50 * 60 * 60 * 24);
                }
                break;
            case "ndf60":
                {
                    calcFrame = (ff + (ss * 60) + (mm * 60 * 60) + (hh * 60 * 60 * 60)) %
                        (60 * 60 * 60 * 24);
                }
                break;
            case "df30":
                {
                    if (fodMode) {
                        calcFrame = ((30 * 60 * 60 * hh) + (30 * 60 * mm) + (30 * ss) + ff)
                            - (2 * (((60 * hh) + mm) - this.intDiv((60 * hh) + mm, 10)));
                    }
                    else {
                        calcFrame = BTiming.nonFodDecode(hh, mm, ss, ff, frameRate);
                    }
                }
                break;
            case "df24":
                {
                    if (fodMode) {
                        // there appears to no standard for 24fps drop frame timecode. This is wrong
                        // but good enough for some uses
                        calcFrame = BTiming.nonFodDecode(hh, mm, ss, ff, frameRate);
                    }
                    else {
                        calcFrame = BTiming.nonFodDecode(hh, mm, ss, ff, frameRate);
                    }
                }
                break;
            case "df60":
                {
                    if (fodMode) {
                        calcFrame = ((60 * 60 * 60 * hh) + (60 * 60 * mm) + (60 * ss) + ff)
                            - (4 * (((60 * hh) + mm) - this.intDiv((60 * hh) + mm, 10)));
                    }
                    else {
                        calcFrame = BTiming.nonFodDecode(hh, mm, ss, ff, frameRate);
                    }
                }
                break;
            case "milli":
                {
                    calcFrame = (ff + (ss * 1000) + (mm * 1000 * 60) + (hh * 1000 * 60 * 60)) %
                        (1000 * 60 * 60 * 24);
                }
                break;
        }
        return calcFrame;
    }
    static nonFodDecode(hh, mm, ss, ff, frameRate) {
        const fps = frameRate.getFramesPerSecond();
        let reqDur = Math.trunc(fps) < fps ? (0.5 * (1 / fps)) : 0;
        reqDur += ss;
        reqDur += mm * 60;
        reqDur += hh * 60 * 60;
        reqDur = (reqDur * fps) + ff;
        return Math.trunc(reqDur);
    }
    static toComp(frames, frameRate, fodMode) {
        let result = { hours: 0, minutes: 0, seconds: 0, frames: 0 };
        if (fodMode) {
            result.days = 0;
        }
        if (frames < 0) {
            frames = frames * -1;
            result.sign = "-";
        }
        else {
            result.sign = "";
        }
        switch (frameRate.getLabel()) {
            case "ndf24":
                {
                    result.days = BTiming.intDiv(frames, 24 * 60 * 60 * 24);
                    result.hours = BTiming.intDiv(frames, 24 * 60 * 60) % 24;
                    result.minutes = BTiming.intDiv(frames, 24 * 60) % 60;
                    result.seconds = BTiming.intDiv(frames, 24) % 60;
                    result.frames = Math.trunc(frames % 24);
                }
                break;
            case "ndf25":
                {
                    result.days = BTiming.intDiv(frames, 25 * 60 * 60 * 24);
                    result.hours = BTiming.intDiv(frames, 25 * 60 * 60) % 24;
                    result.minutes = BTiming.intDiv(frames, 25 * 60) % 60;
                    result.seconds = BTiming.intDiv(frames, 25) % 60;
                    result.frames = Math.trunc(frames % 25);
                }
                break;
            case "ndf30":
                {
                    result.days = BTiming.intDiv(frames, 30 * 60 * 60 * 24);
                    result.hours = BTiming.intDiv(frames, 30 * 60 * 60) % 24;
                    result.minutes = BTiming.intDiv(frames, 30 * 60) % 60;
                    result.seconds = BTiming.intDiv(frames, 30) % 60;
                    result.frames = Math.trunc(frames % 30);
                }
                break;
            case "ndf50":
                {
                    result.days = BTiming.intDiv(frames, 50 * 60 * 60 * 24);
                    result.hours = BTiming.intDiv(frames, 50 * 60 * 60) % 24;
                    result.minutes = BTiming.intDiv(frames, 50 * 60) % 60;
                    result.seconds = BTiming.intDiv(frames, 50) % 60;
                    result.frames = Math.trunc(frames % 50);
                }
                break;
            case "ndf60":
                {
                    result.days = BTiming.intDiv(frames, 60 * 60 * 60 * 24);
                    result.hours = BTiming.intDiv(frames, 60 * 60 * 60) % 24;
                    result.minutes = BTiming.intDiv(frames, 60 * 60) % 60;
                    result.seconds = BTiming.intDiv(frames, 60) % 60;
                    result.frames = Math.trunc(frames % 60);
                }
                break;
            case "df30":
                {
                    if (fodMode) {
                        result = BTiming.convertDropToComponents(30, frames, result);
                    }
                    else {
                        result = BTiming.nonFodEncode(frames, frameRate);
                    }
                }
                break;
            case "df24":
                {
                    if (fodMode) {
                        result = BTiming.nonFodEncode(frames, frameRate);
                    }
                    else {
                        result = BTiming.nonFodEncode(frames, frameRate);
                    }
                }
                break;
            case "df60":
                {
                    if (fodMode) {
                        result = BTiming.convertDropToComponents(60, frames, result);
                    }
                    else {
                        result = BTiming.nonFodEncode(frames, frameRate);
                    }
                }
                break;
            case "milli":
                {
                    result.days = BTiming.intDiv(frames, 1000 * 60 * 60 * 24);
                    result.hours = BTiming.intDiv(frames, 1000 * 60 * 60) % 24;
                    result.minutes = BTiming.intDiv(frames, 1000 * 60) % 60;
                    result.seconds = BTiming.intDiv(frames, 1000) % 60;
                    result.frames = Math.trunc(frames % 1000);
                }
                break;
        }
        if (fodMode) {
            delete result.days;
        }
        return result;
    }
    static encodeTcString(frames, frameRate, fodMode, withMs, withDays) {
        const comp = BTiming.toComp(frames, frameRate, fodMode);
        let result = "";
        if (withDays && comp.days > 0) {
            result += comp.days + ".";
        }
        else if (!withDays && !fodMode && comp.days > 0) {
            comp.hours += comp.days * 24;
        }
        result += this.pad(comp.hours, 2) + ":" + this.pad(comp.minutes, 2) + ":" + this.pad(comp.seconds, 2);
        if (!withMs) {
            result = (comp.sign || "") + result + frameRate.getSeparator() + this.pad(Math.trunc(comp.frames), 2);
        }
        else {
            const fps = frameRate.getFramesPerSecond();
            const pow = typeof withMs === "number" ? withMs : 3;
            const milliseconds = (frames % fps) / fps;
            const seconds = (frames - (frames % fps)) / fps;
            const tsec = Math.trunc(seconds);
            const decimal = (seconds - tsec) + milliseconds;
            const remainder = decimal * Math.pow(10, pow);
            const display = Math.trunc(remainder);
            result += "." + `${display}`.padStart(pow, '0');
        }
        return result;
    }
    static nonFodEncode(frames, frameRate) {
        let result = {};
        const fps = frameRate.getFramesPerSecond();
        const seconds = frames / fps;
        const tsec = Math.trunc(seconds + 0.0000001);
        result["frames"] = Math.trunc((seconds - tsec) / (1 / fps) + 0.0000001);
        result["hours"] = BTiming.intDiv(tsec, 60 * 60) % 24;
        result["minutes"] = BTiming.intDiv(tsec, 60) % 60;
        result["seconds"] = tsec % 60;
        return result;
    }
    static pad(val, size) {
        var s = String(val);
        while (s.length < (size || 2)) {
            s = "0" + s;
        }
        return s;
    }
    static convertDropToComponents(tcBase, frames, result) {
        frames = Math.floor(frames);
        frames = BTiming.convertDropToNonDrop(frames, tcBase);
        const framesInDay = 60 * 60 * 24 * tcBase;
        const framesInHour = 60 * 60 * tcBase;
        const framesInMinute = 60 * tcBase;
        result.days = BTiming.intDiv(frames, framesInDay);
        frames -= result.days * framesInDay;
        result.hours = BTiming.intDiv(frames, framesInHour);
        frames -= result.hours * framesInHour;
        result.minutes = BTiming.intDiv(frames, framesInMinute);
        frames -= result.minutes * framesInMinute;
        result.seconds = BTiming.intDiv(frames, tcBase);
        frames -= result.seconds * tcBase;
        result.frames = Math.round(frames);
        return result;
    }
    static convertDropToNonDrop(frames, tcBase) {
        const toDrop = tcBase == 30 ? 2 : 4;
        const ONE_ND_MINUTE = 60 * tcBase;
        const TEN_ND_MINUTES = 10 * ONE_ND_MINUTE;
        const ONE_DROP_MINUTE = BTiming.convertNonDropToDrop(ONE_ND_MINUTE, tcBase);
        const TEN_DROP_MINUTES = BTiming.convertNonDropToDrop(TEN_ND_MINUTES, tcBase);
        const l1 = toDrop * BTiming.intDiv((frames - toDrop), TEN_DROP_MINUTES);
        let toReturn = frames;
        toReturn += toDrop * BTiming.intDiv((frames - toDrop - l1), ONE_DROP_MINUTE);
        toReturn -= toDrop * BTiming.intDiv(toReturn, (TEN_ND_MINUTES + toDrop));
        return toReturn;
    }
    static convertNonDropToDrop(frames, tcBase) {
        const toDrop = tcBase == 30 ? 2 : 4;
        const ONE_MINUTE = 60 * tcBase;
        const TEN_MINUTES = 10 * ONE_MINUTE;
        let toReturn = frames;
        toReturn -= toDrop * BTiming.intDiv(frames, ONE_MINUTE);
        toReturn += toDrop * BTiming.intDiv(frames, TEN_MINUTES);
        return toReturn;
    }
}
exports.BTiming = BTiming;
