"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlarmSender = exports.OneGoodValueAlarmTrigger = exports.SelfClearingAlarmTrigger = exports.LatchingAlarmTrigger = exports.MinMaxAlarmTrigger = exports.AlarmType = void 0;
const _ = require("lodash");
const Subscriber_1 = require("./Subscriber");
const LoginDetails_1 = require("./LoginDetails");
const Log_1 = require("../logging/Log");
var AlarmType;
(function (AlarmType) {
    AlarmType[AlarmType["MinMax"] = 0] = "MinMax";
    AlarmType[AlarmType["Latching"] = 1] = "Latching";
    AlarmType[AlarmType["OneGoodValue"] = 2] = "OneGoodValue";
    AlarmType[AlarmType["SelfClearing"] = 3] = "SelfClearing";
})(AlarmType || (exports.AlarmType = AlarmType = {}));
class AlarmTrigger {
    constructor(alarmTriggerConfig, alarmTypeInfo) {
        this.name = alarmTriggerConfig.name;
        this.alarmId = alarmTriggerConfig.alarmId;
        this.description = alarmTriggerConfig.description;
        this.enabled = alarmTriggerConfig.enabled;
        this.channel = this.getChannel(alarmTriggerConfig);
        this.alarmTypeInfo = alarmTypeInfo;
    }
    getChannel(alarmTriggerConfig) {
        if (alarmTriggerConfig.address && alarmTriggerConfig.oid) {
            return alarmTriggerConfig.address + "/" + alarmTriggerConfig.oid;
        }
        else {
            return alarmTriggerConfig.lane || alarmTriggerConfig.address || alarmTriggerConfig.channel || alarmTriggerConfig.channelName;
        }
    }
    getLabel(value, valueLabel) {
        if (valueLabel) {
            return this.description.replace(/\?/, valueLabel);
        }
        else {
            return this.description.replace(/\?/, value);
        }
    }
}
class StatefulAlarmTrigger extends AlarmTrigger {
    constructor(alarmTriggerConfig, alarmTypeInfo) {
        super(alarmTriggerConfig, alarmTypeInfo);
        this.state = false;
    }
    setAlarm(label) {
        if (!this.state) {
            this.state = true;
            this.label = label;
            AlarmSender.directSetAlarm(this.alarmId, label);
        }
    }
    clearAlarm() {
        if (this.state) {
            this.state = false;
            AlarmSender.directClearAlarm(this.alarmId);
        }
    }
    retriggerAlarm() {
        if (this.state) {
            AlarmSender.directSetAlarm(this.alarmId, this.label);
        }
    }
}
class MinMaxAlarmTrigger extends StatefulAlarmTrigger {
    constructor(alarmTriggerConfig, alarmTypeInfo) {
        super(alarmTriggerConfig, alarmTypeInfo);
        this.min = alarmTriggerConfig.min;
        this.max = alarmTriggerConfig.max;
    }
    setValue(value, options) {
        if (value > this.max) {
            this.setAlarm(this.getLabel(value, options.label));
        }
        else if (value < this.min) {
            this.setAlarm(this.getLabel(value, options.label));
        }
        else {
            this.clearAlarm();
        }
    }
}
exports.MinMaxAlarmTrigger = MinMaxAlarmTrigger;
class LatchingAlarmTrigger extends AlarmTrigger {
    constructor(alarmTriggerConfig, alarmTypeInfo) {
        super(alarmTriggerConfig, alarmTypeInfo);
    }
    setValue(value, options) {
        AlarmSender.directSetAlarm(this.alarmId, this.getLabel(value, options.label));
    }
    retriggerAlarm() {
        // Don't retrigger on a latching alarm
    }
}
exports.LatchingAlarmTrigger = LatchingAlarmTrigger;
class SelfClearingAlarmTrigger extends StatefulAlarmTrigger {
    setValue(value, options) {
        if (value) {
            this.setAlarm(this.getLabel(value, options && options.label));
        }
        else {
            this.clearAlarm();
        }
    }
}
exports.SelfClearingAlarmTrigger = SelfClearingAlarmTrigger;
class OneGoodValueAlarmTrigger extends StatefulAlarmTrigger {
    constructor(alarmTriggerConfig, alarmTypeInfo) {
        super(alarmTriggerConfig, alarmTypeInfo);
        this.goodValue = alarmTypeInfo.goodValue;
    }
    setValue(value, options) {
        if (value !== this.goodValue) {
            this.setAlarm(this.getLabel(value, options.label));
        }
        else {
            this.clearAlarm();
        }
    }
}
exports.OneGoodValueAlarmTrigger = OneGoodValueAlarmTrigger;
class AlarmSender {
    static setupAlarmSubscriber(alarmService = "alarmService", loginDetails) {
        if (this.alarmSubscriber == null) {
            Log_1.Log.logInfo("Creating alarm service subscriber");
            this.alarmSubscriber = new Subscriber_1.Subscriber(alarmService, loginDetails, null, {
                onConnected: AlarmSender.alarmSubscriberConnected,
                onDisconnected: AlarmSender.alarmSubscriberDisconnected
            });
        }
    }
    //static alarmSubscriber: Subscriber;
    static setup(appId, loginDetails, alarmTriggerConfig, alarmTypes, alarmService = "alarmService") {
        this.appId = appId;
        this.loginDetails = loginDetails;
        // this.alarmSubscriber = new Subscriber(alarmService, loginDetails, null, {
        //     onConnected: AlarmSender.alarmSubscriberConnected,
        //     onDisconnected: AlarmSender.alarmSubscriberDisconnected
        // });
        this.setupAlarmSubscriber(alarmService, loginDetails);
        var alarmTriggers = _.map(alarmTriggerConfig, (config) => {
            var alarmTrigger = null;
            var alarmTypeInfo = alarmTypes[config.name];
            if (alarmTypeInfo) {
                switch (alarmTypeInfo.type) {
                    case AlarmType.MinMax:
                        {
                            alarmTrigger = new MinMaxAlarmTrigger(config, alarmTypeInfo);
                        }
                        break;
                    case AlarmType.Latching:
                        {
                            alarmTrigger = new LatchingAlarmTrigger(config, alarmTypeInfo);
                        }
                        break;
                    case AlarmType.OneGoodValue:
                        {
                            alarmTrigger = new OneGoodValueAlarmTrigger(config, alarmTypeInfo);
                        }
                        break;
                    case AlarmType.SelfClearing:
                        {
                            alarmTrigger = new SelfClearingAlarmTrigger(config, alarmTypeInfo);
                        }
                        break;
                }
            }
            return alarmTrigger;
        });
        this.alarmTriggers = _.filter(alarmTriggers, (alarmTrigger) => {
            return alarmTrigger !== null;
        });
        // Clear all alarms on startup
        this.alarmSubscriber.connect(alarmService)
            .then(() => {
            Log_1.Log.logInfo("Connected to alarm service");
            _(this.alarmTriggers)
                .map((alarmTrigger) => {
                return alarmTrigger.alarmId;
            })
                .uniq()
                .each((alarmId) => {
                this.directClearAlarm(alarmId);
            });
            this.isSetup = true;
        })
            .catch((err) => {
            Log_1.Log.logError("Unable to connect to alarm service: ", err);
        });
    }
    static alarmSubscriberConnected() {
        Log_1.Log.logInfo("New alarm service activated ");
        // A new alarm service has become active send out our known state
        _.each(AlarmSender.alarmTriggers, (alarmTrigger) => {
            alarmTrigger.retriggerAlarm();
        });
    }
    static alarmSubscriberDisconnected() {
        Log_1.Log.logInfo("New alarm service deactivated ");
    }
    static directSetAlarm(alarmId, description) {
        if (this.isSetup) {
            Log_1.Log.logTrace(`Set alarm ${alarmId} ${description}`);
            return AlarmSender.alarmSubscriber.sendControl("triggerAlarm", {
                "id": alarmId,
                "state": true,
                "description": description
            });
        }
        else {
            //try once to setup
            Log_1.Log.logDebug("Alarm sender not set up yet, try to set up");
            AlarmSender.setup(LoginDetails_1.LoginDetails.get().getProviderId(), LoginDetails_1.LoginDetails.get(), null, null);
            return AlarmSender.alarmSubscriber.sendControl("triggerAlarm", {
                "id": alarmId,
                "state": true,
                "description": description
            });
        }
    }
    static directClearAlarm(alarmId) {
        if (this.isSetup) {
            Log_1.Log.logTrace(`Clear alarm ${alarmId}`);
            return AlarmSender.alarmSubscriber.sendControl("triggerAlarm", {
                "id": alarmId,
                "state": false
            });
        }
        else {
            Log_1.Log.logDebug("Alarm sender not set up yet, try to set up");
            AlarmSender.setup(LoginDetails_1.LoginDetails.get().getProviderId(), LoginDetails_1.LoginDetails.get(), null, null);
            return AlarmSender.alarmSubscriber.sendControl("triggerAlarm", {
                "id": alarmId,
                "state": false
            });
        }
    }
    static getAlarmTrigger(name, valueOptions) {
        var channel = null;
        if (valueOptions) {
            if (valueOptions.channel) {
                channel = valueOptions.channel;
            }
        }
        if (channel) {
            return _.find(this.alarmTriggers, (alarmTrigger) => {
                return (alarmTrigger.name === name) && ("" + channel === "" + alarmTrigger.channel);
            });
        }
        else {
            return _.find(this.alarmTriggers, (alarmTrigger) => {
                return alarmTrigger.name === name;
            });
        }
    }
    /**
     * Send or clear an alarm by sending the new alarm data through to the alarmType
     * @param name The name of the alarm trigger
     * @param value The value to set the alarm to. Based on the alarm type this will set or clear alarms appropriately.
     * @param valueOptions Options to trigger and display alarms in specific ways
     */
    static setValue(alarmTriggerName, value, valueOptions) {
        if (this.isSetup) {
            var alarmTrigger = AlarmSender.getAlarmTrigger(alarmTriggerName, valueOptions);
            if (alarmTrigger) {
                if (!valueOptions) {
                    valueOptions = {};
                }
                alarmTrigger.setValue(value, valueOptions);
            }
        }
    }
}
exports.AlarmSender = AlarmSender;
AlarmSender.isSetup = false;
AlarmSender.alarmSubscriber = null;
