"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoQcProvider = void 0;
const path = require("path");
const _ = require("lodash");
const Provider_1 = require("./Provider");
const EsbMessage_1 = require("./EsbMessage");
const BusbyLocationLookupSubscriber_1 = require("../subscribers/BusbyLocationLookupSubscriber");
class AutoQcProvider {
    constructor(providerId, loginDetails, driver) {
        this.providerId = providerId;
        this.loginDetails = loginDetails;
        this.driver = driver;
        this.provider = new Provider_1.Provider(providerId, loginDetails, this.controlProcessor.bind(this));
        this.locationService = new BusbyLocationLookupSubscriber_1.BusbyLocationLookupSubscriber(loginDetails);
    }
    setConfigChangedUpdater(updater) {
        this.provider.setConfigChangedUpdater(updater);
    }
    addCliCommandProcessor(command, helpText, processor) {
        this.provider.addCliCommandProcessor(command, helpText, processor);
    }
    addCliCommand(command) {
        this.provider.addCliCommand(command);
    }
    async init() {
        await this.locationService.waitUntilConnected();
        await this.provider.connect();
        await this.provider.waitUntilActive();
        this.addCliCommandProcessor("profiles", "Get all profiles", this.getProfiles.bind(this));
        this.addCliCommandProcessor("check", "QC a file, -p profileId -s sourceFile", this.check.bind(this));
    }
    async getProfiles() {
        try {
            const profiles = await this.driver.getProfiles();
            return _.map(profiles.profiles, (profile) => {
                if (profile.description) {
                    return profile.name + " : " + profile.description;
                }
                else {
                    return profile.name;
                }
            });
        }
        catch (err) {
            return [
                `Failed getting profiles`
            ];
        }
    }
    async check(params) {
        if (!params.hasParam("p")) {
            return [
                "No profile id provided"
            ];
        }
        if (!params.hasParam("s")) {
            return [
                "No source file provided"
            ];
        }
        const profileId = params.getStringParam("p");
        const sourceFile = params.getStringParam("s");
        const autoQcJob = await this.driver.check(profileId, sourceFile);
        return ["Auto QC ID: " + autoQcJob.autoQcId];
    }
    async getConfig() {
        return await this.provider.getConfig();
    }
    replaceFilenameExtension(filename, extension) {
        if (!extension) {
            return filename;
        }
        else {
            return filename.replace(/[^.]+$/, extension);
        }
    }
    async controlProcessor(incoming) {
        const params = incoming.getParams();
        try {
            switch (incoming.getMessageName()) {
                case "getProfiles": {
                    return await EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, await this.driver.getProfiles());
                }
                case "check": {
                    const profile = incoming.getParams().profile;
                    const source = incoming.getParams().source;
                    const options = incoming.getParams().options;
                    let sourceFile;
                    if (_.isString(source)) {
                        sourceFile = source;
                    }
                    else {
                        let sourceLocation;
                        if (source.location) {
                            sourceLocation = (await this.locationService.getLocation({ locationName: source.location })).getResult();
                        }
                        sourceFile = path.join(sourceLocation ? sourceLocation.mount : '', source.path, source.filename);
                    }
                    try {
                        const autoQcJob = await this.driver.check(profile, sourceFile, options);
                        if (this.driver.getPreviewUrl) {
                            autoQcJob.previewUrl = this.driver.getPreviewUrl(autoQcJob);
                        }
                        return await EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, autoQcJob);
                    }
                    catch (e) {
                        return await EsbMessage_1.EsbMessage.makeControlReplyNak(this.providerId, incoming, e.message);
                    }
                }
                case "getAutoQcInfo": {
                    const autoQcId = incoming.getParams().autoQcId;
                    const autoQcInfo = await this.driver.getAutoQcInfo(autoQcId);
                    return await EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, autoQcInfo);
                }
                case "getReport": {
                    const autoQcId = incoming.getParams().autoQcId;
                    const options = incoming.getParams().options;
                    return await EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, await this.driver.getReport(autoQcId, options));
                }
                default: {
                    return Promise.resolve(EsbMessage_1.EsbMessage.makeControlReplyNak(this.providerId, incoming, 'message name not recognised'));
                }
            }
        }
        catch (e) {
            return Promise.resolve(EsbMessage_1.EsbMessage.makeControlReplyNak(this.providerId, incoming, e.message));
        }
    }
}
exports.AutoQcProvider = AutoQcProvider;
