"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CliParser = void 0;
const CliParams_1 = require("./CliParams");
const Log_1 = require("../../logging/Log");
const Commands_1 = require("./Commands");
const CliCommandDescription_1 = require("./CliCommandDescription");
const Params_1 = require("./Params");
class CliParser {
    static get() {
        if (!this.parser) {
            this.parser = new CliParser();
        }
        return this.parser;
    }
    constructor() {
        this.commands = {};
        this.commandProcessors = {};
        this.makeBasicCommands();
    }
    makeBasicCommands() {
        this.addCommand(new Commands_1.HelpCommand(this.commands, this.commandProcessors));
        this.addCommand(new Commands_1.DescriptionCommand(this.commands, this.commandProcessors));
        this.addCommand(new Commands_1.MemoryStatsCommand());
        this.addCommand(new Commands_1.TimeCommand());
        this.addCommand(new Commands_1.ConfigCommand());
        this.addCommand(new Commands_1.SubscribersCommand());
        this.addCommand(new Commands_1.DebugCommand());
        this.addCommand(new Commands_1.LogLevelCommand());
    }
    addCommand(command) {
        Log_1.Log.getNamespace('cli:add')(command.getCommandDescription().getCommand(), command.getCommandDescription().getDescription());
        this.commands[command.getCommandDescription().getCommand().toLowerCase()] = command;
    }
    addCommandProcessor(arg0, arg1, arg2 = () => [], autoCompleteProcessor = () => []) {
        const command = typeof arg0 === "string" ? arg0 : arg0.getCommand();
        const description = typeof arg1 === "string" ? arg1 : arg0.getDescription();
        Log_1.Log.getNamespace('cli:parser:addProcessor')(command, description);
        const commandDescription = typeof arg0 === "string" ? CliCommandDescription_1.CliCommandDescription.make(command, description) : arg0;
        this.commandProcessors[command] = {
            description: commandDescription,
            processCommand: typeof arg1 === "string" ? arg2 : arg1,
            autoComplete: typeof arg1 === "string" ? autoCompleteProcessor : arg2
        };
    }
    async processCommand(command, provider) {
        const debug = Log_1.Log.getNamespace('cli:command');
        let result = [];
        const params = new CliParams_1.CliParams();
        const args = command.split(/\s+/);
        const cmd = args[0];
        if (!this.commands[cmd] && !this.commandProcessors[cmd]) {
            result.push(`command ${cmd} not available use ? to find available commands`);
        }
        else {
            for (let i = 0; i < args.length; i++) {
                const arg = args[i];
                if (arg.startsWith("-") && isNaN(Number.parseInt(arg))) {
                    const param = [];
                    while (args[i + 1] && (!args[i + 1].startsWith("-") || !isNaN(Number.parseInt(args[i + 1])))) {
                        param.push(args[++i]);
                    }
                    debug(cmd, param.length, param);
                    params.addParam(arg, param.join(" "));
                }
            }
            try {
                const process = this.commands[cmd] ?? this.commandProcessors[cmd];
                result = await process.processCommand(params, provider);
            }
            catch (e) {
                Log_1.Log.logError(`Error Processing Cli Command ${command}`, e);
                result.push(`error running ${cmd}`);
                Log_1.Log.logException(e);
            }
        }
        return result;
    }
    async processAutoComplete(command, param, soFar, args, provider) {
        const debug = Log_1.Log.getNamespace('cli:autocomplete');
        let result = [];
        const cmd = command;
        if (!this.commands[cmd] && !this.commandProcessors[cmd]) {
            result.push(`command ${cmd} not available use ? to find available commands`);
        }
        else {
            try {
                const description = this.commands[cmd]?.getCommandDescription() ?? this.commandProcessors[cmd].description;
                const parameter = description.getParam(param);
                if (parameter instanceof Params_1.CliAutoCompleteParam) {
                    result = await parameter.getAutoComplete()(soFar, args, provider);
                }
                else {
                    result = await parameter.getOptions();
                }
            }
            catch (e) {
                Log_1.Log.logError(`Error Processing Cli Auto Complete ${command}`, e);
                result.push(`error autocomplete ${cmd}`);
                Log_1.Log.logException(e);
            }
        }
        return result;
    }
    async processEsbCommand(msg, provider) {
        let result = [];
        try {
            result = await this.processCommand(msg.getParams().commandLine ?? "", provider);
        }
        catch (e) {
        }
        result.push(">");
        return { result };
    }
    async processEsbAutoComplete(msg, provider) {
        let result = [];
        try {
            result = await this.processAutoComplete(msg.getParams().command, msg.getParams().param, msg.getParams().soFar, msg.getParams().args, provider);
        }
        catch (e) {
        }
        return { result };
    }
}
exports.CliParser = CliParser;
