"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EsbTypedMessage = exports.EsbMessage = void 0;
const luxon_1 = require("luxon");
const model_1 = require("./model");
class EsbMessage {
    static makeStatusMessage(from, messageName, params) {
        const d = new Date();
        return new EsbMessage(from, model_1.MessageType.status, d, EsbMessage.nextSeq++, messageName, params);
    }
    static makeStatusMessageResend(from, messageName, params) {
        const d = new Date();
        return new EsbMessage(from, model_1.MessageType.status, d, EsbMessage.nextSeq++, messageName, params, true);
    }
    static makeControlMessage(from, messageName, params) {
        const d = new Date();
        return new EsbMessage(from, model_1.MessageType.control, d, EsbMessage.nextSeq++, messageName, params);
    }
    static makeControlReply(from, incoming, params) {
        const d = new Date();
        return new EsbMessage(from, model_1.MessageType.controlReply, d, incoming.getSequenceNumber(), incoming.getMessageName(), params);
    }
    static makeControlReplyAck(from, incoming) {
        return EsbMessage.makeControlReply(from, incoming, { "status": "ok" });
    }
    static makeControlReplyNak(from, incoming, reason) {
        return EsbMessage.makeControlReply(from, incoming, { "status": "error", "reason": reason });
    }
    static makeFromJsonString(st) {
        const bao = JSON.parse(st);
        return EsbMessage.makeFromJson(bao);
    }
    static makeFromJson(bao) {
        const mt = model_1.MessageType[bao.messageType];
        const sn = parseInt(bao.sequenceNumber);
        return new EsbMessage(bao.from, mt, new Date(bao.timestamp), sn, bao.messageName, bao.params);
    }
    static makeSystemMessage(from, messageName, params) {
        const d = new Date();
        return new EsbMessage(from, model_1.MessageType.system, d, EsbMessage.nextSeq++, messageName, params);
    }
    static makeUpdateMessage(from, messageName, params) {
        const d = new Date();
        return new EsbMessage(from, model_1.MessageType.update, d, EsbMessage.nextSeq++, messageName, params);
    }
    static makeSystemReply(from, incoming, params) {
        const d = new Date();
        return new EsbMessage(from, model_1.MessageType.systemReply, d, incoming.getSequenceNumber(), incoming.getMessageName(), params);
    }
    static makeSystemReplyNak(from, incoming, reason) {
        return EsbMessage.makeSystemReply(from, incoming, { status: "error", reason: reason });
    }
    static makeSystemReplyAck(from, incoming) {
        return EsbMessage.makeSystemReply(from, incoming, { status: "ok" });
    }
    constructor(from, messageType, timestamp, sequenceNumber, messageName, params, resend = false) {
        this.from = from;
        this.messageType = messageType;
        this.timestamp = timestamp;
        this.sequenceNumber = sequenceNumber;
        this.messageName = messageName;
        this.params = params;
        this.resend = resend;
        this.toService = "";
    }
    setToService(toService) {
        this.toService = toService;
    }
    getFrom() {
        return this.from;
    }
    getMessageType() {
        return this.messageType;
    }
    getTimeStamp() {
        return this.timestamp;
    }
    getDateTime() {
        return luxon_1.DateTime.fromJSDate(this.timestamp, { zone: "utc" });
    }
    getSequenceNumber() {
        return this.sequenceNumber;
    }
    // You should very rarely need to set the sequence number manually, only if routing messages in an unusual way.
    setSequenceNumber(sequenceNumber) {
        this.sequenceNumber = sequenceNumber;
    }
    getMessageName() {
        return this.messageName;
    }
    getParams() {
        return this.params;
    }
    setParams(params) {
        this.params = params;
    }
    getErrorParams() {
        return this.params;
    }
    updateParams(params) {
        this.params = params;
    }
    getAsJson() {
        return {
            from: this.from,
            messageType: model_1.MessageType[this.messageType],
            timestamp: this.timestamp.toISOString(),
            sequenceNumber: this.sequenceNumber,
            messageName: this.messageName,
            params: this.params,
            resend: this.resend,
            toService: this.toService
        };
    }
    getBigDataLogMessage(providerId) {
        return {
            providerId: providerId,
            from: this.from,
            messageType: model_1.MessageType[this.messageType],
            timestamp: this.timestamp.toISOString(),
            sequenceNumber: this.sequenceNumber,
            messageName: this.messageName
        };
    }
    getAsJsonString() {
        return JSON.stringify(this.getAsJson());
    }
    getStatus() {
        if (this.params?.status) {
            return this.params.status;
        }
        else {
            return "ok";
        }
    }
    isError() {
        if (this.params?.status) {
            return this.params.status === "error";
        }
        else {
            return false;
        }
    }
    toString() {
        return "from=" + this.from + "\n" +
            "timestamp=" + this.timestamp.toISOString() + "\n" +
            "messageType=" + model_1.MessageType[this.messageType] + "\n" +
            "sequence=" + this.sequenceNumber + "\n" +
            "messageName=" + this.messageName + "\n" +
            "params=" + JSON.stringify(this.getParams());
    }
}
exports.EsbMessage = EsbMessage;
EsbMessage.nextSeq = 0;
class EsbTypedMessage extends EsbMessage {
    /**
     * Get the parameters of the message.
     * Specify the message name generic to get the correct type.
     * @type {MN} specify the message name
     */
    getParams() {
        return super.getParams();
    }
}
exports.EsbTypedMessage = EsbTypedMessage;
