"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RabbitConnection = void 0;
const amqplib = require("amqplib");
const _ = require("lodash");
const uuid = require("uuid");
const EsbMessage_1 = require("./EsbMessage");
const Configuration_1 = require("./Configuration");
const Log_1 = require("../logging/Log");
const ProviderState_1 = require("./ProviderState");
const PromiseTimeout_1 = require("../Utilities/PromiseTimeout");
const STATUS_EXCHANGE_SUFFIX = '_statusEx';
const CONTROL_EXCHANGE_SUFFIX = '_controlEx';
const CONTROL_QUEUE_SUFFIX = '_controlQ';
const STATUS_QUEUE_SUFFIX = '_statusQ';
const REPLY_QUEUE_SUFFIX = '_replyQ';
const DIRECTORY_REPLY_QUEUE_SUFFIX = '_dirReplyQ';
const DIRECTORY_UPDATE_QUEUE_SUFFIX = '_dirUpdateQ';
class RabbitConnection {
    constructor(loginDetails, providerMode) {
        this.loginDetails = loginDetails;
        this.providerMode = providerMode;
        this.dirServerConn = 'dirServerConn'; //control connection to directory service
        this.providerServerConn = 'providerConn';
        this.subscriberServerConn = 'subscriberConn';
        this.dirExchange = 'directoryEx';
        this.updateExchange = 'updateExTopic';
        this.qDurable = false;
        this.qExclusive = true;
        this.qAutoDelete = true;
        this.xDurable = false;
        this.deliveryMode = 1; // 1 - don't persist messages, 2 - persist messages
        this.consumers = {};
        this.statusChanged = true;
        this.connection = null;
        this.channels = {};
        this.processors = {};
        this.providerChanged = false;
        this.dirUpdateQueueName = null;
        this.defaultShortTimeout = 5;
        this.busbyEsbProtocolVersion = 1;
        this.statusRoutingKeys = new Set();
        this.getServiceProviderErrors = 0;
    }
    setSubId(subId) {
        this.subscriberId = subId;
    }
    getDirReplyQueueName(id) {
        return id + DIRECTORY_REPLY_QUEUE_SUFFIX;
    }
    getDirUpdateQueueName(id) {
        if (!this.dirUpdateQueueName) {
            this.dirUpdateQueueName = id + DIRECTORY_UPDATE_QUEUE_SUFFIX;
        }
        return this.dirUpdateQueueName;
    }
    getProviderControlExchangeName() {
        return this.providerId + CONTROL_EXCHANGE_SUFFIX;
    }
    getProviderStatusExchangeName() {
        return this.providerId + STATUS_EXCHANGE_SUFFIX;
    }
    getOldProviderStatusExchangeName(oldProviderId) {
        return oldProviderId + STATUS_EXCHANGE_SUFFIX;
    }
    getProviderControlQueueName() {
        return this.providerId + CONTROL_QUEUE_SUFFIX;
    }
    getSubscriberStatusQueueName() {
        return `${this.subscriberId}_${this.providerId}${STATUS_QUEUE_SUFFIX}`;
    }
    getSubscriberReplyQueueName() {
        return `${this.subscriberId}_${this.providerId}${REPLY_QUEUE_SUFFIX}`;
    }
    getProviderId() {
        return this.providerId;
    }
    setConfigChangedUpdater(configChanged) {
        this.configChangedUpdater = configChanged;
    }
    listenDirectoryChanged(directoryChanged) {
        this.onDirectoryChanged = directoryChanged;
    }
    getStatusChanged() {
        const result = this.statusChanged;
        this.statusChanged = false;
        return result;
    }
    setDirectoryStatusProcessor(dsp) {
        this.directoryStatusProcessor = dsp;
    }
    setChannelConnecting(channelName, connecting) {
        if (!this.channels[channelName]) {
            this.channels[channelName] = {
                channel: null,
                connecting: connecting,
                setup: false,
                resolvers: []
            };
        }
        else {
            this.channels[channelName].connecting = connecting;
        }
    }
    isSubscriberConnected() {
        return this.isConnected("subscriberChannel") && this.isConnected("directoryChannel");
    }
    isConnected(channelName) {
        return _.includes(_.keys(this.channels), channelName);
    }
    async disconnect(source, connection = false) {
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:disconnect`)("disconnecting", source, connection);
        try {
            if (this.subscriberId) {
                if (connection) {
                    this.connection?.close?.().catch(err => { });
                    this.channels = {};
                    this.connection = null;
                    this.onSubscriberActive(false, "disconnect connection");
                    if (this.subscriberOptions?.onDisconnected) {
                        this.subscriberOptions.onDisconnected();
                    }
                }
                else {
                    const currentChannel = this.channels["subscriberChannel"];
                    delete this.channels["subscriberChannel"];
                    currentChannel?.channel?.close();
                }
            }
            else {
                if (this.connection != null) {
                    this.connection?.close().catch(err => { });
                }
                this.channels = {};
                this.connection = null;
                this.onSubscriberActive?.(false, "disconnect connection");
            }
        }
        catch (err) {
            Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:disconnect`)("error in disconnection", source, connection);
        }
    }
    async makeConnection() {
        if (this.connection == null) {
            const currentHost = this.loginDetails.getHost();
            try {
                const clientProperties = { busby: { providerId: this.loginDetails.getProviderId(), provider: this.providerMode, id: this.providerMode ? this.providerId : this.subscriberId } };
                const amqpUrl = `amqp://${this.loginDetails.getUsername()}:${this.loginDetails.getPassword()}@${currentHost}:5672` + '/%2Fsquared?heartbeat=60';
                this.connection = await amqplib.connect(amqpUrl, { clientProperties });
                this.connection.on("close", async () => {
                    Log_1.Log.logError(`connection closed ${this.providerMode ? this.providerId : this.subscriberId}`);
                    await this.disconnect("connection close", true);
                    this.connection = null;
                });
                this.connection.on("error", async (err) => {
                    Log_1.Log.logError(`connection errored ${this.providerMode ? this.providerId : this.subscriberId} ${err}`);
                    await this.disconnect("connection errored", true);
                    this.connection = null;
                    if (this.providerMode) {
                        this.loginDetails.nextHost();
                        const newHost = this.loginDetails.getHost();
                        if (currentHost !== newHost) {
                            Log_1.Log.logWarn(`core host changed because of error ${currentHost} -> ${newHost}`);
                        }
                    }
                });
                Log_1.Log.logInfo(`${this.providerMode ? 'provider ' + this.providerId : 'subscriber ' + this.subscriberId} connected to core ${currentHost} ${this.connection?.connection?.serverProperties?.cluster_name}`);
                Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:makeConnection`)(currentHost, _.omit(this.connection?.connection?.serverProperties, "capabilities"));
            }
            catch (err) {
                Log_1.Log.logError(`connection error ${this.providerMode ? this.providerId : this.subscriberId} ${err}`);
                await this.disconnect("connection exception", true);
                this.connection = null;
                if (this.providerMode) {
                    this.loginDetails.nextHost();
                    const newHost = this.loginDetails.getHost();
                    if (currentHost !== newHost) {
                        Log_1.Log.logWarn(`core host changed ${currentHost} -> ${newHost}`);
                    }
                }
            }
        }
        return this.connection;
    }
    async connect(name, channelName) {
        let channel = null;
        try {
            const connection = await this.makeConnection();
            if (connection != null) {
                channel = await connection.createChannel();
                channel.on('close', async () => {
                    Log_1.Log.logInfo(`channel closed ${name} ${channelName} ${this.providerMode ? this.providerId : this.subscriberId}`);
                    if (!this.providerChanged || channelName === "directoryChannel") {
                        await this.disconnect("channel close");
                    }
                });
                channel.on('error', async (err) => {
                    Log_1.Log.logError(`channel errored ${name} ${channelName} ${this.providerMode ? this.providerId : this.subscriberId}`, err);
                    await this.disconnect("channel error", true);
                });
            }
            else {
                Log_1.Log.logError("cannot connect channel " + channelName + " no connection");
            }
        }
        catch (err) {
            Log_1.Log.logError(`Unable to setup connection`);
        }
        return channel;
    }
    async doRequest(exchangeName, channel, msg, msgType, replyQueue, consumer, timeoutSecs) {
        const messId = uuid.v4();
        let result = null;
        try {
            if (Log_1.Log.getLogLevel(Log_1.LogLevel.ESB)) {
                Log_1.Log.logESB('outgoing message [' + (this.subscriberId ? this.subscriberId : this.providerId) + "]: " + messId + ", " + msg.getMessageName() + ", " + JSON.stringify(msg.getParams()));
            }
            const promiseResolution = {};
            const promise = new Promise((resolve, reject) => {
                promiseResolution.resolve = resolve;
                promiseResolution.reject = reject;
            });
            _.set(this.consumers, [consumer, messId], promiseResolution);
            const res = PromiseTimeout_1.PromiseTimeout.promiseTimeout(timeoutSecs * 1000, promise);
            await channel?.publish(exchangeName, msgType, Buffer.from(msg.getAsJsonString()), {
                messageId: messId,
                replyTo: replyQueue,
                type: msgType,
                contentType: 'application/json',
                deliveryMode: this.deliveryMode,
                contentEncoding: 'utf8',
                correlationId: messId
            });
            result = await res.catch(err => { throw err; });
        }
        catch (err) {
            Log_1.Log.logError('Failed to do amqp request exchangeName=' + exchangeName + ' messageName=' + msg.getMessageName() + ' error=' + err);
        }
        if (_.isUndefined(result) || _.isNull(result)) {
            return Promise.reject("No reply to request: " + msg.getAsJsonString());
        }
        return result;
    }
    async doPublish(exchangeName, channel, msg, msgType) {
        const messId = uuid.v4();
        await channel.publish(exchangeName, msgType, Buffer.from(msg.getAsJsonString()), {
            messageId: messId,
            replyTo: '',
            type: msgType,
            contentType: 'application/json',
            deliveryMode: this.deliveryMode,
            contentEncoding: 'utf8'
        });
    }
    makeConsumer(consumerType, messageProcessor) {
        return (channel, replyQueue) => {
            Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:makeConsumer:${consumerType}`)(replyQueue);
            return channel.consume(replyQueue, async (msg) => {
                let content;
                try {
                    content = EsbMessage_1.EsbMessage.makeFromJsonString(msg.content.toString());
                }
                catch (e) {
                    Log_1.Log.logError(`Failed to parse message routing=${msg?.fields?.routingKey} exc=${msg?.fields?.exchange} messId=${msg?.properties?.messageId} mess=${msg.content.toString()}`, e);
                    throw e;
                }
                if (Log_1.Log.getLogLevel(Log_1.LogLevel.ESB)) {
                    Log_1.Log.logESB('incoming message [' + (this.subscriberId ? this.subscriberId : this.providerId) + "]: " + "messId=" + msg.properties.messageId + " mess=" + JSON.stringify(content));
                }
                if (messageProcessor) {
                    const esbMessage = await messageProcessor(content, this.serviceId);
                    if (esbMessage) {
                        try {
                            if (esbMessage && esbMessage.getAsJsonString) {
                                const buffer = Buffer.from(esbMessage.getAsJsonString());
                                channel.publish('', msg.properties.replyTo, buffer, {
                                    messageId: msg.properties.messageId,
                                    type: 'control.reply',
                                    contentType: 'application/json',
                                    correlationId: msg.properties.messageId,
                                    deliveryMode: this.deliveryMode,
                                    contentEncoding: 'utf8'
                                });
                            }
                        }
                        catch (err) {
                            return Log_1.Log.logError("Error replying to message " + err, err);
                        }
                    }
                }
                else if (_.has(this.consumers[consumerType], msg.properties.correlationId)) {
                    try {
                        this.consumers[consumerType][msg.properties.correlationId].resolve(content);
                    }
                    catch (err) {
                        Log_1.Log.logError(err);
                        this.consumers[consumerType].messId.reject(err);
                    }
                    delete this.consumers[consumerType][msg.properties.correlationId];
                }
            }, {
                noAck: true
            });
        };
    }
    async setupDirectoryConnectionRun(id) {
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupDirectoryConnection`)(id);
        try {
            const channel = await this.connect(this.dirServerConn, 'directoryChannel');
            if (channel) {
                await channel.assertExchange(this.dirExchange, 'fanout', { durable: this.xDurable, autoDelete: false });
                await channel.assertExchange(this.updateExchange, 'topic', { durable: this.xDurable, autoDelete: false });
                await channel.assertQueue(this.getDirUpdateQueueName(id), {
                    durable: this.qDurable,
                    autoDelete: this.qAutoDelete,
                    maxLength: 1000,
                    exclusive: this.qExclusive
                });
                await channel.assertQueue(this.getDirReplyQueueName(id), {
                    durable: this.qDurable,
                    autoDelete: this.qAutoDelete,
                    maxLength: 1000,
                    exclusive: this.qExclusive
                });
                Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupDirectoryConnection:bind`)(this.getDirUpdateQueueName(id), this.updateExchange, this.subscriberOptions?.promiscuous ? "#" : `directoryUpdate.*.${this.serviceId || '*'}`);
                await channel.bindQueue(this.getDirUpdateQueueName(id), this.updateExchange, this.subscriberOptions?.promiscuous ? "#" : `directoryUpdate.*.${this.serviceId || '*'}`);
                Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupDirectoryConnection:bind`)(this.getDirUpdateQueueName(id), this.updateExchange, `configPublished.#`);
                await channel.bindQueue(this.getDirUpdateQueueName(id), this.updateExchange, `configPublished.#`);
                await this.makeConsumer('directory')(channel, this.getDirReplyQueueName(id));
                await this.makeConsumer('directoryStatus', this.processors['directory'])(channel, this.getDirUpdateQueueName(id));
                this.channels['directoryChannel'] ??= {};
                this.channels['directoryChannel'].channel = channel;
                Log_1.Log.logESB("Channel setup directoryChannel");
            }
            else {
                Log_1.Log.logError("cannot setup directoryChannel, no connection");
            }
        }
        catch (err) {
            Log_1.Log.logError(`Failed setting up directoryChannel ${this.providerMode ? this.providerId : this.subscriberId} - ${err}`);
        }
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupDirectoryConnection`)(!!this.channels['directoryChannel']);
        this.setupDirectoryConnectionPromise = null;
        return this.channels['directoryChannel']?.channel;
    }
    async setupDirectoryConnection(id, directoryProcessor) {
        if (directoryProcessor) {
            this.processors['directory'] = directoryProcessor;
        }
        if (this.channels['directoryChannel']) {
            return this.channels['directoryChannel']?.channel;
        }
        else {
            if (!this.setupDirectoryConnectionPromise) {
                this.setupDirectoryConnectionPromise = this.setupDirectoryConnectionRun(id);
            }
            return await this.setupDirectoryConnectionPromise;
        }
    }
    async setupProviderConnectionRun() {
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupProviderConnection`)("new");
        try {
            let channel = await this.connect(this.providerServerConn, 'providerChannel');
            if (channel != null) {
                this.busbyEsbProtocolVersion = await this.getProtocolVersion(this.providerId);
                let fails = false;
                try {
                    await channel.assertExchange(this.getProviderStatusExchangeName(), this.busbyEsbProtocolVersion > 1 ? 'topic' : 'fanout', {
                        durable: this.xDurable,
                        autoDelete: false
                    });
                }
                catch (err) {
                    if (this.busbyEsbProtocolVersion > 1) {
                        fails = true;
                    }
                    else {
                        throw err;
                    }
                }
                if (fails) {
                    channel = await this.connect(this.providerServerConn, 'providerChannel');
                    await channel.deleteExchange(this.getProviderStatusExchangeName());
                    await channel.assertExchange(this.getProviderStatusExchangeName(), this.busbyEsbProtocolVersion > 1 ? 'topic' : 'fanout', {
                        durable: this.xDurable,
                        autoDelete: false
                    });
                }
                await channel.assertExchange(this.getProviderControlExchangeName(), 'topic', {
                    durable: this.xDurable,
                    autoDelete: false
                });
                await channel.assertQueue(this.getProviderControlQueueName(), {
                    durable: this.qDurable,
                    autoDelete: this.qAutoDelete,
                    maxLength: 1000,
                    exclusive: this.qExclusive
                });
                Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupProviderConnection:bind`)(this.getProviderControlQueueName(), this.getProviderControlExchangeName(), 'control');
                await channel.bindQueue(this.getProviderControlQueueName(), this.getProviderControlExchangeName(), 'control');
                await this.makeConsumer('provider', this.processors['control'])(channel, this.getProviderControlQueueName());
                this.channels['providerChannel'] ??= {};
                this.channels['providerChannel'].channel = channel;
                Log_1.Log.logESB("Channel setup providerChannel " + this.providerId);
            }
            else {
                Log_1.Log.logErrorAndThrow("cannot setup providerChannel, no connection");
            }
        }
        catch (err) {
            Log_1.Log.logError(`Failed setting up providerChannel ${this.providerId} - ${err}`);
        }
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupProviderConnection`)(!!this.channels['providerChannel']);
        this.setupProviderConnectionPromise = null;
        return this.channels['providerChannel']?.channel;
    }
    async setupProviderConnection(controlProcessor, connectionChanged) {
        Log_1.Log.logESB("Setting up provider connection");
        if (controlProcessor) {
            this.processors['control'] = controlProcessor;
        }
        if (connectionChanged) {
            this.onSubscriberActive = connectionChanged;
        }
        if (this.channels['providerChannel']) {
            return this.channels['providerChannel']?.channel;
        }
        else {
            if (!this.setupProviderConnectionPromise) {
                this.setupProviderConnectionPromise = this.setupProviderConnectionRun();
            }
            return await this.setupProviderConnectionPromise;
        }
    }
    addStatusBinding(messageName) {
        if (!this.statusRoutingKeys.has(messageName)) {
            Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:statusBinding:add`)(this.getSubscriberStatusQueueName(), this.getProviderStatusExchangeName(), messageName, this.busbyEsbProtocolVersion);
            this.statusRoutingKeys.add(messageName);
            if (this.busbyEsbProtocolVersion > 1 && this.isSubscriberConnected()) {
                Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:statusBinding:bind`)(this.getSubscriberStatusQueueName(), this.getProviderStatusExchangeName(), messageName);
                this.channels['subscriberChannel']?.channel?.bindQueue(this.getSubscriberStatusQueueName(), this.getProviderStatusExchangeName(), messageName);
            }
        }
    }
    removeStatusBinding(messageName) {
        if (this.statusRoutingKeys.delete(messageName)) {
            Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:statusBinding:remove`)(this.getSubscriberStatusQueueName(), this.getProviderStatusExchangeName(), messageName, this.busbyEsbProtocolVersion);
            if (this.busbyEsbProtocolVersion > 1 && this.isSubscriberConnected() && messageName !== "status") {
                Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:statusBinding:unbind`)(this.getSubscriberStatusQueueName(), this.getProviderStatusExchangeName(), messageName);
                this.channels['subscriberChannel']?.channel?.unbindQueue(this.getSubscriberStatusQueueName(), this.getProviderStatusExchangeName(), messageName);
            }
        }
    }
    getProtocolInfo() {
        return { version: this.busbyEsbProtocolVersion };
    }
    getStatusRoutingKeys() {
        if (this.busbyEsbProtocolVersion == 1) {
            return ['#'];
        }
        else {
            return Array.from(this.statusRoutingKeys).concat(['status']);
        }
    }
    async setupSubscriberConnectionRun() {
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupSubscriberConnection`)("new", this.providerId);
        try {
            let channel = await this.connect(this.subscriberServerConn, 'subscriberChannel');
            if (channel != null && this.providerId !== null) {
                this.busbyEsbProtocolVersion = await this.getProtocolVersion(this.subscriberId);
                let fails = false;
                try {
                    await channel.assertExchange(this.getProviderStatusExchangeName(), this.busbyEsbProtocolVersion > 1 ? 'topic' : 'fanout', {
                        durable: this.xDurable,
                        autoDelete: false
                    });
                }
                catch (err) {
                    if (this.busbyEsbProtocolVersion > 1) {
                        fails = true;
                    }
                    else {
                        throw err;
                    }
                }
                if (fails) {
                    channel = await this.connect(this.subscriberServerConn, 'subscriberChannel');
                    await channel.deleteExchange(this.getProviderStatusExchangeName());
                    await channel.assertExchange(this.getProviderStatusExchangeName(), this.busbyEsbProtocolVersion > 1 ? 'topic' : 'fanout', {
                        durable: this.xDurable,
                        autoDelete: false
                    });
                }
                await channel.assertExchange(this.getProviderControlExchangeName(), 'topic', {
                    durable: this.xDurable,
                    autoDelete: false
                });
                await channel.assertQueue(this.getSubscriberStatusQueueName(), {
                    durable: this.qDurable,
                    autoDelete: this.qAutoDelete,
                    maxLength: 1000,
                    exclusive: this.qExclusive
                });
                await channel.assertQueue(this.getSubscriberReplyQueueName(), {
                    durable: this.qDurable,
                    autoDelete: this.qAutoDelete,
                    maxLength: 1000,
                    exclusive: this.qExclusive
                });
                for (const k of this.getStatusRoutingKeys()) {
                    Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupSubscriberConnection:bind`)(this.getSubscriberStatusQueueName(), this.getProviderStatusExchangeName(), k);
                    await channel.bindQueue(this.getSubscriberStatusQueueName(), this.getProviderStatusExchangeName(), k);
                }
                await this.makeConsumer('subscriber')(channel, this.getSubscriberReplyQueueName());
                await this.makeConsumer('subscriberStatus', this.processors['status'])(channel, this.getSubscriberStatusQueueName());
                this.channels['subscriberChannel'] ??= {};
                this.channels['subscriberChannel'].channel = channel;
                this.onSubscriberActive(true, "subscriber setup");
            }
            else {
                Log_1.Log.logErrorAndThrow("cannot setup subscriberChannel, no connection");
            }
        }
        catch (err) {
            Log_1.Log.logError(`Failed setting up subscriberChannel ${this.subscriberId} - ${err}`);
        }
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupSubscriberConnection`)(!!this.channels['subscriberChannel']);
        this.setupSubscriberConnectionPromise = null;
        return this.channels['subscriberChannel']?.channel;
    }
    async setupSubscriberConnection(statusProcessor) {
        if (statusProcessor) {
            this.processors['status'] = statusProcessor;
        }
        if (this.channels['subscriberChannel']) {
            return this.channels['subscriberChannel']?.channel;
        }
        else {
            if (!this.setupSubscriberConnectionPromise) {
                this.setupSubscriberConnectionPromise = this.setupSubscriberConnectionRun();
            }
            return await this.setupSubscriberConnectionPromise;
        }
    }
    async rebindStatusQueue(providerId) {
        const channel = this.channels['subscriberChannel']?.channel;
        if (channel) {
            const queueName = this.getSubscriberStatusQueueName();
            const oldExchange = this.getOldProviderStatusExchangeName(providerId);
            const newExchange = this.getProviderStatusExchangeName();
            try {
                Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:rebindStatusQueue`)(queueName, oldExchange, newExchange);
                if (newExchange !== this.getOldProviderStatusExchangeName(null)) {
                    for (const k of this.getStatusRoutingKeys()) {
                        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:rebindStatusQueue:bind`)(queueName, newExchange, k);
                        await channel.bindQueue(queueName, newExchange, k);
                    }
                }
                if (oldExchange !== this.getOldProviderStatusExchangeName(null) && oldExchange !== newExchange) {
                    for (const k of this.getStatusRoutingKeys()) {
                        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:rebindStatusQueue:unbind`)(queueName, newExchange, k);
                        await channel.unbindQueue(queueName, oldExchange, k);
                    }
                }
            }
            catch (err) {
                Log_1.Log.logError(`could not bind status queue ${queueName} ${oldExchange} ${newExchange}`);
            }
        }
    }
    hasProviderId() {
        return !!this.providerId;
    }
    async directoryStatusUpdates(incoming) {
        if (this.directoryStatusProcessor) {
            this.directoryStatusProcessor(incoming);
        }
        if (incoming.getMessageName() === "directoryUpdate") {
            const updateServiceId = incoming.getParams().serviceId;
            const updateProviderId = incoming.getParams().providerId;
            const providerState = incoming.getParams().providerState;
            if (this.onDirectoryChanged) {
                await this.onDirectoryChanged(incoming.getParams().serviceId, incoming.getParams().providerId, providerState);
            }
            if (updateServiceId === this.serviceId) {
                if (providerState === ProviderState_1.ProviderState.online) {
                }
                else if (providerState === ProviderState_1.ProviderState.active || providerState === ProviderState_1.ProviderState.primary) {
                    Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:directoryStatusUpdates:providerChanged`)(this.serviceId, this.providerId, updateProviderId);
                    Log_1.Log.logESB("active provider of service " + this.serviceId + " changed to " + updateProviderId);
                    if (this.subscriberId) {
                        if (this.providerId !== updateProviderId) {
                            Log_1.Log.logWarn(`Provider for ${this.serviceId} changed from ${this.providerId || "DISCONNECTED"} to ${updateProviderId}`);
                        }
                        this.providerChanged = true;
                        this.providerId = updateProviderId;
                        await this.disconnect("service status update, provider change");
                        await this.setupSubscriberConnection();
                    }
                }
                else if (providerState === ProviderState_1.ProviderState.nocomms || providerState === ProviderState_1.ProviderState.offline) {
                    if (updateProviderId === this.providerId) {
                        this.statusChanged = true;
                        Log_1.Log.logWarn(`Current provider for ${this.serviceId} has gone to ${incoming.getParams().providerState}`);
                        if (this.subscriberId) {
                            this.providerId = null;
                            this.onSubscriberActive(false, "provider change state");
                            await this.disconnect("service status update, provider gone");
                        }
                    }
                }
                else if (providerState === ProviderState_1.ProviderState.forceFail) {
                    if (this.providerMode && (updateProviderId === this.providerId)) {
                        Log_1.Log.logError("Force fail called");
                        await this.disconnect("service status update, force fail");
                        Log_1.Log.flushAndExit(100);
                        setTimeout(() => {
                            process.exit(100);
                        }, 1000);
                    }
                }
                this.statusChanged = true;
            }
            else if (!this.serviceId && this.subscriberId && this.providerId) {
                switch (providerState) {
                    case ProviderState_1.ProviderState.nocomms:
                    case ProviderState_1.ProviderState.forceFail:
                        this.statusChanged = true;
                        this.onSubscriberActive(false, "specific provider");
                        await this.disconnect("service status update, specific gone");
                        break;
                    case ProviderState_1.ProviderState.active:
                    case ProviderState_1.ProviderState.standby:
                    case ProviderState_1.ProviderState.online:
                    case ProviderState_1.ProviderState.offline:
                        if (!this.isSubscriberConnected()) {
                            this.providerChanged = true;
                            this.providerId = updateProviderId;
                            await this.disconnect("service status update, specific change");
                            await this.setupSubscriberConnection();
                        }
                        break;
                }
            }
        }
        else if (incoming.getMessageName() === 'configPublished') {
            if (this.configChangedUpdater && this.configChangedUpdater.onConfigChanged && this.configName) {
                const configuration = await this.getConfiguration(this.configName);
                this.configChangedUpdater.onConfigChanged(configuration);
            }
        }
    }
    async getProtocolVersion(id) {
        let result = 1;
        const channel = await this.setupDirectoryConnection(id, this.directoryStatusUpdates.bind(this));
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(id, 'getProtocolVersion', {});
        const response = await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(id), 'directory', this.defaultShortTimeout);
        const reply = EsbMessage_1.EsbMessage.makeFromJson(response);
        if (!reply.isError()) {
            result = reply.getParams().protocolVersion;
        }
        return result;
    }
    async getServiceProvider(serviceId) {
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:getServiceProvider`)(serviceId);
        let result = "";
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        if (channel != null) {
            Log_1.Log.logESB('Get directory client config ' + this.dirServerConn + " " + serviceId);
            const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'getProvider', { serviceId: serviceId });
            const response = await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', this.defaultShortTimeout);
            const reply = EsbMessage_1.EsbMessage.makeFromJson(response);
            if (reply.isError()) {
                this.providerId = null;
                Log_1.Log.logESB('Failed to get directory client config ' + reply.getParams().reason);
                let error = `Failed to get directory connection (${serviceId}): ${reply.getParams().reason}`;
                this.getServiceProviderErrors++;
                if (this.getServiceProviderErrors <= 10) {
                    Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:getServiceProvider`)(serviceId, `${error}`);
                    Log_1.Log.logError(error + (this.getServiceProviderErrors === 10 ? ' suppressing errors' : ''));
                }
            }
            else {
                this.getServiceProviderErrors = 0;
                const previousId = this.providerId;
                this.providerId = reply.getParams().providerId;
                if (this.subscriberId) {
                    if (previousId !== this.providerId) {
                        this.rebindStatusQueue(previousId);
                    }
                }
                result = this.providerId;
                Log_1.Log.logESB('got provider ' + this.providerId + ' for service ' + serviceId);
            }
        }
        else {
            Log_1.Log.logError("cannot get channel to get service id");
        }
        return result;
    }
    async publishConfig(configData) {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Publish config ' + this.dirServerConn + " ");
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'publishConfig', { configData: configData });
        return await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', 60);
    }
    async getSystemConfig(version) {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Get config ' + this.dirServerConn + " ");
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'getSystemConfig', { version });
        return await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', this.defaultShortTimeout);
    }
    async getSystemConfigVersions() {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Get system config versions ' + this.dirServerConn + " ");
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'getAvailableSystemConfigVersions', {});
        return await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', 60);
    }
    async getCurrentSystemConfig() {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Get system config ' + this.dirServerConn + " ");
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'getCurrentSystemConfig', {});
        return await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', this.defaultShortTimeout);
    }
    async getCurrentSurveyConfig() {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Get survey config ' + this.dirServerConn + " ");
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'getCurrentSurveyConfig', {});
        return await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', this.defaultShortTimeout);
    }
    async getConfigOverview() {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Get config overview ' + this.dirServerConn + " ");
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'getConfigOverview', {});
        return await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', this.defaultShortTimeout);
    }
    async saveSystemConfig(configData, comment, user) {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Publish config ' + this.dirServerConn + " ");
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'saveSystemConfig', { configData, comment, user });
        return await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', this.defaultShortTimeout);
    }
    async saveSurveyConfig(configuration, comment) {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Publish survey config ' + this.dirServerConn + " ");
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'saveSurveyConfig', { configuration, comment });
        return await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', this.defaultShortTimeout);
    }
    async getSystemDescription() {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        Log_1.Log.logESB('Get system description ' + this.dirServerConn + " ");
        const result = await this.sendControlMessageToDirectoryService('getSystemDescription', {});
        return result;
    }
    async saveProviderConfig(configData, providerId, host) {
        const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, 'saveProviderConfig', { configData, providerId, host });
        const reply = await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(this.subscriberId), 'directory', this.defaultShortTimeout);
        return !reply.isError();
    }
    setConfigName(configName) {
        this.configName = configName;
    }
    async getConfiguration(configName) {
        return this.doGetConfiguration(configName);
    }
    async doGetConfiguration(configName, count = 0) {
        let config = null;
        let id = (this.providerMode) ? this.providerId : this.subscriberId;
        try {
            const channel = await this.setupDirectoryConnection(id, this.directoryStatusUpdates.bind(this));
            const message = EsbMessage_1.EsbMessage.makeSystemMessage(configName, 'getConfiguration', { configName: configName });
            const response = await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(id), 'directory', this.defaultShortTimeout);
            const reply = EsbMessage_1.EsbMessage.makeFromJson(response);
            if (reply.isError()) {
                Log_1.Log.logError(`error getting config ${reply.getStatus()} (${count})`);
                if (configName === this.providerId) {
                    throw new Error(reply.getStatus());
                }
            }
            else {
                let params = reply.getParams();
                config = Configuration_1.Configuration.makeFromJson(params);
                if (configName === this.providerId) {
                    if (config.getConfigData() && config.getConfigData().logLevels) {
                        _.each(config.getConfigData().logLevels, (value, key) => {
                            if (!Log_1.LogLevel[key]) {
                                Log_1.Log.logError(`Log Level [${key}] is unknown`);
                            }
                            else {
                                Log_1.Log.setLogLevel(Log_1.LogLevel[key], value);
                            }
                        });
                    }
                    if (config.getConfigData() && config.getConfigData().logKeepDays) {
                        Log_1.Log.setKeepDays(config.getConfigData().logKeepDays);
                    }
                    if (config.getConfigData() && config.getConfigData().logZipAfter) {
                        Log_1.Log.setZipAfter(config.getConfigData().logZipAfter);
                    }
                }
            }
        }
        catch (e) {
            if (this.providerMode && configName === this.providerId) {
                Log_1.Log.logError(`error getting config for this provider ${configName} attempt ${count}`, e);
                // Exit after three attempts to get config
                if (count === 3) {
                    Log_1.Log.logError("exiting after three attemts");
                    Log_1.Log.flushAndExit(101);
                    setTimeout(() => {
                        // Force exit after 1 second if logs haven't flushed
                        process.exit(101);
                    }, 1000);
                }
                else {
                    await this.disconnect("Failed getting config", true);
                    this.connection = null;
                    const currentHost = this.loginDetails.getHost();
                    this.loginDetails.nextHost();
                    const newHost = this.loginDetails.getHost();
                    if (currentHost !== newHost) {
                        Log_1.Log.logWarn(`core host changed because of failure to get config ${currentHost} -> ${newHost}`);
                    }
                    await new Promise((resovle) => setTimeout(resovle, 500 * (count + 1)));
                    return this.doGetConfiguration(configName, count + 1);
                }
            }
            else {
                throw e;
            }
        }
        return config;
    }
    getCertificate(providerId, alternativeNames) {
        let id = (this.providerMode) ? this.providerId : this.subscriberId;
        return this.setupDirectoryConnection(id, this.directoryStatusUpdates.bind(this))
            .then((channel) => {
            let message;
            message = EsbMessage_1.EsbMessage.makeSystemMessage(id, 'getCertificate', { providerId, alternativeNames });
            return this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(id), 'directory', 60);
        })
            .then((response) => {
            let reply;
            reply = EsbMessage_1.EsbMessage.makeFromJson(response);
            if (reply.isError()) {
                return Log_1.Log.logErrorAndReject(reply.getAsJsonString());
            }
            return reply.getParams();
        });
    }
    async getService(providerId) {
        const channel = await this.setupDirectoryConnection(providerId, this.directoryStatusUpdates.bind(this));
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(providerId, 'getService', { providerId: providerId });
        const response = await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(providerId), 'directory', this.defaultShortTimeout);
        if (response.isError()) {
            return Log_1.Log.logErrorAndReject(response.getStatus());
        }
        return response.getParams().serviceId;
    }
    async sendControlMessageToDirectoryService(messageName, params) {
        let result = null;
        try {
            const channel = await this.setupDirectoryConnection(this.subscriberId, this.directoryStatusUpdates.bind(this));
            let toSend;
            toSend = EsbMessage_1.EsbMessage.makeSystemMessage(this.subscriberId, messageName, params);
            Log_1.Log.logESB('send control message to directory service ' + toSend.getAsJsonString());
            const response = await PromiseTimeout_1.PromiseTimeout.promiseTimeout(20000, await this.doRequest(this.dirExchange, channel, toSend, 'status', this.getSubscriberReplyQueueName(), 'subscriber', this.defaultShortTimeout));
            result = EsbMessage_1.EsbMessage.makeFromJson(response);
        }
        catch (err) {
            Log_1.Log.logError('Failed to send control message to directory service ' + err);
        }
        return result;
    }
    async tryActive(providerId) {
        const channel = await this.setupDirectoryConnection(providerId, this.directoryStatusUpdates.bind(this));
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(providerId, 'tryActive', { providerId: providerId });
        const reply = await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(providerId), 'directory', this.defaultShortTimeout);
        if (reply.isError()) {
            throw new Error(reply.getParams().reason);
        }
        return Boolean(reply.getParams().isActive);
    }
    async tryOnline(providerId, howBusy = 0.5) {
        const channel = await this.setupDirectoryConnection(providerId, this.directoryStatusUpdates.bind(this));
        const message = EsbMessage_1.EsbMessage.makeSystemMessage(providerId, 'tryOnline', { providerId: providerId, howBusy });
        const reply = await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(providerId), 'directory', this.defaultShortTimeout);
        if (reply.isError()) {
            throw new Error(reply.getParams().reason);
        }
        return Boolean(reply.getParams().isOnline);
    }
    async setServiceAvailability(providerId, state) {
        let result = false;
        try {
            const channel = await this.setupDirectoryConnection(providerId, this.directoryStatusUpdates.bind(this));
            const message = EsbMessage_1.EsbMessage.makeSystemMessage(providerId, 'setServiceAvailability', {
                providerId: providerId,
                providerState: ProviderState_1.ProviderState[state]
            });
            const reply = await this.doRequest(this.dirExchange, channel, message, 'status', this.getDirReplyQueueName(providerId), 'directory', this.defaultShortTimeout);
            result = !reply.isError();
        }
        catch (e) {
        }
        finally {
            return result;
        }
    }
    async setupProvider(providerId, serviceId, controlProcessor, onActive) {
        this.providerId = providerId;
        this.serviceId = serviceId;
        return await this.setupProviderConnection(controlProcessor, onActive);
    }
    async sendStatus(message) {
        try {
            const channel = await this.setupProviderConnection();
            const routingKey = this.busbyEsbProtocolVersion == 2 ? message.getMessageName() : '*';
            await this.doPublish(this.getProviderStatusExchangeName(), channel, message, routingKey);
        }
        catch (err) {
            Log_1.Log.logError(`Unable to send status message ${message.getMessageName()}`);
        }
    }
    async setupSubscriber(serviceId, statusProcessor, onActive, options) {
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupSubscriber`)(serviceId);
        this.serviceId = serviceId;
        this.subscriberOptions = options;
        this.statusProcessor = statusProcessor;
        this.processors['status'] = statusProcessor;
        this.onSubscriberActive = onActive;
        this.onDirectoryChanged = options?.onDirectoryChange;
        let channel = false;
        try {
            const providerId = await this.getServiceProvider(serviceId);
            if (providerId) {
                channel = !!await this.setupSubscriberConnection(this.statusProcessor);
            }
        }
        catch (err) {
        }
        return !!channel && this.providerId;
    }
    async setupSubscriberProvider(providerId, statusProcessor, onActive, options) {
        Log_1.Log.getNamespace(`rabbit:${this.providerMode ? this.providerId : this.subscriberId}:setupSubscriberProvider`)(providerId);
        this.serviceId = null;
        this.subscriberOptions = options;
        this.statusProcessor = statusProcessor;
        this.processors['status'] = statusProcessor;
        this.onSubscriberActive = onActive;
        this.onDirectoryChanged = options?.onDirectoryChange;
        this.providerId = providerId;
        let channel = null;
        try {
            channel = await this.setupSubscriberConnection(this.statusProcessor);
        }
        catch (err) {
        }
        return !!channel ? providerId : null;
    }
    async sendControl(messageName, params, timeoutSecs) {
        let result = null;
        try {
            const channel = await this.setupSubscriberConnection();
            let toSend;
            toSend = EsbMessage_1.EsbMessage.makeControlMessage(this.subscriberId, messageName, params);
            toSend.setToService(this.serviceId);
            const timeout = timeoutSecs ? timeoutSecs : 20;
            const response = await this.doRequest(this.getProviderControlExchangeName(), channel, toSend, 'control', this.getSubscriberReplyQueueName(), 'subscriber', timeout);
            result = EsbMessage_1.EsbMessage.makeFromJson(response);
        }
        catch (err) {
            Log_1.Log.logError('Failed to send control message ' + err);
        }
        return result;
    }
    async close() {
        await this.disconnect("close called", true);
    }
}
exports.RabbitConnection = RabbitConnection;
