"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Provider = void 0;
const EsbMessage_1 = require("./EsbMessage");
const MultiRabbitConnection_1 = require("./MultiRabbitConnection");
const LoginDetails_1 = require("./LoginDetails");
const Log_1 = require("../logging/Log");
const ProviderState_1 = require("./ProviderState");
const Subscriber_1 = require("./Subscriber");
const FileTransferProvider_1 = require("./FileTransferProvider");
const BigDataSubscriber_1 = require("../subscribers/BigDataSubscriber");
const Cli_1 = require("./Cli");
const CliCommandDescription_1 = require("./Cli/CliCommandDescription");
class Provider {
    static async makeProvider(cp) {
        const provider = new Provider(LoginDetails_1.LoginDetails.get().getProviderId(), LoginDetails_1.LoginDetails.get(), cp);
        return provider;
    }
    constructor(providerId, loginDetails, cp, options = {}) {
        this.providerId = providerId;
        this.loginDetails = loginDetails;
        this.messageLoggingBigDataService = "";
        this.bigData = null;
        this.secondaryProvider = false;
        this.onActive = async (active, location) => {
            if (this.providerSetup && !active) {
                this.providerSetup = false;
                this.waitingPromise = null;
            }
        };
        this.tryActiveFailures = 0;
        this.rabbitConnection = new MultiRabbitConnection_1.RabbitConnection(loginDetails, true);
        this.providerSetup = false;
        this.providerStatus = ProviderState_1.ProviderState.offline;
        this.controlProcessor = this.hookIntoControlProcessor(cp);
        this.rabbitConnection.setConfigChangedUpdater(this);
        this.cliParser = Cli_1.CliParser.get();
        this.secondaryProvider = options.secondaryProvider ?? false;
        process.on("unhandledRejection", (error) => {
            console.error(error); // This prints error with stack included (as for normal errors)
            throw error; // Following best practices re-throw error and let the process exit with error code
        });
    }
    async onConfigChanged(configuration) {
        Log_1.Log.getNamespace(`provider:${this.providerId}:onConfigChanged`)("changed");
        this.configuration = configuration;
        if (configuration.getConfigData().messageLoggingBigDataService) {
            this.messageLoggingBigDataService = configuration.getConfigData().messageLoggingBigDataService;
        }
        this.redundancyMode = configuration.getConfigData().serviceRedundancyMode || "activeStandby";
        if (this.configChanged) {
            this.configChanged.onConfigChanged(configuration);
        }
    }
    enableFileTransfer() {
        this.fileTransferProvider = new FileTransferProvider_1.FileTransferProvider(this.providerId);
    }
    async getBigData() {
        if (this.bigData == null) {
            const serviceId = await this.getServiceId();
            this.bigData = new BigDataSubscriber_1.BigDataSubscriber(this.messageLoggingBigDataService, this.providerId, this.loginDetails, serviceId);
        }
        return this.bigData;
    }
    hookIntoControlProcessor(cp) {
        return async (msg) => {
            let reply;
            Log_1.Log.getNamespace(`provider:${this.providerId}:control:${msg.getMessageName()}`)("recv", msg.getParams());
            if (this.fileTransferProvider !== null && msg.getMessageName().startsWith("qq")) {
                try {
                    let result;
                    if (msg.getMessageName() == "qqGetFilesInFolder") {
                        result = await this.fileTransferProvider.getFilesInFolder(msg);
                    }
                    else if (msg.getMessageName() == "qqGetFileDetails") {
                        result = await this.fileTransferProvider.getFileDetails(msg);
                    }
                    else if (msg.getMessageName() == "qqGetFileBlock") {
                        result = await this.fileTransferProvider.getFileBlock(msg);
                    }
                    else if (msg.getMessageName() == "qqWriteFileBlock") {
                        result = await this.fileTransferProvider.writeFileBlock(msg);
                    }
                    else if (msg.getMessageName() == "qqDeleteFile") {
                        result = await this.fileTransferProvider.deleteFile(msg);
                    }
                    else if (msg.getMessageName() == "qqCliCommand") {
                        result = await this.cliParser.processEsbCommand(msg, this);
                    }
                    else if (msg.getMessageName() == "qqCliAutoComplete") {
                        result = await this.cliParser.processEsbAutoComplete(msg, this);
                    }
                    else {
                        throw new Error("Not implemented");
                    }
                    reply = EsbMessage_1.EsbMessage.makeControlReply(this.providerId, msg, result);
                }
                catch (err) {
                    reply = EsbMessage_1.EsbMessage.makeControlReplyNak(this.providerId, msg, `failed processing command - ${err}`);
                }
            }
            else if (msg.getMessageName() == "qqResendStatus") {
                reply = await this.doResendStatus(msg);
            }
            else {
                reply = await cp(msg);
            }
            if (this.messageLoggingBigDataService != "") {
                const inLog = msg.getBigDataLogMessage(this.providerId);
                const replyLog = reply.getBigDataLogMessage(this.providerId);
                const bigData = await this.getBigData();
                const serviceId = await this.getServiceId();
                await bigData.insert("esbMessage", serviceId, inLog, msg.getTimeStamp().toISOString());
                await bigData.insert("esbMessage", serviceId, replyLog, reply.getTimeStamp().toISOString());
            }
            Log_1.Log.getNamespace(`provider:${this.providerId}:control:${msg.getMessageName()}`)("send", reply.getParams());
            return reply;
        };
    }
    async doResendStatus(msg) {
        for (const message of msg.getParams().messages) {
            await this.resendStatusEvent(message.messageName, message.params);
        }
        return EsbMessage_1.EsbMessage.makeControlReplyAck(this.providerId, msg);
    }
    setConfigChangedUpdater(configChanged) {
        this.configChanged = configChanged;
    }
    listenDirectoryChanged(directoryChanged) {
        this.rabbitConnection.listenDirectoryChanged(directoryChanged);
    }
    async waitUntilConnected() {
        if (!this.waitingPromise) {
            Log_1.Log.getNamespace(`provider:${this.providerId}:wait`)(`called`);
            this.waitingPromise = new Promise(async (resolve) => {
                let reconnects = 0;
                let timeout;
                const tryConnect = async () => {
                    try {
                        await this.connect();
                        if (this.providerSetup && reconnects > 0) {
                            reconnects = 0;
                        }
                        if (this.providerSetup) {
                            Log_1.Log.getNamespace(`provider:${this.providerId}:wait`)(`connected`);
                        }
                    }
                    catch (e) {
                        reconnects++;
                        if (reconnects < 10) {
                            Log_1.Log.logError(`Could not connect provider ${this.providerId}, waiting to re-try`);
                        }
                        else if (reconnects == 10) {
                            Log_1.Log.logError(`Could not connect provider ${this.providerId}, waiting to re-try, suppressing errors to every 100 attempts`);
                        }
                        else if (reconnects % 100 === 0) {
                            Log_1.Log.logError(`Could not connect provider ${this.providerId}, waiting to re-try, attempts ${reconnects}`);
                        }
                    }
                    ;
                    if (this.providerSetup) {
                        resolve(true);
                        clearTimeout(timeout);
                    }
                    else {
                        timeout = setTimeout(tryConnect, 5000);
                    }
                };
                await tryConnect();
            });
        }
        return this.waitingPromise;
    }
    /**
     * @deprecated use waitUntilConnected() instead
     */
    async connect(isDirectory) {
        if (!this.providerSetup) {
            if (this.serviceAvailabilityTimeout) {
                clearTimeout(this.serviceAvailabilityTimeout);
                this.serviceAvailabilityTimeout = null;
            }
            let serviceId;
            try {
                if (isDirectory) {
                    serviceId = await this.rabbitConnection.getService(this.providerId).catch(err => {
                        return null;
                    });
                }
                else {
                    serviceId = await this.getServiceId();
                }
                if (serviceId || isDirectory) {
                    if (!this.secondaryProvider) {
                        LoginDetails_1.LoginDetails.get().setServiceId(serviceId);
                    }
                }
                const setup = await this.rabbitConnection.setupProvider(this.providerId, serviceId, this.controlProcessor, this.onActive);
                this.providerSetup = !!setup;
            }
            catch (err) {
                Log_1.Log.logError(`Unable to setup provider`, err);
                throw err;
            }
        }
        return this.providerSetup;
    }
    /**
     * Sets the service state.
     * @param providerState the provider state
     * @returns a promise
     * @deprecated use `waitUntilActive` or `waitUntilOnline` loops, or use `tryActive` or `tryOnline`
     */
    setServiceState(providerState) {
        this.providerStatus = providerState;
        return this.rabbitConnection.setServiceAvailability(this.providerId, this.providerStatus);
    }
    getServiceState() {
        return this.providerStatus;
    }
    getProviderId() {
        return this.providerId;
    }
    async getServiceId() {
        if (!this.serviceId) {
            const currentHost = LoginDetails_1.LoginDetails.get().getHost();
            try {
                const service = await Subscriber_1.Subscriber.connectSendClose(this.loginDetails, 'directory', 'getService', {
                    providerId: this.providerId
                }, { timeoutSecs: 5 });
                this.serviceId = service.getParams().serviceId;
            }
            catch (err) {
                LoginDetails_1.LoginDetails.get().nextHost();
                const newHost = LoginDetails_1.LoginDetails.get().getHost();
                if (newHost !== currentHost) {
                    Log_1.Log.logWarn(`core host changed cannot get service id ${currentHost} -> ${newHost}`);
                }
            }
        }
        return this.serviceId;
    }
    isActive() {
        return this.providerStatus === ProviderState_1.ProviderState.active;
    }
    isOnline() {
        return this.providerStatus === ProviderState_1.ProviderState.online;
    }
    async tryActive() {
        if (!this.providerSetup) {
            await this.connect();
        }
        if (this.redundancyMode == "activeActive") {
            return this.tryOnline();
        }
        else {
            const isActive = await this.rabbitConnection.tryActive(this.providerId);
            if (isActive) {
                this.providerStatus = ProviderState_1.ProviderState.active;
            }
            else {
                this.providerStatus = ProviderState_1.ProviderState.standby;
            }
            return isActive;
        }
    }
    async tryOnline() {
        if (!this.providerSetup) {
            await this.connect();
        }
        const isOnline = await this.rabbitConnection.tryOnline(this.providerId, this.busyness);
        if (isOnline) {
            this.providerStatus = ProviderState_1.ProviderState.online;
        }
        else {
            this.providerStatus = ProviderState_1.ProviderState.offline;
        }
        return isOnline;
    }
    waitUntilActive(sendServiceState) {
        return new Promise((resolve) => {
            const tryLoop = async () => {
                try {
                    const result = await this.tryActive();
                    if (sendServiceState && result) {
                        await this.setServiceState(ProviderState_1.ProviderState.active);
                    }
                    if (result) {
                        this.tryActiveFailures = 0;
                        resolve(null);
                    }
                }
                catch (e) {
                    Log_1.Log.logError("tryActive error: ", e);
                    this.tryActiveFailures++;
                    if (this.tryActiveFailures >= 3) {
                        Log_1.Log.flushAndExit(50);
                    }
                }
                setTimeout(tryLoop.bind(this), 3000);
            };
            setTimeout(tryLoop.bind(this), 3000);
        });
    }
    waitUntilOnline() {
        return new Promise((resolve) => {
            const tryLoop = async () => {
                try {
                    const result = await this.tryOnline();
                    if (result) {
                        this.tryActiveFailures = 0;
                        resolve(null);
                    }
                }
                catch (e) {
                    Log_1.Log.logError("tryOnline error: ", e);
                    this.tryActiveFailures++;
                    if (this.tryActiveFailures >= 3 && !process.env.NO_EXIT) {
                        Log_1.Log.flushAndExit(50);
                    }
                }
                setTimeout(tryLoop.bind(this), 3000);
            };
            setTimeout(tryLoop.bind(this), 3000);
        });
    }
    async sendStatusEvent(messageName, params, options) {
        const message = EsbMessage_1.EsbMessage.makeStatusMessage(this.providerId, messageName, params);
        try {
            await this.connect();
            await this.rabbitConnection.sendStatus(message);
            if (this.messageLoggingBigDataService != "" && !options?.dontLog) {
                const statusLog = message.getBigDataLogMessage(this.providerId);
                const bigData = await this.getBigData();
                const serviceId = await this.getServiceId();
                await bigData.insert("esbMessage", serviceId, statusLog, message.getTimeStamp().toISOString());
            }
        }
        catch (err) {
            Log_1.Log.logEx('Error in provider sendStatusEvent:', err);
        }
    }
    async resendStatusEvent(messageName, params) {
        const message = EsbMessage_1.EsbMessage.makeStatusMessageResend(this.providerId, messageName, params);
        try {
            await this.connect();
            await this.rabbitConnection.sendStatus(message);
        }
        catch (err) {
            Log_1.Log.logEx('Error in provider resendStatusEvent:', err);
        }
    }
    async getConfig() {
        const configuration = await this.rabbitConnection.getConfiguration(this.providerId);
        this.configuration = configuration;
        this.rabbitConnection.setConfigName(this.providerId);
        if (configuration.getConfigData().messageLoggingBigDataService) {
            this.messageLoggingBigDataService = configuration.getConfigData().messageLoggingBigDataService;
        }
        this.redundancyMode = configuration.getConfigData().serviceRedundancyMode || "activeStandby";
        return configuration;
    }
    async getSharedData(name) {
        let configuration = await this.rabbitConnection.getConfiguration(name + "__data");
        return configuration.getConfigData();
    }
    async getCertificate(providerId, alternativeNames) {
        let certificate = await this.rabbitConnection.getCertificate(providerId === null ? "ca" : providerId, alternativeNames);
        return certificate;
    }
    async lookupConfig(providerId) {
        return await this.rabbitConnection.getConfiguration(providerId);
    }
    getConfigOverview() {
        return this.rabbitConnection.getConfigOverview();
    }
    addCliCommand(command) {
        this.cliParser.addCommand(command);
    }
    addCliCommandProcessor(arg0, arg1, arg2 = () => [], autoCompleteProcessor = () => []) {
        const command = typeof arg0 === "string" ? arg0 : arg0.getCommand();
        const description = typeof arg1 === "string" ? arg1 : arg0.getDescription();
        Log_1.Log.getNamespace('cli:provider:addProcessor')(command, description);
        const commandDescription = typeof arg0 === "string" ? CliCommandDescription_1.CliCommandDescription.make(command, description) : arg0;
        this.cliParser.addCommandProcessor(commandDescription, typeof arg1 === "string" ? arg2 : arg1, typeof arg1 === "string" ? autoCompleteProcessor : arg2);
    }
    close() {
        this.rabbitConnection.close();
    }
    makeControlReply(incoming, params) {
        return EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, params);
    }
    makeControlReplyAck(incoming) {
        return EsbMessage_1.EsbMessage.makeControlReplyAck(this.providerId, incoming);
    }
    makeControlReplyNak(incoming, reason) {
        return EsbMessage_1.EsbMessage.makeControlReplyNak(this.providerId, incoming, reason);
    }
    getLoadedConfiguration() {
        return this.configuration;
    }
    setBusyness(busyness) {
        this.busyness = busyness;
    }
}
exports.Provider = Provider;
