"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscodeProvider = void 0;
const path = require("path");
const _ = require("lodash");
const Provider_1 = require("./Provider");
const EsbMessage_1 = require("./EsbMessage");
const BusbyLocationLookupSubscriber_1 = require("../subscribers/BusbyLocationLookupSubscriber");
class TranscodeProvider {
    constructor(providerId, loginDetails, driver) {
        this.providerId = providerId;
        this.loginDetails = loginDetails;
        this.driver = driver;
        this.provider = new Provider_1.Provider(providerId, loginDetails, this.controlProcessor.bind(this));
        this.locationService = new BusbyLocationLookupSubscriber_1.BusbyLocationLookupSubscriber(loginDetails);
    }
    setConfigChangedUpdater(updater) {
        this.provider.setConfigChangedUpdater(updater);
    }
    async init() {
        await this.locationService.waitUntilConnected();
        await this.provider.connect();
        await this.provider.waitUntilActive();
    }
    async getConfig() {
        return await this.provider.getConfig();
    }
    replaceFilenameExtension(filename, extension) {
        if (!extension) {
            return filename;
        }
        else {
            return filename.replace(/[^.]+$/, extension);
        }
    }
    async controlProcessor(incoming) {
        const params = incoming.getParams();
        try {
            switch (incoming.getMessageName()) {
                case "getProfiles": {
                    const profile = incoming.getParams().profile;
                    return await EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, await this.driver.getProfiles());
                }
                case "transcode": {
                    const profile = incoming.getParams().profile;
                    const source = incoming.getParams().source;
                    const destination = incoming.getParams().destination;
                    const options = incoming.getParams().options;
                    let sourceFile;
                    if (_.isString(source)) {
                        sourceFile = source;
                    }
                    else {
                        let sourceLocation;
                        if (source.location) {
                            sourceLocation = (await this.locationService.getLocation({ locationName: source.location })).getResult();
                        }
                        sourceFile = path.join(sourceLocation ? sourceLocation.mount : '', source.path, source.filename);
                    }
                    let destinationFile;
                    if (_.isString(destination)) {
                        destinationFile = destination;
                    }
                    else {
                        let destinationLocation;
                        if (destination.location) {
                            destinationLocation = (await this.locationService.getLocation({ locationName: destination.location })).getResult();
                        }
                        destinationFile = path.join(destinationLocation ? destinationLocation.mount : '', destination.path, destination.filename || sourceFile);
                    }
                    destinationFile = this.replaceFilenameExtension(destinationFile, _.get(options, 'extension'));
                    try {
                        const transcodeJob = await this.driver.transcode(profile, sourceFile, destinationFile, options);
                        if (this.driver.getPreviewUrl) {
                            transcodeJob.previewUrl = this.driver.getPreviewUrl(transcodeJob);
                        }
                        return await EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, transcodeJob);
                    }
                    catch (e) {
                        return await EsbMessage_1.EsbMessage.makeControlReplyNak(this.providerId, incoming, e.message);
                    }
                }
                case "getTranscodeInfo": {
                    const transcodeId = incoming.getParams().transcodeId;
                    const transcodeInfo = await this.driver.getTranscodeInfo(transcodeId);
                    return await EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, transcodeInfo);
                }
                case "getReport": {
                    const transcodeId = incoming.getParams().transcodeId;
                    const options = incoming.getParams().options;
                    return await EsbMessage_1.EsbMessage.makeControlReply(this.providerId, incoming, await this.driver.getReport(transcodeId, options));
                }
                default: {
                    return Promise.resolve(EsbMessage_1.EsbMessage.makeControlReplyNak(this.providerId, incoming, 'message name not recognised'));
                }
            }
        }
        catch (e) {
            return Promise.resolve(EsbMessage_1.EsbMessage.makeControlReplyNak(this.providerId, incoming, e.message));
        }
    }
}
exports.TranscodeProvider = TranscodeProvider;
