"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WsSubscriberAgent = void 0;
const _ = require("lodash");
const os = require("os");
const Subscriber_1 = require("./Subscriber");
const LoginDetails_1 = require("./LoginDetails");
const Log_1 = require("../logging/Log");
const subscribers_1 = require("../subscribers");
const SubscriberManager_1 = require("../subscribers/interfaces/SubscriberManager");
class WsSubscriberAgent {
    get user() {
        return this._user;
    }
    set user(user) {
        if (user) {
            Log_1.Log.logInfo(`Socket ${this.socketId} user ${user.username}`);
        }
        this.debug('set user', user?.username);
        this._user = user;
    }
    constructor(socket, clientIp, emitBackChannelMessage, emitStatusMessage, registerMessageProcessor) {
        this.clientIp = clientIp;
        this.emitBackChannelMessage = emitBackChannelMessage;
        this.emitStatusMessage = emitStatusMessage;
        this.registerMessageProcessor = registerMessageProcessor;
        this.registeredProcessors = {};
        this.registeredProcessorFunctions = {};
        this.getClientIp = () => {
            return this.clientIp;
        };
        this.getClientIpString = () => {
            return this.clientIp?.ipv4 ?? this.clientIp?.ipv6 ?? this.clientIp?.value;
        };
        this.statusProcessor = (statusProcessorId) => {
            if (!this.emitters[statusProcessorId]) {
                this.emitters[statusProcessorId] = (message) => {
                    this.emitStatusMessage(statusProcessorId, message);
                };
                this.emitters[statusProcessorId].statusProcessorId = statusProcessorId;
            }
            return this.emitters[statusProcessorId];
        };
        this.subscriberIds = {};
        this.emitters = {};
        if (typeof socket === "string") {
            this.socketId = socket;
        }
        else {
            this.socket = socket;
            this.socketId = socket.id;
            const host = socket.handshake?.headers?.['host'];
            const remoteAddress = socket.request?.connection?.remoteAddress;
            try {
                let forwarded = socket.handshake?.headers?.['forwarded'] || [];
                let forwardedFor = socket.handshake?.headers?.['x-forwarded-for'] || [];
                let forwardedHost = socket.handshake?.headers?.['x-forwarded-host'] || [];
                if (forwarded) {
                    forwardedFor ??= [];
                    if (typeof forwardedHost === "string") {
                        forwardedHost = forwardedHost.split(",");
                    }
                    if (typeof forwardedFor === "string") {
                        forwardedFor = forwardedFor.split(",");
                    }
                    if (typeof forwarded === "string") {
                        forwarded = forwarded.split(",");
                    }
                    for (const header of forwarded) {
                        const values = header.split(",").map(_.trim);
                        for (const value of values) {
                            const pairs = value.split(";").map(p => p.split("="));
                            for (const [k, v] of pairs) {
                                if (k === "for") {
                                    if (v.startsWith("\"[")) {
                                        forwardedFor.push(v.slice(2, v.indexOf("]")));
                                    }
                                    else {
                                        forwardedFor.push(v);
                                    }
                                }
                            }
                        }
                    }
                }
                let clientIp = forwardedFor[0] || remoteAddress;
                if (clientIp) {
                    if (clientIp.includes(':') && clientIp.includes('.')) {
                        const lastColon = clientIp.lastIndexOf(':');
                        const period = clientIp.indexOf('.');
                        if (lastColon < period) {
                            let ipv6 = clientIp.substring(0, lastColon + 1);
                            if (ipv6.endsWith(":")) {
                                ipv6 = ipv6.slice(0, -1);
                            }
                            const ipv4 = clientIp.substring(lastColon + 1);
                            this.clientIp = { ipv4: ipv4, ipv6: ipv6 };
                        }
                        else {
                            this.clientIp = { ipv4: clientIp.substring(0, lastColon) };
                        }
                    }
                    else if (clientIp.includes('.')) {
                        this.clientIp = { ipv4: clientIp };
                    }
                    else if (clientIp.includes(':')) {
                        if (clientIp.endsWith(":")) {
                            clientIp = clientIp.slice(0, -1);
                        }
                        this.clientIp = { ipv6: clientIp };
                    }
                    this.clientIp.value = clientIp;
                }
                Log_1.Log.logInfo(`Socket ${this.socketId} connection to ${host} from client ${clientIp}`);
                Log_1.Log.logDebug(`Socket ${this.socketId} connection address:[${remoteAddress}] x-forwarded-for:[${forwardedFor.join(", ")}] forwarded:[${forwarded.join(", ")}] x-forwarded-host:[${forwardedHost.join(", ")}]`);
            }
            catch (err) {
                Log_1.Log.logError(`Socket ${this.socketId} Error detecting client ip ${err.message}`, err);
                Log_1.Log.logInfo(`Socket ${this.socketId} connection to ${host} from client ${remoteAddress}`);
            }
            this.emitBackChannelMessage = (id, message) => {
                Log_1.Log.getNamespace(`socket:${this.socketId}:subscriberBackChannel:out:${id}:${message.messageName}`)(id, message);
                return new Promise((resolve, reject) => {
                    socket.emit(`${id}_subscriberBackChannel`, message, (reply) => {
                        resolve(reply);
                    });
                });
            };
            this.emitStatusMessage = (statusProcessorId, message) => {
                socket.emit(statusProcessorId, message);
            };
            this.registerMessageProcessor = (messageProcessorId, processor) => {
                Log_1.Log.getNamespace(`socket:${this.socketId}:registerMessageProcessor:${messageProcessorId}`)("registered", this.clientIp);
                Log_1.Log.logDebug(`Socket ${this.socketId} registered message processor ${messageProcessorId}`);
                if (this.registeredProcessors[messageProcessorId]) {
                    Log_1.Log.logError(`Socket ${this.socketId} Message processor ${messageProcessorId} already registered ${this.registeredProcessors[messageProcessorId] !== processor ? "with a different processor" : "with the same processor"}`);
                }
                else {
                    this.registeredProcessors[messageProcessorId] = processor;
                    this.registeredProcessorFunctions[messageProcessorId] = (message, callback) => {
                        _.each(message.options, (value, option) => {
                            switch (option) {
                                case "fromIp":
                                    _.merge(message.params, { $fromIp: this.clientIp });
                                    break;
                                case "fromUser": {
                                    if (this.user) {
                                        _.merge(message.params, { $fromUser: _.omit(this.user, "details") });
                                    }
                                    break;
                                }
                            }
                        });
                        processor(message)
                            .then((result) => {
                            callback(result);
                        });
                    };
                    socket.on(messageProcessorId, this.registeredProcessorFunctions[messageProcessorId]);
                }
            };
            socket.on('subscriberBackChannel', (message, callback) => {
                const { params: { subscriberId, serviceId, statusProcessorId }, messageName } = message;
                Log_1.Log.getNamespace(`socket:${this.socketId}:subscriberBackChannel:in:${subscriberId}:${messageName}`)(subscriberId, serviceId, statusProcessorId);
                this.handleBackChannelRequest(message)
                    .then((reply) => {
                    callback(reply);
                })
                    .catch(err => {
                    Log_1.Log.logError(`Socket ${this.socketId} Invalid back channel request ${JSON.stringify(message)}`);
                    Log_1.Log.logException(err);
                });
            });
            socket.on("disconnect", (reason, descrition) => {
                Log_1.Log.logInfo(`Socket ${this.socketId} disconnected user:[${this.user?.username}] reason:[${reason}] description:[${descrition}]`);
            });
        }
        this.debug = Log_1.Log.getNamespace(`socket:${this.socketId}:wsagent`);
        const nets = os.networkInterfaces();
        this.testHosts = ["localhost", "127.0.0.1", os.hostname()];
        for (const name of Object.keys(nets)) {
            for (const net of nets[name]) {
                if (net.family === 'IPv4') {
                    this.testHosts.push(net.address);
                }
            }
        }
    }
    async handleBackChannelRequest(message) {
        const { subscriberId, serviceId, statusProcessorId, params } = message.params;
        this.debug.extend('backchannel')(message.messageName, subscriberId, serviceId, statusProcessorId);
        try {
            switch (message.messageName) {
                case 'construct': return this.construct(message);
                case 'connect': return this.connect(message);
                case 'close': return this.close(message);
                case 'getConfigOverview': return this.getConfigOverview(message);
                case 'lookupConfig': return this.lookupConfig(message);
                case 'connectSendClose': return WsSubscriberAgent.connectSendClose(message);
                case 'addStatusSubscription': return this.addStatusSubscription(message);
                case 'removeStatusSubscription': return this.removeStatusSubscription(message);
            }
        }
        catch (e) {
            Log_1.Log.logError(`Socket ${this.socketId} Failed to handle back channel request`, e);
        }
        throw new Error("Failed to handle back channel request");
    }
    async construct(message) {
        const { subscriberId, statusProcessorId, subscriptions } = message.params;
        this.debug(`construct`, subscriberId, statusProcessorId);
        if (!this.subscriberIds[subscriberId]) {
            this.setup(subscriberId, statusProcessorId);
        }
        return true;
    }
    async addStatusSubscription(message) {
        const { subscriberId, messageName } = message.params;
        if (this.subscriberIds[subscriberId].subscriber) {
            this.subscriberIds[subscriberId].subscriber.status[messageName](this.subscriberIds[subscriberId].statusProcessor);
        }
    }
    async removeStatusSubscription(message) {
        const { subscriberId, messageName } = message.params;
        if (this.subscriberIds[subscriberId].subscriber) {
            this.subscriberIds[subscriberId].subscriber.status[messageName](this.subscriberIds[subscriberId].statusProcessor).dispose();
        }
    }
    setup(subscriberId, statusProcessorId) {
        let statusProcessor = null;
        let messageProcessorId = subscriberId + '_message';
        if (statusProcessorId) {
            statusProcessor = this.statusProcessor(statusProcessorId);
        }
        this.subscriberIds[subscriberId] = {
            messageProcessorId,
            statusProcessor,
            onConnected: () => this.emitBackChannelMessage(subscriberId, { messageName: "subscriberStatusChange", connected: true }),
            onDisconnected: () => this.emitBackChannelMessage(subscriberId, { messageName: "subscriberStatusChange", connected: false }),
            onDirectoryChange: (service, provider, state) => this.emitBackChannelMessage(subscriberId, { messageName: "directoryChanged", service, provider, state }),
            callback: async (message) => {
                if (this.subscriberIds[subscriberId].subscriber) {
                    if (message.options?.batch === null || message.options?.batch === undefined) {
                        message.options = { ...message.options, batch: true };
                    }
                    const debug = Log_1.Log.getNamespace(`socket:${this.socketId}:wsagent:${messageProcessorId}`);
                    debug(`Sending message [${subscriberId}] ${message.messageName}`);
                    Log_1.Log.logTrace(`Socket ${this.socketId} Sending message [${subscriberId}] ${message.messageName}`);
                    if (!this.subscriberIds[subscriberId].subscriber) {
                        debug(`Cancelling message as no longer in use [${subscriberId}] ${message.messageName}`);
                        return null;
                    }
                    await this.subscriberIds[subscriberId].subscriber.getSubscriber().connect();
                    const reply = await this.subscriberIds[subscriberId].subscriber.getSubscriber().sendControl(message.messageName, message.params, message.options);
                    debug(`Received reply [${subscriberId}] ${message.messageName}`);
                    Log_1.Log.logTrace(`Socket ${this.socketId} Received reply [${subscriberId}] ${message.messageName}`);
                    return reply;
                }
            }
        };
        this.registerMessageProcessor(messageProcessorId, this.subscriberIds[subscriberId].callback);
    }
    async connect(message) {
        const { subscriberId, serviceId, statusProcessorId, loginDetails: ld, subscriptions } = message.params;
        const debug = this.debug.extend(subscriberId);
        if (!this.subscriberIds[subscriberId]) {
            this.setup(subscriberId, statusProcessorId);
        }
        if (!this.subscriberIds[subscriberId].subscriber) {
            debug(`connecting ${subscriberId}`);
            let loginDetails = LoginDetails_1.LoginDetails.get();
            if (ld) {
                // If login details provided from frontend add provider id
                if (!ld.providerId || ld.providerId === "noId") {
                    ld.providerId = loginDetails.getProviderId();
                }
                const test = LoginDetails_1.LoginDetails.makeFromJson(ld);
                if (test.getHosts().filter(h => loginDetails.getHosts().includes(h)).length === 0) {
                    // If the client login details hosts do not reference a local address
                    if (test.getHosts().filter(h => this.testHosts.includes(h)).length === 0) {
                        ld.providerId = `${ld.providerId}--${os.hostname()}`;
                    }
                }
                loginDetails = LoginDetails_1.LoginDetails.makeFromJson(ld);
            }
            const subscriber = SubscriberManager_1.SubscriberManager.getManager(loginDetails, 'subscriber')
                .getSubscriber(serviceId, subscribers_1.GenericSubscriber, null, {
                onConnected: this.subscriberIds[subscriberId].onConnected,
                onDisconnected: this.subscriberIds[subscriberId].onDisconnected,
                onDirectoryChange: this.subscriberIds[subscriberId].onDirectoryChange,
            });
            this.subscriberIds[subscriberId].subscriber = subscriber;
        }
        if (this.subscriberIds[subscriberId].statusProcessor && subscriptions) {
            for (const messageName of subscriptions) {
                this.subscriberIds[subscriberId].subscriber.status[messageName](this.subscriberIds[subscriberId].statusProcessor);
            }
        }
        await this.subscriberIds[subscriberId].subscriber.waitUntilConnected().catch(err => console.error(err));
        return this.subscriberIds[subscriberId].messageProcessorId;
    }
    close(message) {
        const { subscriberId } = message.params;
        this._close(subscriberId);
        return Promise.resolve();
    }
    _close(subscriberId) {
        Log_1.Log.logDebug(`Socket ${this.socketId} Closing ${subscriberId}`);
        const debug = Log_1.Log.getNamespace(`socket:${this.socketId}:wsagent:${subscriberId}`);
        debug(`Closing`);
        const s = this.subscriberIds[subscriberId];
        if (s) {
            const subscriber = s.subscriber;
            if (subscriber) {
                if (s.statusProcessor) {
                    subscriber?.clearStatusProcessor(s.statusProcessor);
                }
                subscriber?.removeConnectedProcessor(s.onConnected);
                subscriber?.removeDisconnectedProcessor(s.onDisconnected);
            }
            delete this.subscriberIds[subscriberId].subscriber;
        }
    }
    closeAll() {
        _.forEach(this.subscriberIds, (val, id) => {
            this._close(id);
        });
    }
    getConfigOverview(message) {
        const { subscriberId } = message.params;
        const subscriber = this.subscriberIds[subscriberId].subscriber;
        return subscriber.getConfigOverview();
    }
    lookupConfig(message) {
        const { lookupId, subscriberId } = message.params;
        const subscriber = this.subscriberIds[subscriberId].subscriber;
        return subscriber.lookupConfig(lookupId);
    }
    static connectSendClose(message) {
        const { serviceId, messageName, params } = message.params;
        return Subscriber_1.Subscriber.connectSendClose(LoginDetails_1.LoginDetails.get(), serviceId, messageName, params);
    }
}
exports.WsSubscriberAgent = WsSubscriberAgent;
WsSubscriberAgent.sessions = {};
WsSubscriberAgent.getAgent = (socket) => {
    if (!WsSubscriberAgent.sessions[socket.id]) {
        WsSubscriberAgent.sessions[socket.id] = new WsSubscriberAgent(socket);
    }
    return WsSubscriberAgent.sessions[socket.id];
};
WsSubscriberAgent.removeAgent = (socket) => {
    if (WsSubscriberAgent.sessions[socket.id]) {
        WsSubscriberAgent.sessions[socket.id].closeAll();
        delete WsSubscriberAgent.sessions[socket.id];
    }
};
