"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WsSubscriberClient = void 0;
const socket_io_client_1 = require("socket.io-client");
const logging_1 = require("../logging");
const SubscriberManager_1 = require("../subscribers/interfaces/SubscriberManager");
const WsSubscriber_1 = require("../subscribers/interfaces/WsSubscriber");
const debug = logging_1.Log.getNamespace("wsSubscriberClient");
class WsSubscriberClient {
    static get() {
        return this.__instance;
    }
    get socket() {
        return this._socket;
    }
    set socket(val) {
        this._socket = val;
    }
    get transport() {
        return this._engine.transport?.name;
    }
    constructor(address) {
        this.socketDisconnctListeners = new Set();
        this.transportListeners = new Set();
        this.onUpgrade = () => {
            debug.extend("engine")("transport", this.transport);
            debug.extend("engine")("upgrade", this.transport);
            for (const processor of this.transportListeners) {
                try {
                    processor(this.transport);
                }
                catch { }
            }
        };
        this.onReconnect = () => {
            debug.extend("socket")("reconnect");
            this.onDisconnect();
            SubscriberManager_1.SubscriberManager.get('wsSubscriber').reconnectAll();
        };
        this.onDisconnect = () => {
            debug.extend("socket")("disconnect");
            for (const processor of this.socketDisconnctListeners) {
                try {
                    processor();
                }
                catch { }
            }
            this.onUpgrade();
        };
        this.onConnect = () => {
            debug.extend("socket")("connect");
            this.onUpgrade();
            SubscriberManager_1.SubscriberManager.get('wsSubscriber').reconnectAll();
        };
        this.onTransportChanged = (cb) => {
            this.transportListeners.add(cb);
            try {
                cb(this.transport);
            }
            catch { }
            return {
                dispose: () => {
                    this.transportListeners.delete(cb);
                }
            };
        };
        WsSubscriberClient.__instance = this;
        if (typeof window !== "undefined") {
            address = address ?? `${window.location.protocol}//${window.location.hostname}:${window.location.port}`;
        }
        if (address) {
            this.socket = module?.hot?.data?.socket ?? (0, socket_io_client_1.io)(address, { timeout: 5000, path: '/socket', });
        }
        else {
            throw new Error(`Address is required when no window is available`);
        }
        SubscriberManager_1.SubscriberManager.get('wsSubscriber', { wsSubscriberInterface: this.getWsSubscriberInterface() });
        WsSubscriber_1.WsSubscriber.setSubscriberInterface(this.getWsSubscriberInterface());
        this.socket.on("connect", this.onConnect);
        this.socket.on("disconnect", this.onDisconnect);
        this.socket.on("reconnect", this.onReconnect);
        this._engine = this.socket.io.engine;
        debug.extend("engine")("transport", this.transport);
        this._engine.once("upgrade", this.onUpgrade);
        if (typeof window !== "undefined") {
            window.__WSCLIENT__ = this;
        }
    }
    getWsSubscriberInterface() {
        const interfaceDebug = debug.extend("interface");
        return ({
            emit: (params) => {
                interfaceDebug.extend("emit")(params);
                return new Promise((resolve, reject) => {
                    this.socket.emit('subscriberBackChannel', params, function (reply) {
                        resolve(reply);
                    });
                });
            },
            sendMessage: (id, message) => {
                interfaceDebug.extend("send")(id, message);
                return new Promise((resolve, reject) => {
                    this.socket.emit(id, message, (reply) => {
                        resolve(reply);
                    });
                });
            },
            registerStatusProcessor: (id, processor) => {
                interfaceDebug.extend("register:status")(id);
                if (!this.socket.listeners(id).includes(processor)) {
                    this.socket.on(id, processor);
                }
            },
            registerSubscriberBackChannelProcessor: (id, processor) => {
                interfaceDebug.extend("register:backchannel")(id);
                id = `${id}_subscriberBackChannel`;
                if (!this.socket.listeners(id).includes(processor)) {
                    this.socket.on(id, processor);
                }
            },
            deregisterStatusProcessor: (id, processor) => {
                interfaceDebug.extend("deregister:status")(id);
                if (this.socket.listeners(id).includes(processor)) {
                    this.socket.off(id, processor);
                }
            },
            deregisterSubscriberBackChannelProcessor: (id, processor) => {
                interfaceDebug.extend("deregister:backchannel")(id);
                id = `${id}_subscriberBackChannel`;
                if (this.socket.listeners(id).includes(processor)) {
                    this.socket.off(id, processor);
                }
            },
            onSocketDisconnect: (processor) => {
                interfaceDebug.extend("register:socketDisconnect")(processor);
                this.socketDisconnctListeners.add(processor);
            },
            offSocketDisconnect: (processor) => {
                interfaceDebug.extend("deregister:socketDisconnect")(processor);
                this.socketDisconnctListeners.delete(processor);
            },
        });
    }
}
exports.WsSubscriberClient = WsSubscriberClient;
