"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScheduleInfoBlock = void 0;
const InfoBlockBuilder_1 = require("../common/InfoBlockBuilder");
const _ = require("lodash");
const uuid = require("uuid");
const schedule_1 = require("../../schedule/schedule");
const primaryEvent_1 = require("../../schedule/primaryEvent");
const source_1 = require("../../schedule/source");
const BDuration_1 = require("../../bTime/BDuration");
const graphicEvent_1 = require("../../schedule/secondaryEvents/graphicEvent");
const subtitle_1 = require("../../schedule/subtitle");
const audioTechnical_1 = require("../../schedule/audioTechnical");
const transition_1 = require("../../schedule/transition");
const recordEvent_1 = require("../../schedule/secondaryEvents/recordEvent");
const secondaryPlayEvent_1 = require("../../schedule/secondaryEvents/secondaryPlayEvent");
const dpiEvent_1 = require("../../schedule/secondaryEvents/dpiEvent");
const asRun_1 = require("../../schedule/asRun");
const OptOut_1 = require("../../schedule/OptOut");
const SCHEDULE_INFO_TYPE = 'schedule';
class ScheduleInfoBlock extends InfoBlockBuilder_1.InfoBlockBuilder {
    constructor(schedule, jsonInfo, jobBuilder) {
        super({ ...jsonInfo, infoType: SCHEDULE_INFO_TYPE }, jobBuilder);
        this.schedule = schedule;
    }
    version(version, date, options) {
        const schedule = this.getSchedule();
        if (!schedule.versions) {
            schedule.versions = [];
        }
        schedule.versions.push(new schedule_1.Version(version, date, options));
        schedule.versions = _.sortBy(schedule.versions, (version) => { return version.date.toIsoString(); });
        return this;
    }
    setEndTime() {
        const schedule = this.getSchedule();
        const playEvents = _.filter(schedule.events, (event) => {
            return _.has(event, 'startDate') && (event.eventType !== "comment");
        });
        const lastEvent = _.last(playEvents);
        if (lastEvent) {
            const duration = lastEvent.duration || BDuration_1.BDuration.zero(lastEvent.startDate.getFrameRate());
            schedule.end = lastEvent.startDate.addDuration(duration);
        }
        return this;
    }
    timezoneOffset(timezone) {
        const schedule = this.getSchedule();
        schedule.timezoneOffset = timezone;
        return this;
    }
    commentEvent(comment, options) {
        const schedule = this.getSchedule();
        const commentEvent = new primaryEvent_1.CommentEvent(this.getId(options), comment, options);
        commentEvent.comment = comment;
        commentEvent.id = this.getId(options);
        schedule.events.push(commentEvent);
        return this;
    }
    makeOffsetDateTime(dateTime) {
        if (this.schedule.timezoneOffset) {
            return dateTime.addDuration(this.schedule.timezoneOffset.getNegativeDuration());
        }
        else {
            return dateTime;
        }
    }
    fixedEvent(startDateTime, options) {
        const schedule = this.getSchedule();
        const playEvent = new primaryEvent_1.PlayEvent(this.getId(options), 'fixed', null);
        playEvent.startDate = this.makeOffsetDateTime(startDateTime);
        playEvent.duration = options ? options.duration : undefined;
        playEvent.endMode = options ? options.endMode : undefined;
        schedule.events.push(playEvent);
        return this;
    }
    manualEvent(options) {
        const schedule = this.getSchedule();
        const playEvent = new primaryEvent_1.PlayEvent(this.getId(options), 'manual', null);
        playEvent.duration = options ? options.duration : undefined;
        if (options && options.startDateTime) {
            playEvent.startDate = this.makeOffsetDateTime(options.startDateTime);
        }
        playEvent.endMode = options ? options.endMode : undefined;
        schedule.events.push(playEvent);
        return this;
    }
    followEvent(options) {
        const schedule = this.getSchedule();
        const playEvent = new primaryEvent_1.PlayEvent(this.getId(options), 'follow', null);
        playEvent.duration = options ? options.duration : undefined;
        if (options && options.startDateTime) {
            playEvent.startDate = this.makeOffsetDateTime(options.startDateTime);
        }
        playEvent.endMode = options ? options.endMode : undefined;
        schedule.events.push(playEvent);
        return this;
    }
    mediaSource(materialId, options) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            const source = new source_1.MediaSource('video', materialId, options ? options.som : undefined);
            currentEvent.source = source;
        }
        else {
            console.error("Make a play event before adding a media source");
        }
        return this;
    }
    routerSource(source) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            currentEvent.source = new source_1.RouterSource(source);
            ;
        }
        else {
            console.error("Make a play event before adding a router source");
        }
        return this;
    }
    aspectRatio(aspectRatio) {
        return this.updateField('aspectRatio', aspectRatio);
    }
    subtitle(language, options) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            if (!currentEvent.subtitles) {
                currentEvent.subtitles = [];
            }
            currentEvent.subtitles.push(new subtitle_1.Subtitle(language, options));
        }
        return this;
    }
    enabled(enabled) {
        return this.updateField('enabled', enabled);
    }
    title(title) {
        return this.updateField('title', title);
    }
    id(id) {
        return this.updateField('id', id);
    }
    comment(comment, primary) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent) {
            // add to the last inserted secondary if present
            if (currentEvent instanceof primaryEvent_1.PlayEvent) {
                if (!primary && currentEvent.secondaryEvents && currentEvent.secondaryEvents.length > 0) {
                    const currentSecondary = _.last(currentEvent.secondaryEvents);
                    currentSecondary.comment = comment;
                }
                else {
                    // else add comment to main event
                    currentEvent.comment = comment;
                }
            }
            else {
                // else add comment to main event
                currentEvent.comment = comment;
            }
        }
        else {
            console.error("Make an event before adding a comment");
        }
        return this;
    }
    audioOutput(output) {
        return this.setAudioTechnicalField('setOutput', output);
    }
    duckEnable(duckEnabled) {
        return this.setAudioTechnicalField('setDuckEnable', duckEnabled);
    }
    optOut(breakId) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            if (!currentEvent.optOut) {
                currentEvent.optOut = new OptOut_1.OptOut(breakId);
            }
        }
        return this;
    }
    setAudioTechnicalField(field, value) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            if (!currentEvent.audioTechnical) {
                currentEvent.audioTechnical = new audioTechnical_1.AudioTechnical();
            }
            currentEvent.audioTechnical[field](value);
        }
        return this;
    }
    videoTransition(type, options) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            if (!currentEvent.transition) {
                currentEvent.transition = {};
            }
            currentEvent.transition.video = new transition_1.VideoTransition(type, options);
        }
        return this;
    }
    cg(page, fields, options) {
        const graphic = new graphicEvent_1.GraphicEvent(this.getId(), options.startMode, page, fields, options);
        this.addSecondary(graphic);
        return this;
    }
    splice(eventId, options) {
        const dpi = new dpiEvent_1.DpiEvent(this.getId(), options.startMode, eventId, options);
        this.addSecondary(dpi);
        return this;
    }
    secondaryPlay(materialId, options) {
        const secondaryPlayEvent = new secondaryPlayEvent_1.SecondaryPlayEvent(this.getId(), options.startMode, materialId, options);
        this.addSecondary(secondaryPlayEvent);
        return this;
    }
    record(materialId, options) {
        const recordEvent = new recordEvent_1.RecordEvent(this.getId(), options.startMode, materialId, options);
        this.addSecondary(recordEvent);
        return this;
    }
    reconcileId(reconcileId) {
        return this.updateField('reconcileId', reconcileId);
    }
    eventId(eventId) {
        return this.updateField('eventId', eventId);
    }
    eventCategory(eventCategory) {
        return this.updateField('eventCategory', eventCategory);
    }
    custom(field, value, type, primary) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            // add to the last inserted secondary if present
            if (currentEvent.secondaryEvents && currentEvent.secondaryEvents.length > 0 && !primary) {
                const currentSecondary = _.last(currentEvent.secondaryEvents);
                currentSecondary.custom = _.merge({}, currentSecondary.custom, { [field]: { value, type } });
            }
            else {
                // else add custom to main event
                currentEvent.custom = _.merge({}, currentEvent.custom, { [field]: { value, type } });
            }
        }
        else {
            console.error("Make a play event before adding a media source");
        }
        return this;
    }
    asRun(result, options) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent || currentEvent instanceof primaryEvent_1.CommentEvent) {
            const asRun = new asRun_1.AsRun(result, options);
            currentEvent.asRun = asRun;
        }
        return this;
    }
    asRunJob(result, options) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            if (!currentEvent.asRunJobs) {
                currentEvent.asRunJobs = [];
            }
            currentEvent.asRunJobs.push(new asRun_1.AsRunJob(result, options));
        }
        return this;
    }
    addSecondary(secondary) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent instanceof primaryEvent_1.PlayEvent) {
            if (!currentEvent.secondaryEvents) {
                currentEvent.secondaryEvents = [];
            }
            currentEvent.secondaryEvents.push(secondary);
        }
        return this;
    }
    updateField(field, value) {
        const schedule = this.getSchedule();
        const currentEvent = _.last(schedule.events);
        if (currentEvent) {
            currentEvent[field] = value;
        }
        else {
            console.error("Make an event before updating " + field);
        }
        return this;
    }
    getId(options) {
        if (_.has(options, 'id')) {
            return options.id;
        }
        else {
            return uuid.v4();
        }
    }
    getSchedule() {
        return this.schedule;
    }
    toJson() {
        return this.schedule.toJson();
    }
}
exports.ScheduleInfoBlock = ScheduleInfoBlock;
if (!InfoBlockBuilder_1.InfoTypeLookup[SCHEDULE_INFO_TYPE]) {
    InfoBlockBuilder_1.InfoTypeLookup[SCHEDULE_INFO_TYPE] = {};
}
InfoBlockBuilder_1.InfoTypeLookup[SCHEDULE_INFO_TYPE].builder = ScheduleInfoBlock;
