"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarinaNormaliser = void 0;
const normaliser_1 = require("./normaliser");
const _ = require("lodash");
const luxon = require("luxon");
const xml2js_1 = require("xml2js");
const schedule_1 = require("../schedule/schedule");
const Time_1 = require("../time/Time");
const ScheduleInfoBlock_1 = require("../infoBlocks/schedule/ScheduleInfoBlock");
const BDateTime_1 = require("../bTime/BDateTime");
const FrameRate_1 = require("../bTime/FrameRate");
const BDuration_1 = require("../bTime/BDuration");
const tcFormat = Time_1.TcFormat.NDF25;
const frameRate = new FrameRate_1.FrameRate("ndf25");
const STARTMODE_LOOKUP = {
    'Sequential': 'follow',
    'Fixed': 'fixed',
    'Manual': 'manual'
};
const ENDMODE_LOOKUP = {
    'Duration': 'duration',
    'Hold': 'hold'
};
const SECONDARY_STARTMODE_LOOKUP = {
    '+ParentStart': 'offsetFromParentStart',
    '-ParentStart': 'offsetFromParentStart',
    '+ParentEnd': 'offsetFromParentEnd',
    '-ParentEnd': 'offsetFromParentEnd'
};
const SECONDARY_ENDMODE_LOOKUP = {
    '+ParentStart': 'offsetFromParentStart',
    '-ParentStart': 'offsetFromParentStart',
    '+ParentEnd': 'offsetFromParentEnd',
    '-ParentEnd': 'offsetFromParentEnd',
    'Duration': 'duration',
    'Hold': 'hold'
};
const DURATION_LOOKUP = {
    'Fast': BDuration_1.BDuration.fromTcString("00:00:00:00", frameRate),
    'Medium': BDuration_1.BDuration.fromTcString("00:00:01:00", frameRate),
    'Slow': BDuration_1.BDuration.fromTcString("00:00:02:00", frameRate)
};
class MarinaNormaliser extends normaliser_1.Normaliser {
    constructor() {
        super(...arguments);
        this.spliceLookup = {
            'splice_start_normal': 'spliceStartNormal',
            'splice_start_immediate': 'spliceStartImmediate',
            'splice_end_normal': 'spliceEndNormal',
            'splice_end_immediate': 'spliceEndImmediate'
        };
    }
    getId() {
        return 'marina';
    }
    getDescription() {
        return 'Reads info from a Marina MPL schedule';
    }
    getAsBuffer() {
        return false;
    }
    getConfigForm() {
        return {
            schema: {
                "type": "object",
                "properties": {
                    "frameRate": {
                        "type": "string",
                        "title": "Frame rate",
                        "enum": [
                            "ndf25",
                            "df30"
                        ],
                        "enumNames": [
                            "ndf25",
                            "df30"
                        ],
                        "default": "ndf25"
                    },
                }
            }
        };
    }
    getVersions(history) {
        const created = _.has(history, 'created[0].$') ?
            _.merge({ version: 'created' }, _.get(history, 'created[0].$')) : null;
        const modified = _.has(history, 'modified[0].$') ?
            _.merge({ version: 'created' }, _.get(history, 'modified[0].$')) : null;
        const version = _([created, modified])
            .filter()
            .map(v => {
            return {
                author: v.author,
                version: v.comment || (created ? "initial version" : undefined),
                date: BDateTime_1.BDateTime.fromIso(v.time, frameRate)
            };
        })
            .value();
        return version;
    }
    addComment(event, scheduleBuilder) {
        if (_.has(event, 'properties[0].event[0].comment[0]')) {
            scheduleBuilder.commentEvent(_.get(event, 'properties[0].event[0].comment[0]'));
        }
        else if (_.has(event, 'properties[0].event[0]')) {
            scheduleBuilder.commentEvent('');
        }
    }
    addCommentMarker(event, scheduleBuilder) {
        if (_.has(event, 'properties[0].event[0].comment[0]')) {
            scheduleBuilder.commentEvent(_.get(event, 'properties[0].event[0].comment[0]'), {
                custom: {
                    eventType: "marker"
                }
            });
        }
    }
    readMarinaTime(time) {
        const timeParts = time.split('T');
        if (timeParts.length === 2) {
            const startDate = timeParts[0];
            const dateTime = luxon.DateTime.fromISO(startDate);
            // convert hh:mm:ss:ff into the equivalent number of milliseconds and add to the start date
            const luxonStartTime = dateTime.plus({ milliseconds: Time_1.Position.PositionFromTcString(tcFormat, timeParts[1]).getAsMilliseconds() });
            return BDateTime_1.BDateTime.fromIso(luxonStartTime.toISO({ includeOffset: false }), this.frameRate);
        }
        else if (timeParts.length === 1) {
            const startDate = timeParts[0];
            const dateTime = luxon.DateTime.fromISO(startDate);
            return BDateTime_1.BDateTime.fromIso(dateTime.toISO(), this.frameRate);
        }
    }
    addScheduleEvent(event, scheduleBuilder) {
        if (_.has(event, 'properties[0]')) {
            const videoDetails = _.get(event, 'properties[0]');
            if (_.has(videoDetails, 'schedule[0].$')) {
                const scheduleTiming = videoDetails.schedule[0].$;
                let duration;
                if (_.has(scheduleTiming, 'endOffset')) {
                    duration = BDuration_1.BDuration.fromTcString(scheduleTiming.endOffset, this.frameRate);
                }
                const startMode = STARTMODE_LOOKUP[scheduleTiming.startType];
                let startDateTime;
                if (_.has(event, 'asRun[0].$.startTime')) {
                    startDateTime = this.readMarinaTime(_.get(event, 'asRun[0].$.startTime'));
                }
                else if (scheduleTiming.startOffset) {
                    startDateTime = this.readMarinaTime(scheduleTiming.startOffset);
                }
                const endMode = ENDMODE_LOOKUP[scheduleTiming.endType];
                switch (startMode) {
                    case "fixed":
                        {
                            scheduleBuilder.fixedEvent(startDateTime, { duration, endMode });
                        }
                        break;
                    case "manual":
                        {
                            scheduleBuilder.manualEvent({ duration, startDateTime, endMode });
                        }
                        break;
                    case "follow":
                        {
                            scheduleBuilder.followEvent({ duration, startDateTime, endMode });
                        }
                        break;
                }
            }
            else {
                scheduleBuilder.followEvent();
            }
        }
    }
    addTransition(event, scheduleBuilder) {
        if (_.has(event, 'properties[0].switch[0]')) {
            const videoDetails = _.get(event, 'properties[0]');
            const transition = videoDetails.switch[0];
            const duration = DURATION_LOOKUP[transition.$.rate];
            scheduleBuilder.videoTransition(transition.$.transition, { duration });
        }
    }
    getSecondaryTiming(event) {
        const scheduleTime = event.schedule[0];
        const options = {};
        if (scheduleTime) {
            if (scheduleTime.$.startType) {
                options.startMode = SECONDARY_STARTMODE_LOOKUP[scheduleTime.$.startType];
            }
            if (scheduleTime.$.startOffset) {
                options.startOffset = BDuration_1.BDuration.fromTcString(scheduleTime.$.startOffset, frameRate);
                if (scheduleTime.$.startType && scheduleTime.$.startType.startsWith("-")) {
                    options.startOffset = options.startOffset.getNegativeDuration();
                }
            }
            if (scheduleTime.$.endType) {
                options.endMode = SECONDARY_ENDMODE_LOOKUP[scheduleTime.$.endType];
            }
            if (scheduleTime.$.endOffset) {
                options.endOffset = BDuration_1.BDuration.fromTcString(scheduleTime.$.endOffset, frameRate);
                if (scheduleTime.$.endType && scheduleTime.$.endType.startsWith("-")) {
                    options.endOffset = options.endOffset.getNegativeDuration();
                }
            }
        }
        return options;
    }
    getSecondaryOptions(event) {
        return {
            comment: _.get(event, 'event[0].comment[0]', undefined)
        };
    }
    addCgEvent(event, scheduleBuild, overrideEventType) {
        const cgEvent = event.properties[0];
        const timingOptions = this.getSecondaryTiming(cgEvent);
        const secondaryOptions = this.getSecondaryOptions(cgEvent);
        const page = _.get(cgEvent, 'media[0].$.mediaName');
        let fields;
        if (_.has(cgEvent, 'mediaStream[0].cg[0].f')) {
            fields = _.map(cgEvent.mediaStream[0].cg[0].f, (field) => {
                return { name: field.$.name, value: field._ };
            });
        }
        scheduleBuild.cg(page, fields, _.merge(secondaryOptions, timingOptions, { overrideEventType }));
    }
    addRecordEvent(event, scheduleBuild) {
        const secRecordEvent = event.properties[0];
        const timingOptions = this.getSecondaryTiming(secRecordEvent);
        if (_.has(secRecordEvent, 'media[0].$.mediaName')) {
            scheduleBuild.record(_.get(secRecordEvent, 'media[0].$.mediaName'), timingOptions);
        }
    }
    addScteEvent(event, scheduleBuild) {
        const scteEvent = event.properties[0];
        const timingOptions = this.getSecondaryTiming(scteEvent);
        const auxData = scteEvent.auxData[0].$;
        if (auxData.startValue) {
            const dpiOptions = {
                command: this.spliceLookup[auxData.startValue],
                uniqueProgramId: auxData.uniqueProgramId,
                dpiDuration: auxData.duration,
            };
            scheduleBuild.splice(auxData.startSpliceEventId, _.merge(timingOptions, dpiOptions));
            this.addAuxData(event, scheduleBuild);
        }
        if (auxData.endValue) {
            const dpiOptions = {
                command: this.spliceLookup[auxData.endValue],
                uniqueProgramId: auxData.uniqueProgramId,
                dpiDuration: auxData.duration,
            };
            scheduleBuild.splice(auxData.endSpliceEventId, _.merge(timingOptions, dpiOptions));
            this.addAuxData(event, scheduleBuild);
        }
    }
    addChildEvents(event, scheduleBuilder) {
        if (_.has(event, 'childEvents[0].event')) {
            for (const child of event.childEvents[0].event) {
                switch (child.$.type) {
                    case "CG":
                        {
                            this.addCgEvent(child, scheduleBuilder);
                        }
                        break;
                    case "Vertigo":
                        {
                            this.addCgEvent(child, scheduleBuilder);
                        }
                        break;
                    case "Logo":
                        {
                            this.addCgEvent(child, scheduleBuilder, "Logo");
                        }
                        break;
                    case "SecRecordHD":
                        {
                            this.addRecordEvent(child, scheduleBuilder);
                        }
                        break;
                    case "SCTE":
                        {
                            this.addScteEvent(child, scheduleBuilder);
                        }
                        break;
                }
                // Aux data
                this.addAuxData(child, scheduleBuilder);
                // Comments
                this.addComments(child, scheduleBuilder);
                // As Run
                const asRun = _.get(event, 'asRun[0]');
                if (_.has(asRun, "$")) {
                    _.each(asRun.$, (prop, key) => {
                        scheduleBuilder.custom(key, prop);
                    });
                }
            }
        }
        if (_.has(event, 'childEvents[0].action')) {
            for (const child of event.childEvents[0].action) {
                switch (child.$.type) {
                    case "SCTE":
                        {
                            this.addScteEvent(child, scheduleBuilder);
                        }
                        break;
                }
            }
        }
    }
    getFeature(features, type, path = '') {
        const foundFeature = _.find(features, feature => {
            return feature.$.type === type;
        });
        if (foundFeature) {
            return _.get(foundFeature, 'properties[0]' + path);
        }
    }
    addSource(event, scheduleBuilder) {
        if (event.$.type === 'PrimaryVideo') {
            const videoDetails = _.get(event, 'properties[0]');
            if (_.has(videoDetails, 'media[0].$')) {
                const materialId = videoDetails.media[0].$.mediaName;
                let som;
                if (_.has(videoDetails, 'mediaStream[0].$.som')) {
                    som = BDateTime_1.BDateTime.fromTimecode(videoDetails.mediaStream[0].$.som, this.frameRate);
                }
                scheduleBuilder.mediaSource(materialId, { som });
            }
            else {
                scheduleBuilder.mediaSource(null);
            }
            if (_.has(videoDetails, 'mediaStream[0].segment[0]')) {
                scheduleBuilder.custom('segment', videoDetails.mediaStream[0].segment[0]);
            }
            if (_.has(videoDetails, 'mediaStream[0].allocation[0]')) {
                scheduleBuilder.custom('segment', videoDetails.mediaStream[0].allocation[0]);
            }
            if (_.has(videoDetails, 'mediaStream[0].video[0]')) {
                scheduleBuilder.custom('videoMediaStream', videoDetails.mediaStream[0].video[0]);
            }
        }
        else if (event.$.type === 'Live') {
            const liveDetails = _.get(event, 'properties[0]');
            const source = _.get(liveDetails, 'switch[0].source[0].logical[0].$.name');
            if (source) {
                scheduleBuilder.routerSource(source);
            }
        }
    }
    addAuxData(event, scheduleBuilder) {
        const details = _.get(event, 'properties[0]');
        if (details) {
            if (_.has(details, 'auxData[0]')) {
                _.each(details.auxData[0].$, (value, key) => {
                    scheduleBuilder.custom(key, value);
                });
            }
        }
    }
    addComments(event, scheduleBuilder) {
        const details = _.get(event, 'properties[0]');
        if (_.has(details, 'event[0].comment[0]')) {
            scheduleBuilder.comment(_.get(details, 'event[0].comment[0]'));
        }
    }
    addVideoEvent(event, scheduleBuilder) {
        // Timing
        this.addScheduleEvent(event, scheduleBuilder);
        // Transition
        this.addTransition(event, scheduleBuilder);
        // Source
        this.addSource(event, scheduleBuilder);
        // Aux data
        this.addAuxData(event, scheduleBuilder);
        // Child events
        this.addChildEvents(event, scheduleBuilder);
        const videoDetails = _.get(event, 'properties[0]');
        // Features
        if (_.has(videoDetails, 'features[0].feature')) {
            // Aspect Ratio
            const arc = this.getFeature(videoDetails.features[0].feature, "ARC", '.macro[0].$.value');
            if (arc) {
                scheduleBuilder.aspectRatio(arc);
            }
            // Subtitle
            const subtitle = this.getFeature(videoDetails.features[0].feature, "Subtitle");
            if (subtitle && _.has(subtitle, 'mediaStream[0].subtitle[0].languages')) {
                const filename = _.get(subtitle, 'media[0].$.mediaName');
                let captionMode;
                const incomingCaptionMode = _.get(subtitle, 'mediaStream[0].subtitle[0].$.captionMode');
                if (incomingCaptionMode) {
                    captionMode = _.lowerCase(incomingCaptionMode) === 'open' ? 'open' : 'closed';
                }
                const setting = _.get(subtitle, 'mediaStream[0].subtitle[0].$.setting');
                if (_.has(subtitle, 'mediaStream[0].subtitle[0].languages[0].lang')) {
                    for (const language of subtitle.mediaStream[0].subtitle[0].languages[0].lang) {
                        scheduleBuilder.subtitle(language, { filename, captionMode, setting });
                    }
                }
            }
            // Audio
            const audioOutput = this.getFeature(videoDetails.features[0].feature, "AudioSwitch", '.macro[0].$.value');
            if (audioOutput) {
                scheduleBuilder.audioOutput(audioOutput);
            }
            const duckEnable = this.getFeature(videoDetails.features[0].feature, "AudioDuck", '.macro[0].$.value');
            if (duckEnable) {
                scheduleBuilder.duckEnable(duckEnable === 'on');
            }
            // Opt out
            const optOut = this.getFeature(videoDetails.features[0].feature, "LocalOptOut");
            if (optOut) {
                const breakId = _.get(optOut, 'media[0].$.mediaName');
                scheduleBuilder.optOut(breakId);
            }
            // Eplay
            const eplay = this.getFeature(videoDetails.features[0].feature, "EPlay");
            if (eplay) {
                const playId = _.get(eplay, 'media[0].$.mediaName');
                scheduleBuilder.secondaryPlay(playId, {});
            }
            const audioSwitchSweCom = this.getFeature(videoDetails.features[0].feature, "AudioSwitchSweCom", '.macro[0].$.value');
            if (audioSwitchSweCom) {
                scheduleBuilder.custom("audioSwitchSweCom", audioSwitchSweCom, null, true);
            }
        }
    }
    addGenericMetadata(event, scheduleBuilder) {
        const baseEvent = _.get(event, 'properties[0]');
        if (_.has(baseEvent, 'event[0].$')) {
            const metadata = baseEvent.event[0];
            scheduleBuilder
                .enabled(event.$.enabled)
                .title(metadata.$.title)
                .reconcileId(metadata.$.reconcileKey)
                .comment(metadata.comment ? metadata.comment[0] : undefined, true)
                .eventId(metadata.$.houseId);
            for (const attribute of _.keys(_.omit(event.$, 'enabled'))) {
                scheduleBuilder.custom(attribute, event.$[attribute], null, true);
            }
            if (_.has(metadata, 'classifications[0].classification')) {
                for (const classification of metadata.classifications[0].classification) {
                    if (classification.$.classification === 'EventType') {
                        scheduleBuilder.eventCategory(classification.$.category);
                    }
                    else {
                        scheduleBuilder.custom(classification.$.classification, classification.$.category, 'classification', true);
                    }
                }
            }
        }
    }
    addAsRunInfo(event, scheduleBuilder) {
        const asRun = event.asRun[0];
        scheduleBuilder.asRun(asRun.$.result, {
            startTime: asRun.$.startTime ? this.readMarinaTime(asRun.$.startTime) : undefined,
            endTime: asRun.$.endTime ? this.readMarinaTime(asRun.$.endTime) : undefined,
            duration: asRun.$.duration ? BDuration_1.BDuration.fromTcString(asRun.$.duration, frameRate) : undefined,
            custom: _.omit(asRun.$, ['startTime', 'endTime', 'duration'])
        });
        if (_.has(event, 'asRun[0].jobs[0].job')) {
            for (const asRunJob of event.asRun[0].jobs[0].job) {
                scheduleBuilder.asRunJob(asRunJob.asRun[0].$.result, { custom: { details: asRunJob } });
            }
        }
    }
    makeEventList(events, scheduleBuilder) {
        return _.map(events, (event, key) => {
            let found = true;
            switch (event.$.type) {
                case 'Comment':
                    this.addComment(event, scheduleBuilder);
                    break;
                case 'Marker':
                    this.addCommentMarker(event, scheduleBuilder);
                    break;
                case 'PrimaryVideo':
                    this.addVideoEvent(event, scheduleBuilder);
                    break;
                case 'Live':
                    this.addVideoEvent(event, scheduleBuilder);
                    break;
                default:
                    if (_.startsWith(event.$.type, 'Comment')) {
                        this.addComment(event, scheduleBuilder);
                    }
                    else {
                        found = false;
                    }
            }
            if (found) {
                if (_.has(event, 'asRun')) {
                    this.addAsRunInfo(event, scheduleBuilder);
                }
                this.addGenericMetadata(event, scheduleBuilder);
            }
        });
    }
    buildSchedule(schedule, playlist, incomingSchedule) {
        const versions = _.has(playlist, 'marinaPlaylist.properties[0].history[0]') ? this.getVersions(playlist.marinaPlaylist.properties[0].history[0]) : undefined;
        const scheduleBuilder = new ScheduleInfoBlock_1.ScheduleInfoBlock(schedule);
        if (_.has(playlist, 'marinaPlaylist.properties[0].channel[0].$.name')) {
            schedule.channel = _.get(playlist, 'marinaPlaylist.properties[0].channel[0].$.name');
        }
        if (_.has(playlist, 'marinaPlaylist.properties[0].channel[0].$.txTime')) {
            const txTime = _.get(playlist, 'marinaPlaylist.properties[0].channel[0].$.txTime');
            const timePart = BDuration_1.BDuration.fromTcString(txTime.substr(11), frameRate);
            const datePart = BDateTime_1.BDateTime.fromIso(txTime.substr(0, 10), frameRate);
            schedule.start = datePart.addDuration(timePart);
        }
        if (_.has(playlist, 'marinaPlaylist.properties[0].channel[0].$.timeOffsetValue')) {
            scheduleBuilder.timezoneOffset(BDuration_1.BDuration.fromTcString(_.get(playlist, 'marinaPlaylist.properties[0].channel[0].$.timeOffsetValue'), frameRate));
        }
        else if (_.has(playlist, 'marinaPlaylist.properties[0].channel[0].$.timeOffset')) {
            const timeOffset = _.get(playlist, 'marinaPlaylist.properties[0].channel[0].$.timeOffset');
            const utcOffset = timeOffset.match("UTC(.*)");
            if (utcOffset) {
                scheduleBuilder.timezoneOffset(BDuration_1.BDuration.zero(frameRate).addHours(parseInt(utcOffset[1])));
            }
        }
        this.makeEventList(playlist.marinaPlaylist.eventList[0].event, scheduleBuilder);
        scheduleBuilder.setEndTime();
        if (versions) {
            for (const version of versions) {
                scheduleBuilder.version(version.version, version.date, { author: version.author });
            }
        }
        return scheduleBuilder.getSchedule();
    }
    buildAsRun(schedule, asRun) {
        const scheduleBuilder = new ScheduleInfoBlock_1.ScheduleInfoBlock(schedule);
        schedule.channel = _.get(asRun, 'marinaAsRun.source[0].$.channelName');
        schedule.start = this.readMarinaTime(_.get(asRun, 'marinaAsRun.$.startTime'));
        schedule.end = this.readMarinaTime(_.get(asRun, 'marinaAsRun.$.endTime'));
        this.makeEventList(asRun.marinaAsRun.eventList[0].event, scheduleBuilder);
        return scheduleBuilder.getSchedule();
    }
    internalTranslate(data, options, incomingSchedule) {
        this.options = options;
        this.frameRate = new FrameRate_1.FrameRate(options.frameRate.toLowerCase());
        return new Promise((resolve, reject) => {
            (0, xml2js_1.parseString)(data, (err, result) => {
                if (err) {
                    reject(err);
                }
                else {
                    try {
                        let schedule;
                        if (incomingSchedule) {
                            schedule = incomingSchedule;
                        }
                        else {
                            schedule = new schedule_1.Schedule("channel", [], "name", "schedule");
                        }
                        if (_.has(result, 'marinaPlaylist')) {
                            resolve(this.buildSchedule(schedule, result, incomingSchedule));
                        }
                        else if (_.has(result, 'marinaAsRun')) {
                            resolve(this.buildAsRun(schedule, result));
                        }
                        else if (_.has(result, 'event')) {
                            const asRunEvent = result["event"];
                            const marinaAsRun = {
                                marinaAsRun: {
                                    $: {
                                        startTime: asRunEvent.asRun[0].$.startTime,
                                        endTime: asRunEvent.asRun[0].$.endTime
                                    },
                                    eventList: [
                                        { event: [asRunEvent] }
                                    ]
                                }
                            };
                            resolve(this.buildAsRun(schedule, marinaAsRun));
                        }
                        else {
                            reject(new Error("Invalid XML"));
                        }
                    }
                    catch (e) {
                        console.error(e.message, e.stack);
                        reject(e);
                    }
                }
            });
        });
    }
}
exports.MarinaNormaliser = MarinaNormaliser;
async function test() {
    const normaliser = new MarinaNormaliser();
    const schedule = await normaliser.internalTranslate(`<?xml version="1.0" encoding="UTF-8"?>
<marinaPlaylist version="3.1">
  <properties>
    <history source="version 1.0">
      <created author="LOUISE" time="2022-05-26T13:42:38Z"/>
    </history>
    <channel name="Web_6" txTime="2022-05-27T05:00:00:00" timeOffset="UTC+2" timeOffsetValue="02:00:00:00"/>
  </properties>
  <eventList>
    <event type="Marker">
      <properties>
        <event>
          <comment>Begin Handover</comment>
        </event>
      </properties>
    </event>
    <event type="Comment">
      <properties>
        <event title="Handover">
          <comment>Missing media: No</comment>
        </event>
      </properties>
    </event>
    <event type="Marker">
      <properties>
        <event>
          <comment>End Handover</comment>
        </event>
      </properties>
    </event>
  </eventList>
</marinaPlaylist>

    
    `, { frameRate: "ndf25" }, null);
    console.log(schedule);
}
//test();
