"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.morpheusSecondScreenNormaliser = void 0;
const normaliser_1 = require("./normaliser");
const _ = require("lodash");
const luxon = require("luxon");
const xml2js_1 = require("xml2js");
class morpheusSecondScreenNormaliser extends normaliser_1.Normaliser {
    constructor() {
        super(...arguments);
        this.MONTHS = { 'JAN': 1, 'FEB': 2, 'MAR': 3, 'APR': 4, 'MAY': 5, 'JUN': 6, 'JUL': 7, 'AUG': 8, 'SEP': 9, 'OCT': 10, 'NOV': 11, 'DEC': 12 };
    }
    getId() {
        return 'morpheusSecondScreen';
    }
    getDescription() {
        return 'Reads info from a Morpheus Second Screen server';
    }
    getAsBuffer() {
        return false;
    }
    getConfigForm() {
        return {
            schema: {
                "type": "object",
                "properties": {
                    "frameRate": {
                        "type": "string",
                        "title": "Frame rate",
                        "enum": [
                            "NDF25",
                            "DF30"
                        ],
                        "enumNames": [
                            "NDF25",
                            "DF30"
                        ],
                        "default": "NDF25"
                    },
                }
            }
        };
    }
    lookupMillisPerFrame(frameRate) {
        let result = 40;
        if (frameRate === 'NDF25') {
            result = 40;
        }
        else if (frameRate === 'DF30') {
            result = 33;
        }
        return result;
    }
    mapSecondScreenValue(value, key, options) {
        if (_.isArray(value)) {
            value = value[0];
        }
        if (key === 'EventOid') {
            return value.replace(/[{}]/g, '');
        }
        else if (key === 'EventType') {
            return value.toLowerCase();
        }
        else if (key === 'StartTime') {
            if (value.match(/^(([0-9])|([0-2][0-9])|([3][0-1]))-(JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC)-\d{4} \d{2}:\d{2}:\d{2}:\d{2}$/)) {
                return luxon.DateTime.fromObject({
                    year: +value.substr(7, 4),
                    month: this.MONTHS[value.substr(3, 3)],
                    day: +value.substr(0, 2),
                    hour: +value.substr(12, 2),
                    minute: +value.substr(15, 2),
                    second: +value.substr(18, 2),
                    millisecond: +value.substr(21, 2) * this.lookupMillisPerFrame(options.frameRate)
                }).toISO({ includeOffset: false });
            }
            else {
                return value;
            }
        }
        else if (key === 'xsi:type') {
            return _.camelCase(value);
            // } else if (value.match(/^(([0-9])|([0-2][0-9])|([3][0-1]))-(JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC)-\d{4} \d{2}:\d{2}:\d{2}:\d{2}$/)) {
            //
            //     return luxon.DateTime.fromObject({
            //         year: +value.substr(7, 4),
            //         month: this.MONTHS[value.substr(3,3)],
            //         day: +value.substr(0, 2),
            //         hour: +value.substr(12, 2),
            //         minute: +value.substr(15, 2),
            //         second: +value.substr(18, 2),
            //         millisecond: +value.substr(21, 2) * this.lookupMillisPerFrame(options.frameRate)
            //     }).toISO({ includeOffset: false })
        }
        else {
            return value;
        }
    }
    mapSecondScreenKey(value, key, options) {
        if (key === 'Type') {
            return 'eventType';
        }
        else if (key === 'xsi:type') {
            return 'objectType';
        }
        else {
            return _.camelCase(key);
        }
    }
    internalTranslate(data, options) {
        return new Promise((resolve, reject) => {
            (0, xml2js_1.parseString)(data, (err, result) => {
                if (err) {
                    reject(err);
                }
                else {
                    const secondScreenEvents = result.EventsMessage.SecondScreenEvents[0].SecondScreenEvent;
                    const eventsMessage = _.omit(_.mapValues(result.EventsMessage, (value, key) => {
                        if ((key !== '$') && (key !== 'SecondScreenEvents')) {
                            return value[0];
                        }
                    }), 'SecondScreenEvents');
                    const attributesFlattened = _.map(secondScreenEvents, (event) => {
                        return _.omit(_.extend(event, event['$'], eventsMessage), '$');
                    });
                    const flattenedSecondScreenEvents = _.map(attributesFlattened, (event) => {
                        return _.chain(event)
                            .mapValues((value, key) => {
                            return this.mapSecondScreenValue(value, key, options);
                        })
                            .mapKeys((value, key) => {
                            return this.mapSecondScreenKey(value, key, options);
                        })
                            .value();
                    });
                    let inBreakContents = null;
                    const groupedByBreaks = _.reduce(flattenedSecondScreenEvents, (result, value, key) => {
                        if (value.objectType === 'breakHeaderEvent') {
                            inBreakContents = [];
                            value.breakContents = inBreakContents;
                            result.push(value);
                        }
                        else if (value.objectType === 'infoEvent' && inBreakContents) {
                            inBreakContents.push(value);
                        }
                        else {
                            result.push(value);
                        }
                        return result;
                    }, []);
                    resolve(groupedByBreaks);
                }
            });
        });
    }
}
exports.morpheusSecondScreenNormaliser = morpheusSecondScreenNormaliser;
//
// new morpheusSecondScreenNormaliser().translate(`<EventsMessage xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
// 	<SystemTimeUtc>03-JUL-2017 13:46:19:09</SystemTimeUtc>
// 	<ChannelName>TX02</ChannelName>
// 	<UtcOffset>00:00:00:00</UtcOffset>
// 	<OnAir>False</OnAir>
// 	<SecondScreenEvents>
// 		<SecondScreenEvent xsi:type="ProgramEvent">
// 			<StartTime>04-JUL-2017 12:01:02:00</StartTime>
// 			<Duration>00:07:00:00</Duration>
// 			<EventOid>{31708278-C5A63E93}</EventOid>
// 			<EventName>SEGMENT 01</EventName>
// 			<MaterialId/>
// 			<Description/>
// 			<GraphicsId/>
// 			<ShowType/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="BreakHeaderEvent">
// 			<EventOid>{716983A2-501FFC5F}</EventOid>
// 			<StartTime>04-JUL-2017 12:07:00:00</StartTime>
// 			<Duration>00:03:00:00</Duration>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:07:00:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{295A15F9-64FBADD2}</EventOid>
// 			<EventName>COMM 01</EventName>
// 			<MaterialId>ABC123</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:07:30:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{2DD11936-08AC6607}</EventOid>
// 			<EventName>COMM 02</EventName>
// 			<MaterialId>DEF456</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:08:00:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{6AA028C4-C0A63544}</EventOid>
// 			<EventName>COMM 03</EventName>
// 			<MaterialId>GHI789</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:08:30:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{5C0B5C3B-CF8631D8}</EventOid>
// 			<EventName>COMM 04</EventName>
// 			<MaterialId>JKL123</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:09:00:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{30650BEB-9E3A4FB5}</EventOid>
// 			<EventName>COMM 05</EventName>
// 			<MaterialId>MNO456</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:09:30:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{143A9EF5-3F778527}</EventOid>
// 			<EventName>COMM 06</EventName>
// 			<MaterialId>PQR789</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="ProgramEvent">
// 			<StartTime>04-JUL-2017 12:10:00:00</StartTime>
// 			<Duration>00:07:00:00</Duration>
// 			<EventOid>{5126939A-7A32AE10}</EventOid>
// 			<EventName>SEGMENT 02</EventName>
// 			<MaterialId/>
// 			<Description/>
// 			<GraphicsId/>
// 			<ShowType/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="BreakHeaderEvent">
// 			<EventOid>{3E94EA7A-8EE90347}</EventOid>
// 			<StartTime>04-JUL-2017 12:17:00:00</StartTime>
// 			<Duration>00:03:00:00</Duration>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:17:00:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{079F4FD6-6B5C739E}</EventOid>
// 			<EventName>COMM 01</EventName>
// 			<MaterialId>ABC123</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:17:30:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{1DDB9CBE-DADAE5C7}</EventOid>
// 			<EventName>COMM 02</EventName>
// 			<MaterialId>DEF456</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:18:00:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{71E0B0C7-C7A19AE6}</EventOid>
// 			<EventName>COMM 03</EventName>
// 			<MaterialId>GHI789</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:18:30:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{71C6A52B-CE37AA16}</EventOid>
// 			<EventName>COMM 04</EventName>
// 			<MaterialId>JKL123</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:19:00:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{166404F8-A5493F61}</EventOid>
// 			<EventName>COMM 05</EventName>
// 			<MaterialId>MNO456</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 		<SecondScreenEvent xsi:type="InfoEvent">
// 			<StartTime>04-JUL-2017 12:19:30:00</StartTime>
// 			<Duration>00:00:30:00</Duration>
// 			<EventOid>{166904AB-8CF91258}</EventOid>
// 			<EventName>COMM 06</EventName>
// 			<MaterialId>PQR789</MaterialId>
// 			<Type>Commercial</Type>
// 			<Description/>
// 			<GraphicsId/>
// 		</SecondScreenEvent>
// 	</SecondScreenEvents>
// </EventsMessage>`, {})
//     .then((result) => {
//         console.log(JSON.stringify(result, null, 2))
//     });
