"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.xlsxNormaliser = void 0;
const normaliser_1 = require("./normaliser");
const xml2js = require("xml2js");
const xlsx = require("xlsx");
const _ = require("lodash");
const encoding = require("text-encoding");
class xlsxNormaliser extends normaliser_1.Normaliser {
    getId() {
        return 'xlsx';
    }
    getDescription() {
        return 'Read xlsx Excel files';
    }
    getAsBuffer() {
        return true;
    }
    getConfigForm() {
        return {
            schema: {
                "type": "object",
                "properties": {
                    "sheetName": {
                        "type": "string",
                        "title": "Sheet name"
                    },
                    "startRow": {
                        "type": "number",
                        "title": "Start Row"
                    }
                }
            },
            uiSchema: {
                "sheetName": {
                    "ui:description": "Leave blank to choose first sheet"
                },
                "startRow": {
                    "ui:description": "Row to start parsing the sheet from. Leave blank to start from 0"
                }
            }
        };
    }
    parseXml(str) {
        return new Promise((resolve, reject) => {
            xml2js.parseString(str, (err, data) => {
                if (err) {
                    reject(null);
                }
                else {
                    resolve(data);
                }
            });
        });
    }
    async removeSheetViewsFromXml(str) {
        var xml = await this.parseXml(str);
        var worksheets = _.get(xml, "Workbook.Worksheet");
        if (xml && worksheets) {
            worksheets = _.map(worksheets, (ws) => {
                return _.omitBy(ws, (v, k) => {
                    return /\:.*sheetViews/.test(k);
                });
            });
            xml.Workbook.Worksheet = worksheets;
        }
        var newXml = new xml2js.Builder().buildObject(xml);
        return xlsx.read(newXml, { type: "string" });
    }
    ec(r, c) {
        return xlsx.utils.encode_cell({ r: r, c: c });
    }
    delete_row(ws, row_index) {
        var variable = xlsx.utils.decode_range(ws["!ref"]);
        for (var R = row_index; R < variable.e.r; ++R) {
            for (var C = variable.s.c; C <= variable.e.c; ++C) {
                ws[this.ec(R, C)] = ws[this.ec(R + 1, C)];
            }
        }
        variable.e.r--;
        ws['!ref'] = xlsx.utils.encode_range(variable.s, variable.e);
    }
    async internalTranslate(data, options) {
        const { startRow, sheetName } = options;
        var workbook;
        try {
            //If data is xml, try to remove the sheetsview component
            workbook = await this.removeSheetViewsFromXml(new encoding.TextDecoder("utf-8").decode(data));
        }
        catch (e) {
            workbook = xlsx.read(data, { type: 'buffer', cellDates: true });
        }
        let worksheet;
        if (sheetName) {
            worksheet = workbook.Sheets[sheetName];
        }
        else {
            worksheet = workbook.Sheets[workbook.SheetNames[0]];
        }
        if (startRow) {
            for (var i = 1; i < startRow; i++) {
                this.delete_row(worksheet, 0);
            }
        }
        //Change all cells of type date to ISO 8601 strings
        var range = xlsx.utils.decode_range(worksheet['!ref']); // get the range
        for (var R = range.s.r; R <= range.e.r; ++R) {
            for (var C = range.s.c; C <= range.e.c; ++C) {
                var cellRef = xlsx.utils.encode_cell({ c: C, r: R }); // construct A1 reference for cell
                if (!worksheet[cellRef])
                    continue; // if cell doesn't exist, move on
                var cell = worksheet[cellRef];
                if (cell.t == "d") {
                    delete cell.w;
                    cell.z = 'YYYY-MM-DD\"T\"HH:mm:ss';
                    worksheet[cellRef] = cell;
                    xlsx.utils.format_cell(worksheet[cellRef]);
                    cell.v = cell.w;
                    xlsx.utils.format_cell(worksheet[cellRef]);
                }
            }
        }
        const json = xlsx.utils.sheet_to_json(worksheet);
        return Promise.resolve(json);
    }
}
exports.xlsxNormaliser = xlsxNormaliser;
