"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BasePluginClient = void 0;
const Debug = require("debug");
const uuid_1 = require("uuid");
const PluginSubscriber_1 = require("../subscribers/interfaces/PluginSubscriber");
const interfaces_1 = require("../subscribers/interfaces");
const esbInterface_1 = require("../esbInterface");
const logging_1 = require("../logging");
const InterfaceUser_1 = require("../selector/InterfaceUser");
const PluginUploader_1 = require("./PluginUploader");
const selector_1 = require("../selector");
const debug = logging_1.Log.getNamespace("basePluginClient");
const debugPortRecv = debug.extend("port:recv");
const debugPortSend = debug.extend("port:send");
class BasePluginClient {
    static get() {
        return this.__INITIALISED;
    }
    constructor() {
        this.connected = false;
        this.onConnectedCallbacks = new Set();
        this.themeCallbacks = new Set();
        this.colorSchemeCallbacks = new Set();
        this.statusProcessors = {};
        this.searchCallbacks = {};
        this.shoppingBasketEnabled = false;
        this.selectorBase = null;
        this.backChannelCallbacks = {};
        this.backChannelProcessors = {};
        this.messageCallbacks = {};
        this.uploadCallbacks = {};
        this.getMessagePort = () => {
            return this.port?.postMessage;
        };
        this.onConnected = (callback) => {
            this.onConnectedCallbacks.add(callback);
            if (this.connected) {
                callback();
            }
            return { dispose: () => this.offConnected(callback) };
        };
        this.offConnected = (callback) => {
            this.onConnectedCallbacks.delete(callback);
        };
        this.isConnected = () => {
            return this.connected;
        };
        this.getUser = () => {
            return this.user;
        };
        this.getConfig = () => {
            return this.config;
        };
        this.getSessionId = () => {
            return this.sessionId;
        };
        this.getAvailableApplications = () => {
            return this.availbleApplications;
        };
        this.getPermissions = () => {
            return this.permissions ?? [];
        };
        this.postMessage = (message, transfer) => {
            debugPortSend(message.type, message);
            this.port.postMessage(message, { transfer });
        };
        this.setupPort = () => {
            const channel = new MessageChannel();
            this.port = channel.port1;
            this.port.onmessage = this.onPortMessage;
            window.parent.postMessage({
                type: '@busby/plugin/connect',
                id: window.location.hash.replace(/^#/, ""),
            }, "*", [channel.port2]);
            module?.hot?.dispose((data) => {
                data.port = this.port;
                this.port.onmessage = null;
            });
        };
        this.getFileLink = (type, fileName, location, path = "/") => {
            if (!path) {
                path = "/";
            }
            if (!path.startsWith("/")) {
                path = "/" + path;
            }
            if (!path.endsWith("/")) {
                path += "/";
            }
            if (fileName.startsWith("/")) {
                fileName = fileName.replace(/^\//, "");
            }
            let result = `${this.selectorBase}/${type === "download" ? "locationDownload" : "locationMedia"}/${location}${path}${fileName}`;
            if (type === "media") {
                result += '?sendFile';
            }
            return result;
        };
        this.getTaskLink = (appId, data) => {
            const params = data ? new URLSearchParams(data) : null;
            return `${this.selectorBase}/${appId}${params ? '?' + params.toString() : ''}`;
        };
        /**
        * Tell selector to select a different app.
        * @param appId The app id to select - see the address bar for these they are the name after the / in the url
        * @param data Optional data which can be used by the other app to select a specific entry
        */
        this.selectApp = (appId, data) => {
            this.postMessage({
                type: "operation/selectApp",
                params: {
                    appId,
                    data,
                },
            });
        };
        this.askModalQuestion = (question, buttons, inputOptions) => {
            return new Promise((resolve) => {
                const callbackId = (0, uuid_1.v4)();
                this.messageCallbacks[callbackId] = resolve;
                this.postMessage({
                    type: "operation/askModalQuestion",
                    callbackId,
                    params: {
                        question,
                        buttons,
                        inputOptions,
                    }
                });
            });
        };
        this.getModalForm = (form) => {
            return new Promise((resolve) => {
                const callbackId = (0, uuid_1.v4)();
                this.messageCallbacks[callbackId] = resolve;
                this.postMessage({
                    type: "operation/getModalForm",
                    callbackId,
                    params: {
                        form,
                    }
                });
            });
        };
        this.getModalConfirmation = (question = "Are you sure?", yesText, noText) => {
            return new Promise((resolve) => {
                const callbackId = (0, uuid_1.v4)();
                this.messageCallbacks[callbackId] = resolve;
                this.postMessage({
                    type: "operation/getModalConfirmation",
                    callbackId,
                    params: {
                        question,
                        yesText,
                        noText,
                    }
                });
            });
        };
        this.startAudioAlert = (alert) => {
            this.postMessage({
                type: "operation/startAudioAlert",
                params: {
                    alert,
                }
            });
        };
        this.stopAudioAlert = (alert) => {
            this.postMessage({
                type: "operation/stopAudioAlert",
                params: {
                    alert,
                }
            });
        };
        this.sendNotification = (title, body, params) => {
            this.postMessage({
                type: "operation/sendNotification",
                params: {
                    title,
                    body,
                    params,
                }
            });
        };
        this.showSearchResults = (visible) => {
            this.postMessage({
                type: "operation/showSearchResults",
                params: {
                    visible,
                }
            });
        };
        this.raiseToast = (type, title, description, options) => {
            this.postMessage({
                type: "operation/raiseToast",
                params: {
                    type,
                    title,
                    description,
                    options
                }
            });
        };
        this.onSearchResults = (searchId, eventHandler) => {
            this.searchCallbacks[searchId] ??= new Set();
            this.searchCallbacks[searchId].add(eventHandler);
            this.postMessage({
                type: "search/register",
                searchId,
            });
            return {
                dispose: () => {
                    this.offSearchResults(searchId, eventHandler);
                }
            };
        };
        this.offSearchResults = (searchId, eventHandler) => {
            this.searchCallbacks[searchId].delete(eventHandler);
            if (this.searchCallbacks[searchId].size === 0) {
                this.postMessage({
                    type: "search/deregister",
                    searchId,
                });
            }
        };
        this.onUserMetadata = (eventHandler) => {
            return this.interfaceUser.onUserMetadata(eventHandler);
        };
        this.offUserMetadata = (eventHandler) => {
            return this.interfaceUser.offUserMetadata(eventHandler);
        };
        this.getSearchDetail = async (searchType, result) => {
            throw new Error("Not yet implemented");
        };
        this.isShoppingBasketEnabled = () => {
            return this.shoppingBasketEnabled;
        };
        this.getTheme = () => {
            return this.theme;
        };
        this.getColorScheme = () => {
            return this.colorScheme;
        };
        this.onColorSchemeChange = (eventHandler) => {
            if (!this.colorSchemeCallbacks.has(eventHandler)) {
                this.colorSchemeCallbacks.add(eventHandler);
            }
            return {
                dispose: () => {
                    this.offThemeChange(eventHandler);
                }
            };
        };
        this.offColorSchemeChange = (eventHandler) => {
            if (this.colorSchemeCallbacks.has(eventHandler)) {
                this.colorSchemeCallbacks.delete(eventHandler);
            }
        };
        this.onThemeChange = (eventHandler) => {
            if (!this.themeCallbacks.has(eventHandler)) {
                this.themeCallbacks.add(eventHandler);
            }
            return {
                dispose: () => {
                    this.offThemeChange(eventHandler);
                }
            };
        };
        this.offThemeChange = (eventHandler) => {
            if (this.themeCallbacks.has(eventHandler)) {
                this.themeCallbacks.delete(eventHandler);
            }
        };
        this.id = window.location.hash.replace(/^#/, "");
        this.onConnectedCallbacks = new Set();
        this.themeCallbacks = new Set();
        this.colorSchemeCallbacks = new Set();
        this.statusProcessors = {};
        this.backChannelCallbacks = {};
        this.backChannelProcessors = {};
        this.messageCallbacks = {};
        if (BasePluginClient.__INITIALISED !== undefined) {
            throw new Error(`An instance of plugin client has already been initialised. ${BasePluginClient.__INITIALISED?.prototype?.constructor?.name} ${this?.prototype?.constructor?.name}`);
        }
        BasePluginClient.__INITIALISED = this;
        window.__BASECLIENT__ = this;
        window.__SUBS__ = interfaces_1.SubscriberManager.get('pluginSubscriber', { pluginSubscriberInterface: this.getPluginSubscriberInterface() });
        window.__DEBUG__ = Debug;
        PluginSubscriber_1.PluginSubscriber.setSubscriberInterface(this.getPluginSubscriberInterface());
        this.onPortMessage = this.onPortMessage.bind(this);
        this.setupPort();
    }
    onPortMessage(msg) {
        debugPortRecv(msg.data.type, msg.data);
        switch (msg.data.type) {
            case "initialised": {
                const { colorScheme, user, config, permissions, selectorBase, options, shoppingBasketEnabled, sessionId, availbleApplications } = msg.data;
                this.user = user;
                this.sessionId = sessionId;
                this.availbleApplications = availbleApplications;
                this.interfaceUser = new InterfaceUser_1.InterfaceUser(user, this.setUserMetadata);
                this.config = config;
                this.permissions = permissions;
                this.selectorBase = selectorBase;
                this.shoppingBasketEnabled = shoppingBasketEnabled;
                this.theme = options.theme;
                this.colorScheme = colorScheme;
                selector_1.SelectorOperations.get().setup(this, options);
                debug("plugin client initalised");
                this.connected = true;
                this.onConnectedCallbacks.forEach((callback) => {
                    try {
                        callback();
                    }
                    catch { }
                });
                this.themeCallbacks.forEach((callback) => {
                    try {
                        callback(this.theme);
                    }
                    catch { }
                });
                this.colorSchemeCallbacks.forEach((callback) => {
                    try {
                        callback(this.colorScheme);
                    }
                    catch { }
                });
                break;
            }
            case "colorScheme": {
                this.colorScheme = msg.data.colorScheme;
                this.colorSchemeCallbacks.forEach((callback) => {
                    try {
                        callback(this.colorScheme);
                    }
                    catch { }
                });
                break;
            }
            case "theme": {
                this.theme = msg.data.theme;
                this.themeCallbacks.forEach((callback) => {
                    try {
                        callback(this.theme);
                    }
                    catch { }
                });
                break;
            }
            case "backchannel/callback": {
                if (this.backChannelCallbacks[msg.data.callbackId]) {
                    this.backChannelCallbacks[msg.data.callbackId](msg.data.message);
                    delete this.backChannelCallbacks[msg.data.callbackId];
                }
                break;
            }
            case "subscriber/backchannel": {
                if (this.backChannelProcessors[msg.data.subscriberId]) {
                    this.backChannelProcessors[msg.data.subscriberId](msg.data.message);
                }
                break;
            }
            case "subscriber/status": {
                if (this.statusProcessors[msg.data.statusProcessorId]) {
                    this.statusProcessors[msg.data.statusProcessorId](msg.data.message);
                }
                break;
            }
            case "subscriber/callback": {
                if (this.messageCallbacks[msg.data.callbackId]) {
                    this.messageCallbacks[msg.data.callbackId](msg.data?.message !== null ? esbInterface_1.EsbMessage.makeFromJson(msg.data.message) : null);
                    delete this.messageCallbacks[msg.data.callbackId];
                }
                break;
            }
            case "operation/callback": {
                if (this.messageCallbacks[msg.data.callbackId]) {
                    this.messageCallbacks[msg.data.callbackId](msg.data.params);
                    delete this.messageCallbacks[msg.data.callbackId];
                }
                break;
            }
            case "upload/callback": {
                if (this.uploadCallbacks[msg.data.callbackId]) {
                    const data = msg.data.params?.body ?
                        new Response(![101, 204, 205, 304].includes(msg.data.params.status) && msg.data.params.body ? new Blob([msg.data.params.body]) : null, {
                            headers: msg.data.params.headers,
                            status: msg.data.params.status,
                            statusText: msg.data.params.statusText,
                            url: msg.data.params.url,
                        })
                        :
                            msg.data.params;
                    this.uploadCallbacks[msg.data.callbackId](data);
                    delete this.uploadCallbacks[msg.data.callbackId];
                }
                break;
            }
            case "search/results": {
                const { searchId, resultType, results, displayColumns } = msg.data;
                if (searchId) {
                    this.searchCallbacks[searchId]?.forEach((fn) => {
                        try {
                            fn?.(resultType, results, displayColumns);
                        }
                        catch { }
                    });
                }
                break;
            }
            case "userMetadata/updated": {
                const { key, value } = msg.data;
                this.interfaceUser.setMetadata(key, value, false);
            }
        }
    }
    getPluginSubscriberInterface() {
        return ({
            emit: (params) => {
                return new Promise((resolve, reject) => {
                    const callbackId = (0, uuid_1.v4)();
                    this.backChannelCallbacks[callbackId] = resolve;
                    this.postMessage({
                        type: "subscriber/backchannel",
                        callbackId,
                        message: params,
                    });
                });
            },
            sendMessage: (id, message) => {
                return new Promise((resolve, reject) => {
                    const callbackId = (0, uuid_1.v4)();
                    this.messageCallbacks[callbackId] = resolve;
                    this.postMessage({
                        type: "subscriber/message",
                        messageProcessorId: id,
                        message,
                        callbackId,
                    });
                });
            },
            registerStatusProcessor: (id, processor) => {
                logging_1.Log.getNamespace('pluginSubscriberInterface:register:status')(id);
                this.statusProcessors[id] = processor;
            },
            registerSubscriberBackChannelProcessor: (id, processor) => {
                logging_1.Log.getNamespace('pluginSubscriberInterface:register:backchannel')(id);
                this.backChannelProcessors[id] = processor;
            }
        });
    }
    /**
     * Gets a download link for a file in a location
     * @param fileName The name if the file to download
     * @param location The name of the location that the file is stored in
     * @param path The path in the location to the file
     */
    fileDownloadLink(fileName, location, path = "/") {
        if (!path) {
            path = "/";
        }
        if (!path.startsWith("/")) {
            path = "/" + path;
        }
        if (!path.endsWith("/")) {
            path += "/";
        }
        if (fileName.startsWith("/")) {
            fileName = fileName.replace(/^\//, "");
        }
        return `${this.selectorBase}/locationDownload/${location}${path}${fileName}`;
    }
    /**
     * Triggers a download of a specified file in a location
     * @param fileName The name if the file to download including any path
     * @param location The name of the location that the file is stored in
     * @param path The path in the location to the file with preceding /
     */
    fileDownload(fileName, location, path = "/") {
        if (!path) {
            path = "/";
        }
        if (!path.startsWith("/")) {
            path = "/" + path;
        }
        if (!path.endsWith("/")) {
            path += "/";
        }
        if (fileName.startsWith("/")) {
            fileName = fileName.replace(/^\//, "");
        }
        this.postMessage({
            type: "operation/download",
            params: {
                fileName, location, path
            },
        });
    }
    /**
     * Uploads a file, returning a promise which resolves on completion and rejects if an error occurs
     * @param file The file to upload
     * @param location The name of the location to upload to (see Locations in Config Editor)
     * @param pathOrParams The Path or Params
     * @param params Optional params to aid in the upload
     */
    async fileUpload(file, location, pathOrParams, params = {}) {
        const path = typeof pathOrParams === "string" ? pathOrParams : null;
        if (typeof pathOrParams !== "string") {
            params = pathOrParams;
        }
        await new Promise((resolve, reject) => {
            const endpoint = `${location}${path ? `/${path}` : ""}`;
            const uploader = new PluginUploader_1.default({
                endpoint,
                file,
                postParams: {
                    fileName: file.name
                }
            }, async ({ chunk, ...params }) => {
                debug("chunk", chunk, params);
                let cb;
                const promise = new Promise((resolve) => cb = resolve);
                const callbackId = (0, uuid_1.v4)();
                this.uploadCallbacks[callbackId] = cb;
                this.postMessage({
                    type: "upload/chunk",
                    callbackId,
                    params: {
                        ...params,
                        chunk,
                    },
                }, [chunk]);
                return promise;
            });
            // subscribe to events
            uploader.on('error', async (err) => {
                if (err.detail instanceof Response) {
                    const error = `Selector returned an error: ${err.detail.status} ${err.detail.statusText}`;
                    console.error(error, await err.detail.text(), err.detail);
                    reject(error);
                }
                else {
                    console.error('Something bad happened', err.detail);
                    reject(`Failed to upload file ${file.name}: ${err.detail}`);
                }
            });
            uploader.on('progress', (progress) => {
                params?.onProgress(progress.detail);
                console.log(`The upload is at ${progress.detail}%`);
            });
            uploader.on('finish', () => {
                console.log('Upload complete');
                resolve(null);
            });
        });
    }
    setUserMetadata(key, value) {
        this.postMessage({
            type: "userMetadata/set",
            params: {
                key,
                value,
            }
        });
    }
    getUserMetadata(key) {
        return this.interfaceUser.getMetadata(key);
    }
    showJobModal(jobUuid, options) {
        this.postMessage({
            type: "operation/showJobModal",
            params: {
                jobUuid,
                options,
            }
        });
    }
    hideJobModal() {
        this.postMessage({
            type: "operation/hideJobModal"
        });
    }
}
exports.BasePluginClient = BasePluginClient;
BasePluginClient.__INITIALISED = undefined;
