"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkflowPluginClient = void 0;
const _ = require("lodash");
const logging_1 = require("../logging");
const BasePluginClient_1 = require("./BasePluginClient");
const debug = logging_1.Log.getNamespace("workflowPluginClient");
class WorkflowPluginClient extends BasePluginClient_1.BasePluginClient {
    static get() {
        if (!WorkflowPluginClient.__workflowinstance) {
            new WorkflowPluginClient();
        }
        return WorkflowPluginClient.__workflowinstance;
    }
    constructor() {
        super();
        this.onJobCallbacks = new Set();
        this.onCompleteCallbacks = new Set();
        this.onJobComplete = async (button, callbackId) => {
            let result = true;
            const failedCallbacks = [];
            for (const cb of this.onCompleteCallbacks.values()) {
                const changeCallback = _.isArrayLike(cb) ? cb[0] : cb;
                const failCallback = _.isArrayLike(cb) ? cb[1] : null;
                try {
                    const res = await changeCallback(button);
                    if (res === true) {
                        if (failCallback) {
                            failedCallbacks.push(failCallback);
                        }
                    }
                    else if (res === false) {
                        result = false;
                        break;
                    }
                    else {
                        // Callback not applicable for this state change.
                    }
                }
                catch { }
            }
            this.postMessage({
                type: "workflowTask/completeCallback",
                callbackId,
                result,
            });
            if (result === false) {
                for (const callback of failedCallbacks) {
                    try {
                        callback(button);
                    }
                    catch { }
                }
            }
        };
        this.onJob = (callback) => {
            this.onJobCallbacks.add(callback);
            if (this.job) {
                callback(this.job);
            }
            return { dispose: () => this.offJob(callback) };
        };
        this.offJob = (callback) => {
            this.onJobCallbacks.delete(callback);
        };
        /**
         * Add a callback to be called when a state change button is pressed to perform validation or other actions to allow or prevent the state change to occur.
         * @param callbacks Callback to check if change is possible, Failed callback to allow for rollback if a later callback returns false.
         * @returns A disposable for turning off the listener
         */
        this.onComplete = (callbacks) => {
            this.onCompleteCallbacks.add(callbacks);
            return { dispose: () => this.offComplete(callbacks) };
        };
        this.offComplete = (callbacks) => {
            this.onCompleteCallbacks.delete(callbacks);
        };
        this.deselectJob = () => {
            this.postMessage({
                type: "workflowTask/deselectJob",
            });
        };
        this.createJob = (job, options) => {
            this.postMessage({
                type: "workflowTask/createJob",
                job,
                options,
            });
        };
        this.disableButton = (buttonId, disabled) => {
            this.postMessage({
                type: "workflowTask/disableButton",
                buttonId,
                disabled,
            });
        };
        this.hideButton = (buttonId, hidden) => {
            this.postMessage({
                type: "workflowTask/hideButton",
                buttonId,
                hidden,
            });
        };
        this.updateDraftInfoBlock = (infoBlock, jobUuid) => {
            this.postMessage({
                type: "workflowTask/updateDraftInfoBlock",
                infoBlock,
                jobUuid: jobUuid || this.job?.uuid,
            });
        };
        this.updateStateInfoBlock = (infoBlock) => {
            this.postMessage({
                type: "workflowTask/updateDraftInfoBlock",
                infoBlock,
            });
        };
        WorkflowPluginClient.__workflowinstance = this;
        window.__WORKFLOWPLUGINCLIENT__ = this;
    }
    async onPortMessage(msg) {
        super.onPortMessage(msg);
        switch (msg.data.type) {
            case "workflowTask/data": {
                debug("Received a Job", msg.data.job.uuid);
                this.job = msg.data.job;
                for (const callback of this.onJobCallbacks.values()) {
                    try {
                        callback(this.job);
                    }
                    catch { }
                }
                break;
            }
            case "workflowTask/complete": {
                debug("Job complete button pressed", this.job?.uuid, msg.data.button?.id);
                this.onJobComplete(msg.data.button, msg.data.callbackId);
            }
        }
    }
    getJob() {
        return this.job;
    }
    getTaskConfig() {
        return this.getConfig()?.task?.config;
    }
}
exports.WorkflowPluginClient = WorkflowPluginClient;
