"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Schedule = exports.MyTest = exports.Version = void 0;
const primaryEvent_1 = require("./primaryEvent");
const _ = require("lodash");
const bTime_1 = require("../bTime");
const jsonpath_plus_1 = require("jsonpath-plus");
const logging_1 = require("../logging");
class Version {
    constructor(version, date, options) {
        this.version = version;
        this.date = date;
        if (options) {
            this.author = options.author;
            this.comment = options.comment;
        }
    }
    static fromJson(json = {}) {
        return new Version(json.version, bTime_1.BDateTime.fromJson(json.date), {
            author: json.author,
            comment: json.comment
        });
    }
    toJson() {
        return {
            author: this.author,
            comment: this.comment,
            date: this.date?.toJson(),
            version: this.version
        };
    }
}
exports.Version = Version;
class MyTest {
}
exports.MyTest = MyTest;
class Schedule {
    constructor(channel, events, name, scheduleType, options) {
        this.channel = channel;
        this.events = events;
        this.name = name;
        this.scheduleType = scheduleType;
        if (options) {
            this.scheduleId = options.scheduleId;
            this.origin = options.origin;
            this.alternateName = options.alternateName;
            this.sourceFileReference = options.sourceFileReference;
            this.timezoneOffset = options.timezoneOffset;
            this.broadcastDate = options.broadcastDate;
            this.start = options.start;
            this.end = options.end;
            this.versions = options.versions;
            this.custom = options.custom;
            this.action = options.action;
        }
    }
    static getKeys() {
        return [
            { "name": "broadcastDate", type: "date" },
            { "name": "channel" },
            { "name": "end", type: "date" },
            { "name": "name" },
            { "name": "origin" },
            { "name": "alternateName" },
            { "name": "scheduleId" },
            { "name": "scheduleType" },
            { "name": "sourceFileReference" },
            { "name": "start", type: "date" },
            { "name": "timezoneOffset", type: "duration" }
        ];
    }
    static fromJson(json = {}) {
        let broadcastDate, start, end, versions = [], events = [], timezoneOffset;
        versions = _.map(json.versions, ver => {
            if (ver instanceof Version) {
                return ver;
            }
            else {
                return Version.fromJson(ver);
            }
        });
        if (json.start) {
            start = bTime_1.BDateTime.fromJson(json.start);
        }
        if (json.end) {
            end = bTime_1.BDateTime.fromJson(json.end);
        }
        if (json.broadcastDate) {
            broadcastDate = bTime_1.BDateTime.fromJson(json.broadcastDate);
        }
        if (json.timezoneOffset) {
            timezoneOffset = bTime_1.BDuration.fromJson(json.timezoneOffset);
        }
        if (json.events) {
            events = _.map(json.events, (event, id) => {
                if (event instanceof primaryEvent_1.CommentEvent || event instanceof primaryEvent_1.PlayEvent) {
                    return event;
                }
                if (event.eventType == "play") {
                    return primaryEvent_1.PlayEvent.fromJson(event);
                }
                if (event.eventType == "comment") {
                    if (!event.startDate) {
                        event.startDate = json.events?.[id + 1]?.startDate;
                    }
                    return primaryEvent_1.CommentEvent.fromJson(event);
                }
                if (event.eventType == "breakHeader") {
                    return primaryEvent_1.BreakHeaderEvent.fromJson(event);
                }
                throw new Error("Invalid event " + event.eventType);
            });
        }
        return new Schedule(json.channel, events, json.name, json.scheduleType, {
            origin: json.origin,
            alternateName: json.alternateName,
            scheduleId: json.scheduleId,
            sourceFileReference: json.sourceFileReference,
            timezoneOffset,
            broadcastDate, start, end, versions,
            custom: json.custom,
            action: json.action
        });
    }
    toJson() {
        return {
            channel: this.channel,
            date: this.broadcastDate && this.broadcastDate.toIsoString(),
            name: this.name,
            origin: this.origin,
            alternateName: this.alternateName,
            scheduleId: this.scheduleId,
            scheduleType: this.scheduleType,
            sourceFileReference: this.sourceFileReference,
            broadcastDate: this.broadcastDate && this.broadcastDate.toJson(),
            start: this.start && this.start.toJson(),
            end: this.end && this.end.toJson(),
            timezoneOffset: this.timezoneOffset && this.timezoneOffset.toJson(),
            versions: this.versions ? _.map(this.versions, version => version?.toJson()) : undefined,
            events: this.events ? _.map(this.events, event => event?.toJson()) : undefined,
            custom: this.custom,
            action: this.action
        };
    }
    clone() {
        return Schedule.fromJson(this.toJson());
    }
    filterEvents(filterFn) {
        const schedule = this.clone();
        schedule.events = _.filter(schedule.events, filterFn);
        return schedule;
    }
    filterEventsByJsonPath(predicate) {
        const schedule = this.clone();
        schedule.events = _.map((0, jsonpath_plus_1.JSONPath)({ path: "$.[?(" + predicate + ")]", json: schedule.toJson().events }), e => primaryEvent_1.PlayEvent.fromJson(e));
        return schedule;
    }
    // Pass function that returns a string that should not be repeated, or return null if always include
    excludeRepeats(repeatFn) {
        const schedule = this.clone();
        schedule.events = _.uniqBy(schedule.events, repeatFn);
        return schedule;
    }
    updateEvents(updateFn) {
        const schedule = this.clone();
        schedule.events = _.flatMap(schedule.events, updateFn);
        return schedule;
    }
    retime(startTime) {
        const schedule = this.clone();
        const firstPlayEvent = _.find(this.events, { eventType: "play" });
        if (firstPlayEvent) {
            firstPlayEvent.startMode = "fixed";
            let time = firstPlayEvent.startDate;
            if (startTime) {
                time = startTime;
            }
            schedule.start = time;
            schedule.broadcastDate = bTime_1.BDateTime.fromIso(time.getDateString() + "T00:00:00", time.frameRate);
            for (const ev of schedule.events) {
                if (ev instanceof primaryEvent_1.PlayEvent) {
                    ev.startDate = time;
                    time = time.addDuration(ev.duration);
                }
            }
        }
        const lastPlayEvent = _.findLast(schedule.events, (event) => {
            return _.has(event, 'startDate');
        });
        schedule.end = lastPlayEvent.startDate;
        return schedule;
    }
    getWithCommercialsInBreaks() {
        const schedule = this.clone();
        schedule.events = [];
        let i = 0;
        let breakEvent;
        while (i < this.events.length) {
            const event = this.events[i];
            if (event instanceof primaryEvent_1.BreakHeaderEvent) {
                breakEvent = event;
                breakEvent.commercials = [];
                schedule.events.push(breakEvent);
            }
            else {
                if (!breakEvent) {
                    // we are not in a break just copy the events
                    schedule.events.push(event);
                }
                else {
                    if (event instanceof primaryEvent_1.PlayEvent) {
                        if (event.groupId && ((event.groupType === "commercial") || !event.groupType)) {
                            // we are in an event add the commercial
                            breakEvent.commercials.push(event);
                        }
                        else {
                            // we have reached the end of the event
                            schedule.events.push(event);
                            breakEvent = null;
                        }
                    }
                    else if (event instanceof primaryEvent_1.BreakHeaderEvent) {
                        throw new Error("Invalid nested break header");
                    }
                }
            }
            i++;
        }
        return schedule;
    }
    // Gets a copy of the schedule with events from the fromEvent inclusive to the toEvent exclusive
    getScheduleFromEventToEvent(fromEvent, toEvent) {
        const schedule = this.clone();
        schedule.events = [];
        let i = 0;
        let found = false;
        let stop = false;
        if (fromEvent) {
            if (toEvent) {
                if (toEvent instanceof primaryEvent_1.PlayEvent && fromEvent instanceof primaryEvent_1.PlayEvent) {
                    if (fromEvent.startDate.framesDifference(toEvent.startDate) > 0) {
                        stop = true;
                    }
                }
            }
            if (!stop) {
                while (i < this.events.length) {
                    const event = this.events[i];
                    if (event.id === fromEvent.id) {
                        found = true;
                    }
                    if (found) {
                        if (event.id === toEvent?.id) {
                            break;
                        }
                        schedule.events.push(event);
                    }
                    i++;
                }
            }
        }
        return schedule;
    }
    findFirstEventAfterTime(time, excludeFn) {
        let i = 0;
        let result;
        while (!result && i < this.events.length) {
            const event = this.events[i];
            if ((event instanceof primaryEvent_1.PlayEvent || event instanceof primaryEvent_1.BreakHeaderEvent || event instanceof primaryEvent_1.CommentEvent) && event.startDate && time) {
                if (event.startDate?.framesDifference(time) > 0) {
                    if (!excludeFn || (excludeFn && excludeFn(event))) {
                        result = event;
                    }
                }
            }
            i++;
        }
        return result;
    }
    findMatchFromIndex(index, predicate, direction) {
        let id = (direction === "down") ? index : this.events.length - index;
        const events = (direction === "down") ? this.events : this.events.slice(0).reverse();
        return _.find(events, predicate, id);
    }
    getProgId(playEvent) {
        return playEvent.programmeId ?? playEvent.title;
    }
    findLastProgrammePartAfterTime(time) {
        const eventAfterTime = this.findFirstEventAfterTime(time);
        let id = this.events.indexOf(eventAfterTime);
        let nextProgramme;
        if (id >= 0) {
            let currentProgramme = this.findMatchFromIndex(id, { eventCategory: "Programme" }, "up");
            nextProgramme = this.findMatchFromIndex(id + 1, { eventCategory: "Programme" }, "down");
            if (!currentProgramme) {
                currentProgramme = nextProgramme;
            }
            if (!currentProgramme) {
                logging_1.Log.logDebug("No current programme");
            }
            else if (currentProgramme instanceof primaryEvent_1.PlayEvent) {
                id = this.events.indexOf(currentProgramme);
                const programmeId = this.getProgId(currentProgramme);
                if (!nextProgramme) {
                    return _.findLast(this.events, (ev) => ev instanceof primaryEvent_1.PlayEvent);
                }
                if (this.getProgId(currentProgramme) !== this.getProgId(nextProgramme)) {
                    return currentProgramme;
                }
                while (nextProgramme && (this.getProgId(nextProgramme) === programmeId)) {
                    nextProgramme = this.findMatchFromIndex(id + 1, { eventCategory: "Programme" }, "down");
                    id = this.events.indexOf(nextProgramme);
                }
                if (nextProgramme) {
                    return this.findMatchFromIndex(id, { eventCategory: "Programme" }, "up");
                }
            }
        }
    }
    findEvent(predicate) {
        return _.find(this.events, predicate);
    }
    mapEvents(fn) {
        const schedule = this.clone();
        schedule.events = _.map(this.events, fn.bind(this));
        return schedule;
    }
    addPreviousEventIds() {
        const schedule = this.clone();
        let previousEventId;
        let previousGroupEventId;
        for (const event of schedule.events) {
            if (!previousGroupEventId || !event.groupId) {
                event.setPreviousEventId(previousEventId);
                if (event instanceof primaryEvent_1.PlayEvent && event.groupId && (event.groupType === 'commercial')) {
                    previousGroupEventId = event.id;
                }
                else {
                    previousEventId = event.id;
                }
            }
            else {
                if (event instanceof primaryEvent_1.PlayEvent && (event.groupType !== 'commercial')) {
                    // first event after a group use the group header as the previous id
                    event.setPreviousEventId(previousEventId);
                    previousEventId = event.id;
                    previousGroupEventId = null;
                }
                else {
                    // when in a group use the previous group id
                    event.setPreviousEventId(previousGroupEventId);
                    previousGroupEventId = event.id;
                }
            }
        }
        return schedule;
    }
}
exports.Schedule = Schedule;
//
// const filtered = Schedule.fromJson({
//     channel: "ABC",
//     events: [
//         { id: "1", eventCategory: "Programme", startDate: BDateTime.fromIso("2021-02-01T10:00:00", "ndf25").toJson(), eventType: "play", duration: BDuration.fromTcString("00:05:00:00", "ndf25").toJson(), source: { sourceType: "media", materialId: "ABC", som: BDateTime.fromTimecode("10:00:00:00", "ndf25").toJson() } },
//         { id: "2", eventCategory: "Live",  startDate: BDateTime.fromIso("2021-02-01T10:30:00", "ndf25").toJson(), eventType: "play", duration: BDuration.fromTcString("00:05:00:00", "ndf25").toJson() },
//         { id: "3", eventCategory: "Programme", startDate: BDateTime.fromIso("2021-02-01T11:00:00", "ndf25").toJson(), eventType: "play", duration: BDuration.fromTcString("00:05:00:00", "ndf25").toJson(), source: { sourceType: "media", materialId: "ABC", som: BDateTime.fromTimecode("10:00:00:00", "ndf25").toJson() } }
//     ]
// })
//     .filterEventsByJsonPath("@.eventCategory==='Live'")
//     .filterEvents(e => e.eventCategory === "Programme")
//     .filterEvents(e => e.duration?.isGreaterThan(BDuration.fromTcString("00:00:02:00", "ndf25")))
//     .excludeRepeats(e => {
//         if ((e.source instanceof MediaSource) && (e.eventCategory === "Programme")) {
//             return e.source.materialId;
//         }
//     })
//     .updateEvents(e => {
//         if ((e.source instanceof MediaSource) && e.duration.isGreaterThan(BDuration.fromTcString("00:02:00:00", "ndf25"))) {
//             e.source.som = e.source.som?.subtractDuration(BDuration.fromTcString("00:00:05:00", "ndf25"))
//             e.duration = e.duration.addDuration(BDuration.fromTcString("00:00:10:00", "ndf25"))
//         }
//         return [e, e];
//     })
//     .retime()
//
//
// console.log(JSON.stringify(filtered.toJson(), null, 2))
