"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileSubscriber = void 0;
const path = require("path");
// import * as fs from "fs-extra-promise"
let fs;
// @ts-ignore
if (typeof window === 'undefined') {
    fs = require("fs-extra-promise");
}
const interfaces_1 = require("./interfaces");
const Log_1 = require("../logging/Log");
const BluebirdPromise = require("bluebird");
BluebirdPromise.config({ cancellation: true });
class FileSubscriber extends interfaces_1.AbstractSubscriber {
    constructor() {
        super(...arguments);
        this.blockSize = 1024 * 64; // don't change this, there are lots of reasons to keep it at 64K, see spencer if you think you need to change it
    }
    async init() {
    }
    async getFile(distantFile, localFile, options = {}) {
        const fileDetailsResult = await this.control.qqGetFileDetails({ file: distantFile });
        if (!fileDetailsResult.isError()) {
            const { size: fileSize } = fileDetailsResult.getResult();
            Log_1.Log.logTrace(`got file details for file ${distantFile} size ${fileSize}`);
            if (options.overwrite && await fs.existsAsync(localFile)) {
                await fs.unlinkAsync(localFile);
            }
            const fd = await fs.openAsync(localFile, options.overwrite ? 'w' : 'wx');
            let pos = 0;
            while (pos < fileSize) {
                const getFileBlockResult = await this.control.qqGetFileBlock({
                    blockSize: this.blockSize,
                    offset: pos,
                    file: distantFile
                });
                if (getFileBlockResult.isError()) {
                    Log_1.Log.logError(`file transfer failed at offset ${pos}`);
                    return;
                }
                else {
                    const { blockSize, block } = getFileBlockResult.getResult();
                    if (blockSize > 0) {
                        const buffer = Buffer.from(block);
                        if (pos + buffer.length > fileSize && pos <= fileSize) {
                            Log_1.Log.logWarn(`File size changed whilst transferring ${distantFile}`);
                            options?.fileSizeChanged?.();
                        }
                        await fs.writeAsync(fd, buffer, 0, buffer.length, pos);
                        pos += buffer.length;
                    }
                }
                const fileProgress = Math.round((pos * 100) / fileSize);
                const limitedProgress = Math.min(fileProgress, 100);
                options?.progress?.(limitedProgress);
            }
            await fs.closeAsync(fd);
        }
        else {
            Log_1.Log.logError(`failed to get file details ${distantFile}`);
        }
    }
    getFileAsArrayBuffer(distantFile, options = {}) {
        return new BluebirdPromise(async (resolve, reject, onCancel) => {
            let cancelled = false;
            onCancel(() => {
                cancelled = true;
                reject({ reason: 'cancelled' });
            });
            const fileDetailsResult = await this.control.qqGetFileDetails({ file: distantFile });
            if (!fileDetailsResult.isError()) {
                const { size: fileSize } = fileDetailsResult.getResult();
                Log_1.Log.logTrace(`got file details for file ${distantFile} size ${fileSize}`);
                let pos = 0;
                const arrayBuffer = new Uint8Array(fileSize);
                while (pos < fileSize && !cancelled) {
                    const getFileBlockResult = await this.control.qqGetFileBlock({
                        blockSize: this.blockSize,
                        offset: pos,
                        file: distantFile
                    });
                    if (getFileBlockResult.isError()) {
                        let error = `file transfer failed at offset ${pos}`;
                        Log_1.Log.logError(error);
                        return reject(error);
                    }
                    else {
                        const { blockSize, block } = getFileBlockResult.getResult();
                        if (blockSize > 0) {
                            let buffer = Buffer.from(block);
                            if (pos > fileSize) {
                                pos += buffer.length;
                                continue;
                            }
                            if (pos + buffer.length > fileSize) {
                                Log_1.Log.logWarn(`File size changed whilst transferring ${distantFile}`);
                                options?.fileSizeChanged?.();
                                buffer = buffer.slice(0, fileSize - pos - buffer.length);
                            }
                            arrayBuffer.set(buffer, pos);
                            pos += buffer.length;
                        }
                    }
                    const fileProgress = Math.round((pos * 100) / arrayBuffer.byteLength);
                    const limitedProgress = Math.min(fileProgress, 100);
                    options?.progress?.(limitedProgress);
                }
                resolve(arrayBuffer);
            }
            else {
                let error = `failed to get file details ${distantFile}`;
                Log_1.Log.logError(error);
                reject(error);
            }
        });
    }
    async writeFile(localFile, distantFile, options = {}) {
        const fileDescriptor = fs.openSync(localFile, 'r');
        const buffer = Buffer.alloc(this.blockSize);
        if (distantFile.endsWith("/")) {
            distantFile = path.join(distantFile, path.basename(localFile));
        }
        let writeOffset = 0;
        let offset = 0;
        let bytesRead = fs.readSync(fileDescriptor, buffer, 0, this.blockSize, null);
        const fileStats = fs.statSync(localFile);
        if (options.overwrite) {
            await this.control.qqDeleteFile({ file: distantFile });
        }
        let exists = false;
        try {
            const result = await this.control.qqGetFileDetails({ file: distantFile });
            if (result && !result.isError() && result.getResult() && result.getResult().size > -1) {
                if (options.append) {
                    const stats = result.getResult();
                    writeOffset = stats.size;
                }
            }
        }
        catch { }
        if (options.overwrite && exists) {
            throw new Error(`File ${distantFile} exists and delete failed`);
        }
        if (!options.append && exists) {
            throw new Error(`File ${distantFile} exists and append or overwrite not enabled`);
        }
        while (bytesRead != 0) {
            let data;
            if (bytesRead < this.blockSize) {
                data = buffer.slice(0, bytesRead);
            }
            else {
                data = buffer;
            }
            await this.control.qqWriteFileBlock({ file: distantFile, block: data.toJSON().data, offset: offset + writeOffset });
            offset = offset + bytesRead;
            bytesRead = fs.readSync(fileDescriptor, buffer, 0, this.blockSize, null);
            const fileProgress = Math.round((offset * 100) / fileStats.size);
            const limitedProgress = Math.min(fileProgress, 100);
            options?.progress?.(limitedProgress);
        }
    }
    writeArrayBuffer(fileName, arrayBuffer, distantFile, options = {}) {
        return new BluebirdPromise(async (resolve, reject, onCancel) => {
            let cancelled = false;
            if (distantFile.endsWith("/")) {
                distantFile = path.join(distantFile, path.basename(fileName));
            }
            onCancel(() => {
                cancelled = true;
                reject({ reason: 'cancelled' });
                try {
                    this.control.qqDeleteFile({ file: distantFile });
                }
                catch { }
            });
            if (options.overwrite) {
                await this.control.qqDeleteFile({ file: distantFile });
            }
            let writeOffset = 0;
            let offset = 0;
            let bytesToRead = arrayBuffer.byteLength;
            let exists = false;
            try {
                const result = await this.control.qqGetFileDetails({ file: distantFile });
                if (result && !result.isError() && result.getResult() && result.getResult().size > -1) {
                    exists = true;
                    if (exists && options?.append) {
                        const stats = result.getResult();
                        writeOffset = stats.size;
                    }
                }
            }
            catch { }
            if (options?.overwrite && exists) {
                return reject(`File ${distantFile} exists and delete failed`);
            }
            if (!options?.append && exists) {
                return reject(`File ${distantFile} exists and append or overwrite not enabled`);
            }
            while (bytesToRead > 0 && !cancelled) {
                let data;
                data = arrayBuffer.slice(offset, offset + this.blockSize);
                await this.control.qqWriteFileBlock({ file: distantFile, block: Array.from(new Int8Array(data)), offset: offset + writeOffset });
                offset += this.blockSize;
                bytesToRead -= this.blockSize;
                const fileProgress = Math.round((offset * 100) / arrayBuffer.byteLength);
                const limitedProgress = Math.min(fileProgress, 100);
                options?.progress?.(limitedProgress);
            }
            resolve();
        });
    }
}
exports.FileSubscriber = FileSubscriber;
