"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericSubscriber = exports.BusbyError = exports.AbstractSubscriber = void 0;
const Log_1 = require("../../logging/Log");
const EsbResult_1 = require("../../esbInterface/EsbResult");
const SubscriberManager_1 = require("./SubscriberManager");
const luxon_1 = require("luxon");
const lodash_1 = require("lodash");
const SubscriberInterface_1 = require("./SubscriberInterface");
class AbstractSubscriber {
    static get(serviceName, statusProcessor, subscriberOptions, ...args) {
        return SubscriberManager_1.SubscriberManager.get().getSubscriber(serviceName, this, statusProcessor, subscriberOptions, ...args);
    }
    static async getConnected(serviceName, statusProcessor, subscriberOptions, ...args) {
        return await SubscriberManager_1.SubscriberManager.get().getConnectedSubscriber(serviceName, this, statusProcessor, subscriberOptions, ...args);
    }
    /**
     * An absract subscriber which connects to the bus to allow you to retrieve status messages and send control messages to services
     * @deprecated consider using the static method `get()` and async `getConnected()` instead
     * @param serviceName The service name to connect to
     * @param loginDetails The login details to use to construct the subscriber
     * @param statusProcessor A processor to listen to status events on
     * @param subscriber You can provide a different subscriber if the real subscriber is to be wrapped in some way (e.g. websockets)
     * @param subscriberOptions Any additional subscriber options which are required
     */
    constructor(serviceName, loginDetails, statusProcessor, subscriber = 'subscriber', //you can provide your own subscriber here if you wanted to wrap the real subscriber in some way (e.g. websockets)
    subscriberOptions, ...args) {
        this.serviceName = serviceName;
        this.loginDetails = loginDetails;
        this.defaultControlOptions = {};
        this.control = new Proxy({}, {
            get: (t, message) => {
                return (params = {}, options) => {
                    return this.post(message, params, options);
                };
            }
        });
        this.status = new Proxy({}, {
            get: (t, event) => {
                return (processor) => {
                    this.onStatusEvent(event, processor);
                    return {
                        dispose: () => this.offStatusEvent(event, processor),
                    };
                };
            }
        });
        this.onStatusEvent = (event, processor) => {
            Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:listeners:status:on`)(event);
            if (event === "*" || event === "#") {
                if (this.statusProcessors.size === 0) {
                    this.subscriber.addStatusSubscription(event);
                }
                this.statusProcessors.add(processor);
            }
            else {
                if (!this.statusEventProcessors[event]) {
                    this.statusEventProcessors[event] = new Set();
                }
                if (this.statusEventProcessors[event].size == 0) {
                    this.subscriber.addStatusSubscription(event);
                }
                this.statusEventProcessors[event].add(processor);
            }
            return { dispose: () => this.offStatusEvent(event, processor) };
        };
        this.offStatusEvent = (event, processor) => {
            Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:listeners:status:off`)(event);
            if (event === "*" || event === "#") {
                this.statusProcessors.delete(processor);
                if (this.statusProcessors.size === 0) {
                    this.subscriber.removeStatusSubscription(event);
                }
            }
            else if (this.statusEventProcessors[event]?.delete(processor)) {
                if (this.statusEventProcessors[event].size === 0) {
                    this.subscriber.removeStatusSubscription(event);
                }
            }
        };
        this.processStatus = (message, serviceId) => {
            const messageName = message.getMessageName();
            const debug = Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:status:${messageName}`);
            const debugParams = Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:status:${messageName}:params`);
            const processors = this.statusEventProcessors[messageName];
            debug(message.getSequenceNumber(), processors?.size ?? 0, message.getFrom(), message.getTimeStamp().toISOString());
            debugParams(message.getSequenceNumber(), message.getParams());
            if (processors?.size > 0) {
                for (const processor of processors) {
                    if (this.statusProcessors.has(processor)) {
                        // If processor is also in the processor list continue to prevent duplicates
                        continue;
                    }
                    try {
                        processor(message, serviceId);
                    }
                    catch (e) {
                        Log_1.Log.logError(`Failed to run on status processor for ${messageName}`, e);
                    }
                }
            }
        };
        this.onConnected = () => {
            Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:connected`)("connected", this.connectedProcessors.size);
            this.connectedAt = luxon_1.DateTime.utc();
            let i = 0;
            for (const processor of this.connectedProcessors.values()) {
                try {
                    processor();
                }
                catch (e) {
                    Log_1.Log.logError(`Failed to run connected processor ${i}`, e);
                }
                i++;
            }
        };
        this.onDisconnected = () => {
            Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:connected`)("disconnected", this.disconnectedProcessors.size);
            this.disconnectedAt = luxon_1.DateTime.utc();
            let i = 0;
            for (const processor of this.disconnectedProcessors.values()) {
                try {
                    processor();
                }
                catch (e) {
                    Log_1.Log.logError(`Failed to run disconnected processor ${i}`, e);
                }
                i++;
            }
        };
        this.onStatusMessage = (msg, service) => {
            this.lastStatusAt = luxon_1.DateTime.utc();
            this.statusMessages++;
            try {
                this.processStatus(msg, service);
            }
            catch (e) {
                Log_1.Log.logError(`Failed to run abstract status processor`, e);
            }
            let i = 0;
            for (const processor of this.statusProcessors.values()) {
                try {
                    processor(msg, service);
                }
                catch (e) {
                    Log_1.Log.logError(`Failed to run status processor ${i}`, e);
                }
                i++;
            }
        };
        this.onDirectoryChanged = (service, provider, state) => {
            Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:directory`)("change", this.directoryChangedProcessors.size, service, provider, state);
            this.disconnectedAt = luxon_1.DateTime.utc();
            let i = 0;
            for (const processor of this.directoryChangedProcessors.values()) {
                try {
                    processor(service, provider, state);
                }
                catch (e) {
                    Log_1.Log.logError(`Failed to run directory change processor ${i}`, e);
                }
                i++;
            }
        };
        /**
         * Adds a function to run when the subscriber has connected.
         * This will run the function when added if the subscriber is currently connected
         * @param onConnected callback to run when subscriber connects
         */
        this.addConnectedProcessor = (onConnected) => {
            if (!this.connectedProcessors.has(onConnected)) {
                Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:listeners:connected`)("on");
                this.connectedProcessors.add(onConnected);
            }
            if (this.isConnected()) {
                onConnected(true);
            }
            return { dispose: () => this.removeConnectedProcessor(onConnected) };
        };
        /**
         * Removes a connected callback which was added to the subscriber
         * @param onConnected callback which was added for when subscriber connected
         */
        this.removeConnectedProcessor = (onConnected) => {
            if (this.connectedProcessors.has(onConnected)) {
                Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:listeners:connected`)("off");
                this.connectedProcessors.delete(onConnected);
            }
        };
        /**
         * Adds a function to run when the subscriber disconnects.
         * This will run the function when added if the subscriber is currently disconnected
         * @param onDisconnected callback to run when subscriber disconnects
         */
        this.addDisconnectedProcessor = (onDisconnected) => {
            if (!this.disconnectedProcessors.has(onDisconnected)) {
                Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:listeners:disconnected`)("on");
                this.disconnectedProcessors.add(onDisconnected);
            }
            if (!this.isConnected()) {
                onDisconnected(true);
            }
            return { dispose: () => this.removeDisconnectedProcessor(onDisconnected) };
        };
        /**
         * Removes a disconnected callback which was added to the subscriber
         * @param onDisconnected callback which was added for when subscriber disconnects
         */
        this.removeDisconnectedProcessor = (onDisconnected) => {
            if (this.disconnectedProcessors.has(onDisconnected)) {
                Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:listeners:disconnected`)("off");
                this.disconnectedProcessors.delete(onDisconnected);
            }
        };
        /**
         * Adds a function to run when a status message is received
         * @deprecated @see this.status for an alternative method to add status processors for specidfic messages
         * @param processor callback to run when subscriber recieves a status message
         */
        this.addStatusProcessor = (processor) => {
            return this.onStatusEvent('*', processor);
        };
        /**
         * Removes a function which was running when a status message was received
         * @param processor callback which was added to run when subscriber recieved a status message
         * @deprecated
         */
        this.removeStatusProcessor = (processor) => {
            this.offStatusEvent('*', processor);
        };
        /**
         * Removes a function from all handlers
         * @param processor callback which was added to run when subscriber recieved a status message
         */
        this.clearStatusProcessor = (processor) => {
            this.offStatusEvent('*', processor);
            for (const key of Object.keys(this.statusEventProcessors)) {
                this.offStatusEvent(key, processor);
            }
        };
        /**
         * Adds a function to run when the directory status of the subscriber has changed
         * @param processor callback to run when the directory status has changed
         */
        this.addDirectoryStatusProcessor = (processor) => {
            if (!this.directoryChangedProcessors.has(processor)) {
                Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:listeners:directory`)("on");
                this.directoryChangedProcessors.add(processor);
            }
            return { dispose: () => this.removeDirectoryStatusProcessor(processor) };
        };
        /**
         * Removes a function which was running when the directory status of the subscriber has changed
         * @param processor callback which was added to run when the directory status has changed
         */
        this.removeDirectoryStatusProcessor = (processor) => {
            if (this.directoryChangedProcessors.has(processor)) {
                Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:listeners:directory`)("off");
                this.directoryChangedProcessors.delete(processor);
            }
        };
        this.connectErrors = 0;
        const SelectedSubscriber = typeof subscriber === "string" ? SubscriberInterface_1.subscriberMap[subscriber] : subscriber;
        if (!SelectedSubscriber) {
            const err = `Subscriber type [${typeof subscriber === "string" ? subscriber : ""}] does not exist or is not intialised. If running from NodeJS import from @busby/esb/node to setup NodeJS support. If running from a plugin please initialise the PluginClient first.`;
            console.error(err);
            throw new Error(err);
        }
        this.statusProcessors = new Set();
        this.directoryChangedProcessors = new Set();
        this.connectedProcessors = new Set();
        this.disconnectedProcessors = new Set();
        this.statusEventProcessors = {};
        this.statusMessages = 0;
        this.createdAt = luxon_1.DateTime.utc();
        this.disconnectedAt = luxon_1.DateTime.utc();
        if (subscriberOptions?.onConnected) {
            this.addConnectedProcessor(subscriberOptions.onConnected);
        }
        if (subscriberOptions?.onDisconnected) {
            this.addDisconnectedProcessor(subscriberOptions.onDisconnected);
        }
        if (subscriberOptions?.onDirectoryChange) {
            this.addDirectoryStatusProcessor(subscriberOptions.onDirectoryChange);
        }
        subscriberOptions = {
            ...subscriberOptions,
            onConnected: this.onConnected,
            onDisconnected: this.onDisconnected,
            onDirectoryChange: this.onDirectoryChanged,
            abstract: true,
            lambdaMode: true,
        };
        this.setDefaultControlOptions();
        this.subscriber = new SelectedSubscriber(serviceName, this.loginDetails, this.onStatusMessage, subscriberOptions);
        if (statusProcessor) {
            this.addStatusProcessor(statusProcessor);
        }
        if (this.onCreated) {
            this.onCreated();
        }
    }
    onCreated() { }
    setDefaultControlOptions(options) {
        this.defaultControlOptions = options ?? {};
    }
    /**
     * Connects to the service.
     * It would be better to call `waitUntilConnected` which handles retrys.
     */
    async connect() {
        Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:connect`)(`called`);
        try {
            if (!this.serviceName) {
                throw new Error(`No service name set - ${this.constructor.name}`);
            }
            const connected = await this.subscriber.connect(this.serviceName);
            if (!connected) {
                throw new Error(this.serviceName + ' failed to connect');
            }
            this.connectErrors = 0;
            try {
                for (const key of Object.keys(this.statusEventProcessors)) {
                    this.subscriber.addStatusSubscription(key);
                }
            }
            catch { }
            return connected;
        }
        catch (error) {
            this.connectErrors++;
            if (this.connectErrors <= 10) {
                if (this.connectErrors === 10) {
                    Log_1.Log.logError(`Suppressing connect errors for ${this.serviceName}`);
                }
                Log_1.Log.logError(error);
            }
            throw new Error(error);
        }
    }
    /**
     * Wait until the service is connected before returning
     */
    async waitUntilConnected() {
        if (this.isConnected()) {
            return true;
        }
        Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:wait`)(`called`);
        return this.subscriber.waitUntilConnected(this.serviceName);
    }
    /**
     * Check whether the subsciber is connected
     */
    isConnected() {
        return this.subscriber?.isConnected() ?? false;
    }
    /**
     * Sends a command to the connected service
     * @param messageName The message to send
     * @param params params object to send with the request
     * @param options additional options for the message
     */
    async _sendControl(messageName, params = {}, options) {
        const debug = Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:control:${messageName}`);
        const debugRequest = Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:control:${messageName}:request`);
        const debugResponse = Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:control:${messageName}:response`);
        debug('send');
        debugRequest((0, lodash_1.omit)(params, "password", "token"));
        Log_1.Log.logESB("Sending busby message [" + messageName + "] to service [" + this.serviceName + "] with params: " + JSON.stringify(params));
        const reply = await this.subscriber.sendControl(messageName, params, Object.assign({}, this.defaultControlOptions, options ?? {}));
        Log_1.Log.logESB("Received busby reply: " + JSON.stringify(reply));
        debug('recv', reply?.getSequenceNumber(), reply?.getFrom(), reply?.getTimeStamp().toISOString());
        debugResponse(reply?.getSequenceNumber(), reply?.getParams());
        return reply;
    }
    /**
     * Run a command on the connected service
     * @param messageName The message to send
     * @param params params object to send with the request
     * @param options additional options for the message
     * @throws `Error` if unable to connect or a timeout
     */
    async post(messageName, params = {}, options) {
        const esbMessage = await this._sendControl(messageName, params, options);
        if (!esbMessage) {
            throw new Error(`No Response to message ${messageName}`);
        }
        return new EsbResult_1.EsbResult(esbMessage);
    }
    /**
     * Sends a command to the connected service
     * @deprecated consider using `this.control` proxy instead which only throws if it timesout or unable to connect
     * @param messageName The message to send
     * @param params params object to send with the request
     * @param options additional options for the message
     * @throws `Error` if unable to connect or a timeout
     * @throws `BusbyError` if response is an error
     */
    async sendControl(messageName, params = {}, options) {
        try {
            const reply = await this._sendControl(messageName, params, options);
            if (reply?.isError()) {
                throw new BusbyError('Message [' + messageName + '] failed with the following response: ' + (reply?.getErrorParams()?.reason ?? "") + " - " + JSON.stringify(reply), reply);
            }
            else if (!reply) {
                throw new Error('No reply to message');
            }
            else {
                return reply;
            }
        }
        catch (e) {
            if (e instanceof BusbyError) {
                throw e;
            }
            else {
                throw new Error(e);
            }
        }
    }
    /**
     * Send a command to the connected service
     * @see control to get the result without throwing an error
     * @deprecated consider using `this.control` proxy instead, as that only throws an error when there is no response and not on any error
     * @param messageName The message to send
     * @param params params object to send with the request
     * @param options additional options for the message
     * @throws `Error` if unable to connect or a timeout
     * @throws `BusbyError` if response is an error
     */
    send(messageName, params = {}, options) {
        return this.sendControl(messageName, params, options)
            .then(esbMessage => {
            if (!esbMessage) {
                throw new Error(`No Response to message ${messageName}`);
            }
            if (esbMessage.isError()) {
                // Not sure it will ever get here
                throw new Error(esbMessage.getErrorParams().reason);
            }
            return new EsbResult_1.EsbResult(esbMessage);
        });
    }
    getConfig() {
        return this.subscriber.getConfig();
    }
    getConfigOverview() {
        return this.subscriber.getConfigOverview();
    }
    lookupConfig(lookupId) {
        return this.subscriber.lookupConfig(lookupId);
    }
    getServiceName() {
        return this.serviceName;
    }
    async close() {
        Log_1.Log.getNamespace(`abstract:${this.constructor.name}:${this.serviceName}:close`)("close", this.getStatus());
        this.statusProcessors = new Set();
        this.connectedProcessors = new Set();
        this.disconnectedProcessors = new Set();
        this.directoryChangedProcessors = new Set();
        this.statusEventProcessors = {};
        this.statusMessages = 0;
        return await this.subscriber.close();
    }
    getSubscriber() {
        return this.subscriber;
    }
    getStatus() {
        return {
            created: this.createdAt?.toISO(),
            connected: this.isConnected(),
            connectedAt: this.connectedAt ? `${this.connectedAt.toISO()} (${this.connectedAt.toRelative()})` : "Never",
            disconnectedAt: this.disconnectedAt ? `${this.disconnectedAt.toISO()} (${this.disconnectedAt.toRelative()})` : "Never",
            lastStatusAt: this.lastStatusAt ? `${this.lastStatusAt.toISO()} (${this.lastStatusAt.toRelative()})` : "Never",
            statusMessages: this.statusMessages,
            listeners: {
                status: this.statusProcessors.size,
                statusMessages: (0, lodash_1.mapValues)(this.statusEventProcessors, (v) => v.size),
                connected: this.connectedProcessors.size,
                disconnected: this.disconnectedProcessors.size,
            }
        };
    }
    static connectSendClose(loginDetails, serviceId, messageName, params, subscriberType = 'subscriber', subscriberOptions) {
        const SelectedSubscriber = SubscriberInterface_1.subscriberMap[subscriberType];
        if (subscriberType === 'wsSubscriber' || subscriberType === 'pluginSubscriber') {
            return SelectedSubscriber.connectSendClose(loginDetails, serviceId, messageName, params, subscriberOptions);
        }
        else {
            return SelectedSubscriber.connectSendClose(loginDetails, serviceId, messageName, params);
        }
    }
    static connectSendCloseSpecific(loginDetails, providerId, messageName, params, subscriberType = 'subscriber', subscriberOptions) {
        const SelectedSubscriber = SubscriberInterface_1.subscriberMap[subscriberType];
        if (subscriberType === 'wsSubscriber' || subscriberType === 'pluginSubscriber') {
            return SelectedSubscriber.connectSendCloseSpecific(loginDetails, providerId, messageName, params, subscriberOptions);
        }
        else {
            return SelectedSubscriber.connectSendCloseSpecific(loginDetails, providerId, messageName, params);
        }
    }
}
exports.AbstractSubscriber = AbstractSubscriber;
class BusbyError extends Error {
    constructor(message, esbMessage) {
        super(message);
        this.esbMessage = esbMessage;
    }
}
exports.BusbyError = BusbyError;
class GenericSubscriber extends AbstractSubscriber {
}
exports.GenericSubscriber = GenericSubscriber;
