"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupSubscriber = exports.Subscriber = exports.NodeSubscriber = void 0;
const uuid = require("uuid");
const MultiRabbitConnection_1 = require("../../esbInterface/MultiRabbitConnection");
const LoginDetails_1 = require("../../esbInterface/LoginDetails");
const Log_1 = require("../../logging/Log");
const async_mutex_1 = require("async-mutex");
const SubscriberInterface_1 = require("./SubscriberInterface");
class NodeSubscriber {
    static async makeSubscriber(subscriberId, statusPocessor) {
        const subscriber = new NodeSubscriber(subscriberId, LoginDetails_1.LoginDetails.get(), statusPocessor);
        return subscriber;
    }
    constructor(subscriberId, loginDetails, statusProcessor, subscriberOptions) {
        this.statusProcessor = statusProcessor;
        this.subscriberOptions = subscriberOptions;
        this.serviceId = null;
        this.providerId = null;
        this.reconnectPoll = null;
        this.reconnectCount = 0;
        this.defaultTimeout = 60;
        this.subscriberActive = false;
        this.shouldBeConnected = true;
        this.messageQueue = {};
        this.subscriberId = loginDetails.getProviderId() + "_" + subscriberId + "_" + uuid.v4();
        this.baseSubscriberId = subscriberId;
        this.configId = subscriberId;
        this.rabbitConnection = new MultiRabbitConnection_1.RabbitConnection(loginDetails, false);
        this.rabbitConnection.setSubId(this.subscriberId);
        this.messageQueueMutex = new async_mutex_1.Mutex();
    }
    getServiceId() {
        return this.serviceId;
    }
    getSubsciberId() {
        return this.subscriberId;
    }
    getSubscriberId() {
        return this.subscriberId;
    }
    getBaseSubscriberId() {
        return this.baseSubscriberId;
    }
    getProviderId() {
        return this.rabbitConnection.getProviderId();
    }
    setConfigChangedUpdater(configChanged) {
        this.rabbitConnection.setConfigChangedUpdater(configChanged);
    }
    setDirectoryStatusProcessor(dsp) {
        this.rabbitConnection.setDirectoryStatusProcessor(dsp);
    }
    setDirectoryChangedProcessor(directoryChanged) {
        this.rabbitConnection.listenDirectoryChanged(directoryChanged);
    }
    saveProviderConfig(configData, providerId, host) {
        this.rabbitConnection.saveProviderConfig(configData, providerId, host);
    }
    addStatusSubscription(messageName) {
        this.rabbitConnection.addStatusBinding(messageName);
        return {
            dispose: () => this.removeStatusSubscription(messageName)
        };
    }
    removeStatusSubscription(messageName) {
        this.rabbitConnection.removeStatusBinding(messageName);
    }
    getStatusRoutingKeys() {
        return this.rabbitConnection.getStatusRoutingKeys();
    }
    getProtocolVersion() {
        return this.rabbitConnection.getProtocolInfo().version;
    }
    async onSubscriberActive(active, location) {
        if (this.subscriberActive !== active) {
            Log_1.Log.getNamespace(`subscriber:${this.serviceId}:active`)(active, location);
            this.subscriberActive = active;
            if (!active) {
                this.subscriberOptions?.onDisconnected?.();
                if (this.shouldBeConnected) {
                    await this.reconnect();
                }
            }
            else {
                if (this.reconnectPoll) {
                    clearTimeout(this.reconnectPoll);
                }
                this.subscriberOptions?.onConnected?.();
            }
        }
    }
    async reconnect() {
        const providerId = this.providerId ?
            await this.rabbitConnection.setupSubscriberProvider(this.providerId, this.statusProcessor, this.onSubscriberActive.bind(this), this.subscriberOptions).catch(err => { })
            :
                await this.rabbitConnection.setupSubscriber(this.serviceId, this.statusProcessor, this.onSubscriberActive.bind(this), this.subscriberOptions).catch(err => { });
        if (!providerId) {
            if (this.reconnectPoll) {
                clearTimeout(this.reconnectPoll);
            }
            this.reconnectPoll = setTimeout(() => this.connecting = this.reconnect(), 10000);
        }
        else {
            clearTimeout(this.reconnectPoll);
            this.connecting = null;
        }
        return !!providerId;
    }
    async connect(serviceId) {
        this.shouldBeConnected = true;
        if (serviceId != undefined) {
            this.serviceId = serviceId;
        }
        let result = this.isConnected();
        if (!result) {
            let connecting = this.connecting;
            if (!connecting) {
                if (this.serviceId) {
                    if (!this.isConnected()) {
                        Log_1.Log.getNamespace(`subscriber:${this.serviceId}:connect`)("connecting");
                    }
                    connecting = this.reconnect();
                    this.connecting = connecting;
                }
                else {
                    Log_1.Log.logError("service name not set");
                }
            }
            if (connecting) {
                result = await connecting;
                this.connecting = null;
            }
        }
        return result;
    }
    async connectProvider(providerId) {
        this.shouldBeConnected = true;
        this.providerId = providerId;
        this.serviceId = `direct-${this.providerId}`;
        let result = this.isConnected();
        if (!result) {
            let connecting = this.connecting;
            if (!connecting) {
                if (this.serviceId) {
                    if (!this.isConnected()) {
                        Log_1.Log.getNamespace(`subscriber:${this.serviceId}:connect`)("connecting");
                    }
                    connecting = this.reconnect();
                    this.connecting = connecting;
                }
                else {
                    Log_1.Log.logError("service name not set");
                }
            }
            if (connecting) {
                result = await connecting;
            }
        }
        return result;
    }
    async waitUntilConnected(serviceId) {
        if (serviceId != undefined) {
            this.serviceId = serviceId;
        }
        this.shouldBeConnected = true;
        if (!this.waitingPromise) {
            Log_1.Log.getNamespace(`subscriber:${serviceId}:wait`)(`called`);
            this.waitingPromise = new Promise(async (resolve, reject) => {
                let reconnects = 0;
                let timeout;
                const tryConnect = async () => {
                    let connected = false;
                    try {
                        connected = await this.connect(serviceId);
                        if (connected && reconnects > 0) {
                            reconnects = 0;
                        }
                        if (connected) {
                            Log_1.Log.getNamespace(`subscriber:${serviceId}:wait`)(`connected`);
                        }
                    }
                    catch (e) {
                        reconnects++;
                        if (reconnects < 10) {
                            Log_1.Log.logError(`Could not connect ${serviceId}, waiting to re-try`);
                        }
                        else if (reconnects == 10) {
                            Log_1.Log.logError(`Could not connect ${serviceId}, waiting to re-try, suppressing errors to every 100 attempts`);
                        }
                        else if (reconnects % 100 === 0) {
                            Log_1.Log.logError(`Could not connect ${serviceId}, waiting to re-try, attempts ${reconnects}`);
                        }
                    }
                    if (connected || !this.shouldBeConnected) {
                        resolve(!!connected);
                        clearTimeout(timeout);
                        this.waitingPromise = null;
                    }
                    else {
                        timeout = setTimeout(tryConnect, 5000);
                    }
                };
                await tryConnect();
            });
        }
        return this.waitingPromise;
    }
    isConnected() {
        const connected = this.subscriberActive && this.rabbitConnection.isSubscriberConnected();
        Log_1.Log.getNamespace(`subscriber:${this.serviceId}:isConnected`)(connected);
        return connected;
    }
    async sendControl(messageName, params = {}, options = {}) {
        Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:mutex`)(messageName, "acquire enter");
        let release = await this.messageQueueMutex.acquire();
        Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:mutex`)(messageName, "acquired enter");
        try {
            let strParams = "";
            if (options.batch) {
                strParams = messageName + "::" + JSON.stringify(params);
                const current = this.messageQueue[strParams];
                if (current) {
                    Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:batch`)(messageName, strParams);
                    Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:mutex`)(messageName, "release existing batch");
                    release();
                    release = null;
                    return current;
                }
            }
            if (await this.connect()) {
                const message = this.rabbitConnection.sendControl(messageName, params, options?.timeoutSecs ?? this.defaultTimeout);
                if (options.batch) {
                    this.messageQueue[strParams] = message;
                }
                Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:mutex`)(messageName, "release sent");
                release();
                release = null;
                const result = await message;
                if (options.batch) {
                    Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:mutex`)(messageName, "acquire batch delete");
                    release = await this.messageQueueMutex.acquire();
                    Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:mutex`)(messageName, "acquired batch delete");
                    delete this.messageQueue[strParams];
                    Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:mutex`)(messageName, "release batch delete");
                    release();
                    release = null;
                }
                return result;
            }
            else {
                return null;
            }
        }
        catch (err) {
            Log_1.Log.logError(`Failed sending control`, err);
        }
        finally {
            Log_1.Log.getNamespace(`subscriber:${this.serviceId}:sendControl:mutex`)(messageName, "release finally");
            release?.();
        }
    }
    sendDs(messageName, params = {}) {
        return this.connect()
            .then(() => {
            return this.rabbitConnection.sendControlMessageToDirectoryService(messageName, params);
        });
    }
    publishConfig(configData) {
        return this.rabbitConnection.publishConfig(configData);
    }
    getCurrentSystemConfig() {
        return this.rabbitConnection.getCurrentSystemConfig();
    }
    getCurrentSurveyConfig() {
        return this.rabbitConnection.getCurrentSurveyConfig();
    }
    getSystemConfig(version) {
        return this.rabbitConnection.getSystemConfig(version);
    }
    getConfigOverview() {
        return this.rabbitConnection.getConfigOverview();
    }
    async saveSystemConfig(configData, comment, user) {
        const reply = await this.rabbitConnection.saveSystemConfig(configData, comment, user);
        return reply.getParams().version;
    }
    async saveSurveyConfig(configuration, comment) {
        const reply = await this.rabbitConnection.saveSurveyConfig(configuration, comment);
        return reply.getParams().version;
    }
    getSystemConfigVersions() {
        return this.rabbitConnection.getSystemConfigVersions();
    }
    getSystemDescription() {
        return this.rabbitConnection.getSystemDescription();
    }
    getConfig() {
        return this.rabbitConnection.getConfiguration(this.getProviderId())
            .then(configuration => {
            this.rabbitConnection.setConfigName(this.getProviderId());
            return configuration;
        });
    }
    lookupConfig(subscriberId) {
        return this.rabbitConnection.getConfiguration(subscriberId);
    }
    async close() {
        this.shouldBeConnected = false;
        clearTimeout(this.reconnectPoll);
        this.reconnectPoll = null;
        return await this.rabbitConnection.close();
    }
    static async connectSendClose(loginDetails, serviceId, messageName, params, options) {
        const subscriber = new NodeSubscriber(serviceId + "_send", loginDetails);
        let response;
        try {
            const connected = await subscriber.connect(serviceId);
            if (connected) {
                response = await subscriber.sendControl(messageName, params, options);
            }
        }
        catch (err) {
        }
        finally {
            await subscriber.close().catch(err => { });
        }
        if (!response) {
            return Log_1.Log.logErrorAndReject('cannot connect to service ' + serviceId + ' to send ' + messageName);
        }
        return response;
    }
    static async connectSendCloseDirect(loginDetails, providerId, messageName, params, options) {
        const subscriber = new NodeSubscriber(providerId + "_directsend", loginDetails);
        let response;
        try {
            const connected = await subscriber.connectProvider(providerId);
            if (connected) {
                response = await subscriber.sendControl(messageName, params, options);
            }
        }
        catch (err) {
        }
        finally {
            await subscriber.close().catch(err => { });
        }
        if (!response) {
            return Log_1.Log.logErrorAndReject('cannot connect to service ' + providerId + ' to send ' + messageName);
        }
        return response;
    }
}
exports.NodeSubscriber = NodeSubscriber;
exports.Subscriber = NodeSubscriber;
const setupSubscriber = () => {
    SubscriberInterface_1.subscriberMap['subscriber'] = NodeSubscriber;
};
exports.setupSubscriber = setupSubscriber;
SubscriberInterface_1.subscriberMap['subscriber'] = NodeSubscriber;
