"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupSubscriber = exports.PluginSubscriber = void 0;
const EsbMessage_1 = require("../../esbInterface/EsbMessage");
const Configuration_1 = require("../../esbInterface/Configuration");
const Log_1 = require("../../logging/Log");
const uuid_1 = require("uuid");
const SubscriberInterface_1 = require("./SubscriberInterface");
const Debug = require("debug");
const luxon_1 = require("luxon");
const BasePluginClient_1 = require("../../plugins/BasePluginClient");
class PluginSubscriber {
    /**
     * @deprecated Should not be called directly. Use the `.get()` or `.getConnected()` functions on Subscribers or use the `SubscriberManager`.
     * The SubscriberManager keeps track of subscribers and only generates one subscriber and status processor
     */
    constructor(subscriberId, loginDetails, statusProcessor = null, subscriberOptions) {
        this.loginDetails = loginDetails;
        this.statusProcessor = statusProcessor;
        this.subscriberOptions = subscriberOptions;
        this.serviceId = null;
        this._connected = false;
        this._connectedBeforeSetup = false;
        this.statusSubscriptions = new Set();
        this.subscriberBackChannel = (message) => {
            Log_1.Log.getNamespace(`pluginSubscriber:${this.serviceId}:backchannel:${message.messageName}`)(message.connected, message);
            switch (message.messageName) {
                case "subscriberStatusChange":
                    {
                        this.connected = message.connected;
                    }
                    break;
                case "directoryChanged":
                    {
                        this.subscriberOptions?.onDirectoryChange?.(message.service, message.provider, message.state);
                    }
                    break;
            }
        };
        this.initalConnection = false;
        this.subscriberId = subscriberId + "_plugin_" + (0, uuid_1.v4)();
        this.subscriberSetup = false;
        this.pluginSubscriberInterface = this.subscriberOptions.pluginSubscriberInterface;
        let params = { subscriberId: this.subscriberId, loginDetails };
        if (this.statusProcessor) {
            const statusProcessorId = this.subscriberId + '_status';
            const processor = (messageJson) => {
                this.statusProcessor(EsbMessage_1.EsbMessage.makeFromJson(messageJson));
                return null;
            };
            this.pluginSubscriberInterface.registerStatusProcessor(statusProcessorId, processor);
            params['statusProcessorId'] = statusProcessorId;
            this.statusProcessorId = statusProcessorId;
        }
        this.pluginSubscriberInterface.registerSubscriberBackChannelProcessor(this.subscriberId, this.subscriberBackChannel);
        const messageName = 'construct';
        this.pluginSubscriberInterface.emit({ messageName, params })
            .then((result) => {
            if (!result) {
                console.error('Failed to construct plugin subscriber');
                throw new Error('Failed to construct plugin subscriber');
            }
        });
    }
    get connected() {
        return this._connected;
    }
    set connected(value) {
        const { onDisconnected, onConnected } = this.subscriberOptions;
        const hasChanged = this._connected !== value;
        if (hasChanged && this.messageProcessorId) {
            this._connected = value;
            if (value) {
                onConnected && onConnected();
            }
            else {
                onDisconnected && onDisconnected();
                if (this.waitEnabled && !this.waitingPromise) {
                    this.waitUntilConnected(this.serviceId);
                }
            }
            if (!this.initalConnection) {
                this.initalConnection = true;
            }
        }
        else if (!this.messageProcessorId) {
            this._connectedBeforeSetup = value;
        }
    }
    async connect(serviceId) {
        if (this.initalConnection) {
            return this.connected;
        }
        if (serviceId) {
            this.serviceId = serviceId;
        }
        const messageName = 'connect';
        const params = {
            serviceId: this.serviceId,
            subscriberId: this.subscriberId,
            statusProcessorId: this.statusProcessorId,
            subscriptions: Array.from(this.statusSubscriptions),
        };
        Log_1.Log.getNamespace(`pluginSubscriber:${this.serviceId}:connect`)(this.connected, params);
        const reply = await this.pluginSubscriberInterface.emit({ messageName, params });
        if (this.messageProcessorId && this.messageProcessorId !== reply) {
            console.error(`Already had a proccessor ID of [${this.messageProcessorId}] but received [${reply}]`);
        }
        Log_1.Log.getNamespace(`pluginSubscriber:${this.serviceId}:connect:response`)(this.connected || this._connectedBeforeSetup, reply, this.messageProcessorId);
        this.messageProcessorId = reply;
        if (!this.initalConnection && this._connectedBeforeSetup) {
            this.connected = true;
        }
        this.initalConnection = true;
        return this.connected;
    }
    async waitUntilConnected(serviceName) {
        this.serviceId = serviceName;
        this.waitEnabled = true;
        if (this.isConnected()) {
            return true;
        }
        Log_1.Log.getNamespace(`pluginSubscriber:${this.serviceId}:wait`)(`called`);
        if (!this.waitingPromise) {
            this.waitingPromise = new Promise(async (resolve, reject) => {
                if (!BasePluginClient_1.BasePluginClient.get().isConnected()) {
                    let pluginConnected;
                    const pluginConnection = new Promise(async (resolve) => {
                        pluginConnected = resolve;
                    });
                    BasePluginClient_1.BasePluginClient.get().onConnected(pluginConnected);
                    await pluginConnection;
                    BasePluginClient_1.BasePluginClient.get().offConnected(pluginConnected);
                }
                let timeout;
                let success = false;
                module?.hot?.dispose(() => {
                    clearTimeout(timeout);
                });
                const tryConnect = async (suppress = false) => {
                    const connected = await this.connect(this.serviceId)
                        .catch((e) => {
                        Log_1.Log.logError(`Could not connect to ${this.serviceId}, waiting to re-try`);
                    });
                    if (connected) {
                        success = true;
                        resolve(true);
                        clearTimeout(timeout);
                        this.waitingPromise = null;
                        // this.subscriberOptions?.onConnected();
                    }
                    else {
                        success = false;
                        if (!suppress) {
                            Log_1.Log.logWarn(`Not connected to ${this.serviceId} yet, waiting to re-try`);
                        }
                        timeout = setTimeout(tryConnect, 2000, !success);
                    }
                };
                tryConnect();
            });
        }
        return this.waitingPromise;
    }
    addStatusSubscription(messageName) {
        if (!this.statusSubscriptions.has(messageName)) {
            this.statusSubscriptions.add(messageName);
            const params = { subscriberId: this.subscriberId, messageName };
            this.pluginSubscriberInterface.emit({ messageName: "addStatusSubscription", params });
            return {
                dispose: () => this.removeStatusSubscription(messageName)
            };
        }
        return {
            dispose: () => this.removeStatusSubscription(messageName)
        };
    }
    removeStatusSubscription(messageName) {
        if (this.statusSubscriptions.delete(messageName)) {
            const params = { subscriberId: this.subscriberId, messageName };
            this.pluginSubscriberInterface.emit({ messageName: "removeStatusSubscription", params });
        }
    }
    isConnected() {
        return this.connected;
    }
    async sendControl(messageName, params = {}, options) {
        const message = { messageName, params };
        if (process.env.NODE_ENV === 'development' && !Debug.enabled("busby:log")) {
            console.debug('Sending ESB message', message, this.messageProcessorId);
        }
        const start = luxon_1.DateTime.utc();
        if (await this.connect()) {
            return this.pluginSubscriberInterface.sendMessage(this.messageProcessorId, { ...message, options })
                .then((reply) => {
                if (process.env.NODE_ENV === 'development' && !Debug.enabled("busby:log")) {
                    console.debug('Received ESB reply', reply, '- ms taken: ', start.diffNow().valueOf() * -1);
                }
                return reply ? EsbMessage_1.EsbMessage.makeFromJson(reply) : null;
            });
        }
        else {
            return null;
        }
    }
    getConfig() {
        return null;
    }
    async getConfigOverview() {
        const messageName = 'getConfigOverview';
        const params = { subscriberId: this.subscriberId };
        const reply = await this.pluginSubscriberInterface.emit({ messageName, params });
        return EsbMessage_1.EsbMessage.makeFromJson(reply);
    }
    async lookupConfig(lookupId) {
        const messageName = 'lookupConfig';
        const params = { subscriberId: this.subscriberId, lookupId };
        const reply = await this.pluginSubscriberInterface.emit({ messageName, params });
        return Configuration_1.Configuration.makeFromJson(reply);
    }
    close() {
        const messageName = 'close';
        const params = { subscriberId: this.subscriberId };
        this.pluginSubscriberInterface.emit({ messageName, params });
        this.waitEnabled = false;
        this.connected = false;
    }
    static async connectSendClose(loginDetails, serviceId, esbMessageName, esbParams, subscriberOptions = { pluginSubscriberInterface: this.subscriberInterface }) {
        const messageName = 'connectSendClose';
        const params = { loginDetails, serviceId, messageName: esbMessageName, params: esbParams };
        if (process.env.NODE_ENV === 'development' && !Debug.enabled("busby:log")) {
            console.debug('Sending connectSendClose message', messageName, params);
        }
        const reply = await subscriberOptions.pluginSubscriberInterface.emit({ messageName, params });
        if (process.env.NODE_ENV === 'development' && !Debug.enabled("busby:log")) {
            console.debug('Received connectSendClose reply', reply);
        }
        return EsbMessage_1.EsbMessage.makeFromJson(reply);
    }
    static setSubscriberInterface(subscriberInterface) {
        this.subscriberInterface = subscriberInterface;
    }
}
exports.PluginSubscriber = PluginSubscriber;
const setupSubscriber = () => {
    SubscriberInterface_1.subscriberMap['pluginSubscriber'] = PluginSubscriber;
};
exports.setupSubscriber = setupSubscriber;
SubscriberInterface_1.subscriberMap['pluginSubscriber'] = PluginSubscriber;
