"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubscriberManager = void 0;
const lodash_1 = require("lodash");
const esbInterface_1 = require("../../esbInterface");
const Log_1 = require("../../logging/Log");
const AbstractSubscriber_1 = require("./AbstractSubscriber");
const lookup = new Function(`try {
    if (this === global) {
        // We're in node
        return 'subscriber';
    }
    if (window) {
        if (window.parent === window) {
            // We're the main selector page
            return 'wsSubscriber';
        }
        // Different parent, must be a plugin
        return 'pluginSubscriber';
    }
    // One of the above conditions should be true
    return null;
} catch(e){
    return 'wsSubscriber';
}`);
class SubscriberManager {
    /**
     * This is provided for the use of configEditor which needs to be able to connect
     * to differnt systems.
     * @deprecated Only for use in Apps which connect to multiple systems.
     * @see SubscriberManager.get for normal usage
     */
    static getManager(loginDetails, type, defaultSubscriberOptions) {
        if (!loginDetails) {
            loginDetails = esbInterface_1.LoginDetails.get();
        }
        if (!type) {
            type = lookup();
            if (!type) {
                throw new Error(`Unable to work out type of subscriber`);
            }
        }
        if (type === "subscriber" && loginDetails.getProviderId() === esbInterface_1.LoginDetails.get().getProviderId()) {
            return this.get(type, defaultSubscriberOptions);
        }
        if (!this._instances) {
            this._instances = {};
        }
        if (!this._instances[loginDetails.getHosts()[0]]) {
            this._instances[loginDetails.getHosts()[0]] = new SubscriberManager(type, defaultSubscriberOptions, loginDetails);
        }
        return this._instances[loginDetails.getHosts()[0]];
    }
    static get(type, defaultSubscriberOptions) {
        if (!type && !this._instance) {
            type = lookup();
            if (!type) {
                throw new Error(`Unable to work out type of subscriber`);
            }
            Log_1.Log.logTrace(`Subscriber manager calculated type: ${type}`);
        }
        else if (type && this._instance && this._instance?.type !== type) {
            throw new Error(`Subscriber manager started with type ${this._instance.type} but you are requsting ${type}`);
        }
        if (!this._instance) {
            this._instance = new SubscriberManager(type, defaultSubscriberOptions);
        }
        return this._instance;
    }
    constructor(type, defaultSubscriberOptions, loginDetails) {
        this.type = type;
        this.defaultSubscriberOptions = defaultSubscriberOptions;
        this.loginDetails = loginDetails;
        this.newSubscriberCallbacks = new Set();
        this.onNewSubscriber = (callback) => {
            this.newSubscriberCallbacks.add(callback);
            return {
                dispose: () => this.offNewSubscriber(callback)
            };
        };
        this.offNewSubscriber = (callback) => {
            this.newSubscriberCallbacks.delete(callback);
        };
        this.subscribers = {};
        this.loginDetails = this.loginDetails ?? esbInterface_1.LoginDetails.get();
    }
    getSubscriberInstance(serviceName, subscriberClass, statusProcessor, subscriberOptions, ...args) {
        subscriberOptions = { ...this.defaultSubscriberOptions, ...subscriberOptions };
        if (!this.subscribers[serviceName]) {
            this.subscribers[serviceName] = {};
        }
        const type = (subscriberClass ?? AbstractSubscriber_1.GenericSubscriber).name;
        if (!this.subscribers[serviceName][type]) {
            Log_1.Log.getNamespace("subscriber:manager")("creating new subscriber", serviceName, type, this.type);
            // Although marked as deprecated this is how this class *needs* to get the 
            // subscriber otherwise it will just loop
            this.subscribers[serviceName][type] = new (subscriberClass ?? AbstractSubscriber_1.GenericSubscriber)(serviceName, this.loginDetails, statusProcessor, this.type, subscriberOptions, ...args);
            for (const callback of this.newSubscriberCallbacks) {
                try {
                    callback(this.subscribers[serviceName][type]);
                }
                catch { }
            }
        }
        if (subscriberOptions?.onConnected) {
            this.subscribers[serviceName][type].addConnectedProcessor(subscriberOptions.onConnected);
        }
        if (subscriberOptions?.onDisconnected) {
            this.subscribers[serviceName][type].addDisconnectedProcessor(subscriberOptions.onDisconnected);
        }
        if (statusProcessor) {
            this.subscribers[serviceName][type].addStatusProcessor(statusProcessor);
        }
        return this.subscribers[serviceName][type];
    }
    getSubscriber(serviceName, subscriberClass, statusProcessor, subscriberOptions, ...args) {
        const subscriber = this.getSubscriberInstance(serviceName, subscriberClass, statusProcessor, subscriberOptions, ...args);
        if (!subscriber.isConnected()) {
            subscriber.waitUntilConnected();
        }
        return subscriber;
    }
    async getConnectedSubscriber(serviceName, subscriberClass, statusProcessor, subscriberOptions, ...args) {
        const subscriber = this.getSubscriberInstance(serviceName, subscriberClass, statusProcessor, subscriberOptions, ...args);
        await subscriber.waitUntilConnected();
        return subscriber;
    }
    closeAll() {
        for (const service of (0, lodash_1.keys)(this.subscribers)) {
            for (const subscriber of (0, lodash_1.keys)(this.subscribers[service])) {
                this.subscribers[service][subscriber].close();
                delete this.subscribers[service][subscriber];
            }
            delete this.subscribers[service];
        }
    }
    reconnectAll() {
        for (const service of (0, lodash_1.keys)(this.subscribers)) {
            for (const subscriber of (0, lodash_1.keys)(this.subscribers[service])) {
                this.subscribers[service][subscriber].connect();
            }
        }
    }
    getAll() {
        return this.subscribers;
    }
}
exports.SubscriberManager = SubscriberManager;
