"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataTransform = exports.transformParentKeys = exports.transformKeys = void 0;
const _ = require("lodash");
const js2xml = require("js2xmlparser");
const JSONPath = require("jsonpath-plus");
const htmlEntities = require("html-entities");
const schedule_1 = require("../schedule/schedule");
const primaryEvent_1 = require("../schedule/primaryEvent");
const BDuration_1 = require("../bTime/BDuration");
const BDateTime_1 = require("../bTime/BDateTime");
const BusbyTableLookupSubscriber_1 = require("../subscribers/BusbyTableLookupSubscriber");
const secondaryEvent_1 = require("../schedule/secondaryEvents/secondaryEvent");
const graphicEvent_1 = require("../schedule/secondaryEvents/graphicEvent");
const recordEvent_1 = require("../schedule/secondaryEvents/recordEvent");
var deasync = null;
if (typeof window === 'undefined') {
    deasync = require('deasync-promise');
}
exports.transformKeys = {
    playlist: schedule_1.Schedule.getKeys(),
    play: primaryEvent_1.PlayEvent.getKeys(),
    graphic: graphicEvent_1.GraphicEvent.getKeys().concat(secondaryEvent_1.SecondaryEventBase.getKeys()),
    graphicField: graphicEvent_1.GraphicEvent.getFieldKeys(),
    record: recordEvent_1.RecordEvent.getKeys().concat(secondaryEvent_1.SecondaryEventBase.getKeys())
};
exports.transformParentKeys = {
    playlist: schedule_1.Schedule.getKeys(),
    play: schedule_1.Schedule.getKeys(),
    graphic: primaryEvent_1.PlayEvent.getKeys(),
    graphicField: graphicEvent_1.GraphicEvent.getKeys().concat(secondaryEvent_1.SecondaryEventBase.getKeys()),
    record: primaryEvent_1.PlayEvent.getKeys()
};
const entities = new htmlEntities.AllHtmlEntities();
class DataTransform {
    constructor() {
        this.helpers = {
            lookup: (table, inputField, outputField, inputValue) => {
                const row = deasync(this.tableLookupSubscriber.findRow({ tableName: table, searchParams: { [inputField]: inputValue } }));
                if (row && row.esbMessage && row.esbMessage.params) {
                    console.log('lookup', table, inputField, outputField, inputValue, row.esbMessage.params.result);
                    return row.esbMessage.params.result[outputField];
                }
                else {
                    return '';
                }
            }
        };
        this.scriptFns = {};
    }
    getRunFunction(code) {
        if (!this.scriptFns[code]) {
            this.scriptFns[code] = new Function(code);
        }
        return this.scriptFns[code];
    }
    runScript(script, object, metadata) {
        let result = null;
        if (script && script.code) {
            const context = {
                result: undefined,
                object,
                ...metadata,
                libs: {
                    _,
                    jsonpath: JSONPath.JSONPath,
                    BDuration: BDuration_1.BDuration,
                    BDateTime: BDateTime_1.BDateTime,
                    helpers: this.helpers
                }
            };
            if (!context.progenitor) {
                context.progenitor = object;
            }
            const code = "const {_, BDuration, BDateTime, helpers} = this.libs;\n" + script.code;
            try {
                this.getRunFunction(code).bind(context)();
                if (this.options.encodeChars) {
                    result = _.isString(context.result) ? entities.encode(context.result) : context.result;
                }
                else {
                    result = context.result;
                }
            }
            catch (e) {
                console.error(e);
            }
        }
        return result;
    }
    runTransformRuleElement(ruleElement, currentElement, rule, metadata) {
        let output = {};
        if (!ruleElement) {
            return;
        }
        // check that this element should be included
        if (this.runScript(ruleElement.includeScript, currentElement, metadata)) {
            // loop through all the values of this element
            for (const value of ruleElement.values) {
                if (value.children) { // if children is defined
                    if (_.has(currentElement, value.children.source)) { // and a source is defined then loop through all the array called children.source in the currentElement and put into the destination
                        metadata.depth++;
                        // the following assumes that there is only one type of child per parent for loop children
                        // this means you can't end up with XML that looks like
                        // <parent><childType1></childType1><childType2></childType2></parent>
                        // hopefully this will never be a problem
                        const target = value.children.destination;
                        const path = target ? value.path + "." + target : value.path;
                        if (target) {
                            _.set(output, [value.path, target], []);
                        }
                        else {
                            _.set(output, value.path, []);
                        }
                        for (const element of currentElement[value.children.source]) {
                            if (metadata.depth === 1) {
                                metadata.position++;
                                console.log(metadata.position);
                            }
                            for (const childId of value.children.ids) {
                                const ruleElement = rule[childId];
                                if (ruleElement && this.runScript(ruleElement.includeScript, element, metadata)) {
                                    metadata.parent = currentElement;
                                    const outputPath = _.get(output, path);
                                    if (outputPath) {
                                        _.get(output, path).push(this.runTransformRuleElement(ruleElement, element, rule, metadata));
                                    }
                                }
                            }
                        }
                        metadata.depth--;
                    }
                    else { // else if there are children with no source then don't change the current element
                        for (const childId of value.children.ids) {
                            const childRule = rule[childId];
                            const result = this.runTransformRuleElement(childRule, currentElement, rule, metadata);
                            if (!_.isEmpty(result)) {
                                if (!_.has(output, value.path)) {
                                    _.set(output, value.path, []);
                                }
                                _.get(output, value.path).push(result);
                            }
                        }
                    }
                }
                else { // if there are no child ids for this value then just calculate it
                    // Check to see if this value should be included
                    if (value.valueScript && this.runScript(value.includeScript, currentElement, metadata)) {
                        _.set(output, value.path, this.runScript(value.valueScript, currentElement, metadata));
                    }
                }
            }
        }
        return output;
    }
    async doTransform(data, rule, metadata) {
        return new Promise((resolve, reject) => {
            const topLevelRuleElement = _.find(rule.transforms, { topLevel: true });
            let result = null;
            if (topLevelRuleElement) {
                const transformMetadata = {
                    progenitor: data,
                    position: 0,
                    depth: 0,
                    metadata
                };
                result = this.runTransformRuleElement(topLevelRuleElement, data, rule.transforms, transformMetadata);
                if (rule.postProcessScript) {
                    try {
                        result = new Function(rule.postProcessScript).bind({
                            data: result,
                            js2xml, _
                        })();
                    }
                    catch (e) {
                        console.error(e);
                    }
                }
                if (rule.exportFormat) {
                    switch (rule.exportFormat.type) {
                        case "xml":
                            {
                                result = js2xml.parse(rule.exportFormat.options.topLevelElement, result);
                                resolve(result);
                            }
                            break;
                        case "json":
                            {
                                resolve(result);
                            }
                            break;
                        default: resolve(result);
                    }
                }
                else {
                    resolve(result);
                }
            }
            if (!result) {
                reject();
            }
        });
    }
    doNameTransform(data, rule, metadata) {
        let result;
        const metadataTransform = _.find(rule.transforms, { "transformKind": "metadata" });
        if (metadataTransform) {
            const outputNameScript = _.find(metadataTransform.values, { "path": "outputname" });
            if (outputNameScript) {
                result = this.runScript(outputNameScript.valueScript, data, metadata);
            }
        }
        return result;
    }
    async transform(data, rule, loginDetails, metadata) {
        let result;
        if (!this.tableLookupSubscriber) {
            this.tableLookupSubscriber = new BusbyTableLookupSubscriber_1.BusbyTableLookupSubscriber(loginDetails);
            await this.tableLookupSubscriber.connect();
        }
        this.options = _.get(rule, 'exportFormat.options', {});
        result = {
            output: await this.doTransform(data, rule, metadata),
            name: this.doNameTransform(data, rule, metadata)
        };
        return result;
    }
}
exports.DataTransform = DataTransform;
