"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RuleTransform = void 0;
const _ = require("lodash");
const luxon = require("luxon");
const luxon_1 = require("luxon");
const UUID = require("uuid");
const JSONPath = require("jsonpath-plus");
const subscribers_1 = require("../subscribers");
const Log_1 = require("../logging/Log");
const helperFunctions_1 = require("./helperFunctions");
const AsyncFunction = Object.getPrototypeOf(async function () { }).constructor;
const useMock = typeof window !== 'undefined';
class RuleTransform {
    constructor(tableLookupSubscriber, onGetEventsAt, onGetPreviousInputs) {
        this.onGetEventsAt = onGetEventsAt;
        this.onGetPreviousInputs = onGetPreviousInputs;
        this.sendMessage = async (serviceName, messageName, params, mock) => {
            if (useMock) {
                return mock;
            }
            return await subscribers_1.GenericSubscriber.get(serviceName).control[messageName](params)
                .then((res) => {
                return res;
            })
                .catch((err) => {
                Log_1.Log.logError(err);
                throw err;
            });
        };
        this.lookup = async (table, inputField, outputField, inputValue) => {
            return await this.tableLookupSubscriber.findRow({ tableName: table, searchParams: { [inputField]: inputValue } })
                .then((res) => {
                return res;
            })
                .catch((err) => {
                Log_1.Log.logError(err);
                throw err;
            });
        };
        this.tableLookupSubscriber = tableLookupSubscriber;
    }
    setScriptLibrary(scripts) {
        this.scripts = scripts;
    }
    async runScript(script, values, transformState) {
        this.transformState = transformState;
        const that = this;
        const context = {
            result: null,
            error: null,
            values,
            transformState,
            _,
            Log: Log_1.Log,
            luxon,
            UUID,
            jsonpath: ({ path, json, wrap }) => {
                return JSONPath.JSONPath({ path, json, wrap });
            },
            helpers: {
                getInputWhere: helperFunctions_1.getInputWhere,
                getInputOffset: helperFunctions_1.getInputOffset,
                getInputAt: helperFunctions_1.getInputAt.bind({ transformState, ...this }),
                getInputAtChildren: helperFunctions_1.getInputAtChildren.bind({ transformState, ...this }),
                getPreviousInstance: helperFunctions_1.getPreviousInstance.bind({ transformState, ...this }),
                getChildFieldWhere: helperFunctions_1.getChildFieldWhere,
                getIndexListWhere: helperFunctions_1.getIndexListWhere,
                getFieldFromObject: helperFunctions_1.getFieldFromObject,
                getChildValue: helperFunctions_1.getChildValue,
                log: helperFunctions_1.log,
                error: helperFunctions_1.error.bind(this),
                getGlobal: helperFunctions_1.getGlobal,
                setGlobal: helperFunctions_1.setGlobal,
                sendMessage: this.sendMessage.bind(that),
                lookup: this.lookup.bind(that)
            }
        };
        if (script) {
            try {
                if (!_.has(script, 'compiled')) {
                    script.compiled = new AsyncFunction("$$", script.code);
                }
                if (!_.has(script, 'compiledInit')) {
                    const initMatch = script.code.match(/\/\* init ([^\x05][^*\/]*)\*\//);
                    if (initMatch) {
                        script.compiledInit = new AsyncFunction("$$", initMatch[1]);
                    }
                }
                if (!_.has(script, 'compiledFinal')) {
                    const finalMatch = script.code.match(/\/\* final ([^\x05][^*\/]*)\*\//);
                    if (finalMatch) {
                        script.compiledFinal = new AsyncFunction("$$", finalMatch[1]);
                    }
                }
                try {
                    //  const result = this.sendMessageSync("alarmService", "getAlarmStatus", null).wait();
                    if (transformState.init && script.compiledInit) {
                        await script.compiledInit(context);
                    }
                    else if (transformState.final && script.compiledFinal) {
                        await script.compiledFinal(context);
                    }
                    else if (!transformState.init && !transformState.final) {
                        await script.compiled(context);
                    }
                }
                catch (e) {
                    Log_1.Log.logError("Script errored: " + e);
                }
                transformState.running = false;
                return transformState;
            }
            catch (e) {
                Log_1.Log.logError(script.code, e);
                throw e;
            }
        }
        else {
            throw "No script defined";
        }
    }
    async transform(inputValues, metadata = {}) {
        if (!this.scripts) {
            return;
        }
        // run through all of the values
        const transformState = {
            inputValues,
            errors: [],
            position: 1,
            metadata: {
                itemCount: inputValues.length,
                ...metadata
            },
            alarms: [],
            clearAlarms: [],
            jobs: [],
            result: [],
            globals: {},
            log: [],
        };
        let scriptLog = [];
        for (const script of this.scripts) {
            delete script.compiled;
            delete script.compiledInit;
            delete script.compiledFinal;
            let filtered = false;
            if (script.filter) {
                const result = JSONPath.JSONPath({ path: script.filter, json: inputValues });
                if (_.isEmpty(result)) {
                    Log_1.Log.logDebug("Ignore rule - filtered by " + script.filter);
                    filtered = true;
                }
            }
            if (!filtered) {
                const start = luxon_1.DateTime.utc().toFormat('HH:mm:ss.SSS');
                transformState.globals = {};
                transformState.position = 1;
                Log_1.Log.logInfo("timing: start " + start);
                await this.runScript(script, [], _.extend({}, transformState, { init: true }));
                Log_1.Log.logInfo("timing: after init " + luxon_1.DateTime.utc().toFormat('HH:mm:ss.SSS'));
                loop: for (const values of inputValues) {
                    await this.runScript(script, values, transformState);
                    // if(!_.isEmpty(transformState.errors)){
                    //     break loop;
                    // }
                    transformState.position++;
                }
                Log_1.Log.logInfo("timing: after loop " + luxon_1.DateTime.utc().toFormat('HH:mm:ss.SSS'));
                await this.runScript(script, [], _.extend({}, transformState, { final: true }));
                Log_1.Log.logInfo("timing: after end " + luxon_1.DateTime.utc().toFormat('HH:mm:ss.SSS'));
                const end = luxon_1.DateTime.utc().toFormat('HH:mm:ss.SSS');
                scriptLog.push({
                    start, end, rule: script.ruleName
                });
            }
        }
        Log_1.Log.logInfo("Run rules " + JSON.stringify(scriptLog));
        if (!_.isArray(inputValues)) {
            inputValues = [inputValues];
        }
        return transformState;
    }
}
exports.RuleTransform = RuleTransform;
