"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Transform = void 0;
const _ = require("lodash");
const luxon = require("luxon");
class Transform {
    setStructure(structure) {
        this.structure = structure;
    }
    setScriptLibrary(scripts) {
        this.scripts = scripts;
    }
    runScript(scriptUuid, values, metadata, transformState) {
        const context = {
            result: null,
            error: null,
            values,
            metadata,
            transformState,
            _,
            luxon
        };
        const script = this.scripts[scriptUuid];
        if (script) {
            if (!_.has(script, 'compiled')) {
                script.compiled = new Function("$$", script.code).bind(context);
            }
            return script.compiled(context);
        }
        else {
            return undefined;
        }
    }
    translateFields(values, metadata, fields, transformState) {
        const fieldResults = _.mapValues(fields, (field) => {
            try {
                let cont = true;
                if (field.metadata && field.metadata.blockScriptUuid) {
                    cont = this.runScript(field.metadata.blockScriptUuid, values, metadata, transformState);
                }
                if (cont) {
                    field.result = this.runScript(field.scriptUuid, values, metadata, transformState);
                }
            }
            catch (e) {
                transformState.errors.push({ message: e.message, input: values, metadata, scriptUuid: field.scriptUuid });
            }
            return field;
        });
        return _.omitBy(fieldResults, (field) => {
            return _.isUndefined(field.result);
        });
    }
    translateStructure(values, metadata, structure, transformState) {
        const result = [];
        for (const element of structure) {
            const include = this.runScript(element.scriptUuid, values, metadata, transformState);
            if (include) {
                const transformedElement = {
                    fields: this.translateFields(values, metadata, element.fields, transformState),
                    children: element.children ? this.translateStructure(values, metadata, element.children, transformState) : undefined
                };
                result.push(transformedElement);
            }
        }
        return result;
    }
    ;
    transform(inputValues, metadata) {
        if (!this.structure) {
            return;
        }
        if (!this.scripts) {
            return;
        }
        const resultStructure = [];
        // run through all of the values
        const transformState = {
            inputValues,
            errors: [],
            position: 1,
            globals: {}
        };
        const metadataStructure = _.filter(_.map(_.cloneDeep(this.structure), (element) => {
            element.fields = _.pickBy(element.fields, (field) => {
                return _.startsWith(field.name, "/_metadata");
            });
            return element;
        }), (elements) => !_.isEmpty(elements.fields));
        const structure = _.filter(_.map(_.cloneDeep(this.structure), (element) => {
            element.fields = _.pickBy(element.fields, (field) => {
                return !_.startsWith(field.name, "/_metadata");
            });
            return element;
        }), (elements) => !_.isEmpty(elements.fields));
        // Run through all of the input values
        for (const values of inputValues) {
            resultStructure.push(this.translateStructure(values, metadata, _.cloneDeep(structure), transformState));
            transformState.position++;
        }
        const outputMetadata = this.translateStructure(inputValues[0], metadata, _.cloneDeep(metadataStructure), transformState);
        return {
            result: resultStructure,
            state: transformState,
            metadata: outputMetadata
        };
    }
}
exports.Transform = Transform;
