"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PipelineActorFactory = exports.UpdateFileLocatorInfoBlockAction = exports.UploadToServerAction = exports.UploadToBrowserAction = exports.MatchChecker = exports.MetadataChecker = exports.WavChecker = exports.PipelineAction = void 0;
const _ = require("lodash");
const subscribers_1 = require("../subscribers");
const Uploader_1 = require("./Uploader");
const Utilities_1 = require("../Utilities");
const UploadManager_1 = require("./UploadManager");
class PipelineAction extends UploadManager_1.PipelineConfig {
}
exports.PipelineAction = PipelineAction;
class PipelineActor {
    constructor(options) {
        this.options = options;
    }
}
class WavChecker extends PipelineActor {
    constructor() {
        super(...arguments);
        this.act = async (action) => {
            try {
                const wavMeta = new Utilities_1.WavMeta(action.getFile());
                const metadata = await wavMeta.getMetadata();
                action.addMetadata(action.key, metadata);
                action.onComplete();
                return true;
            }
            catch (e) {
                action.onError(e.message);
                return false;
            }
        };
    }
}
exports.WavChecker = WavChecker;
class MetadataChecker extends PipelineActor {
    constructor(options) {
        super(options);
        this.act = async (action) => {
            const metadataToCheck = action.getMetadata()?.[this.options.keyToCheck];
            if (_.isEmpty(metadataToCheck)) {
                action.onError("Missing metadata");
                return false;
            }
            else if (!_.isMatch(action.getMetadata()[this.options.keyToCheck], this.options.pattern)) {
                action.onError("Invalid metadata");
                return false;
            }
            return true;
        };
    }
}
exports.MetadataChecker = MetadataChecker;
class MatchChecker extends PipelineActor {
    constructor(options) {
        super(options);
        this.act = async (action, data) => {
            let match = false;
            for (const row of data) {
                const dropMatch = _.template(this.options.pattern);
                const dropRegex = new RegExp(dropMatch(row));
                const result = dropRegex.exec(action.getFile().name);
                if (result) {
                    match = true;
                    // add the job to the metadata
                    action.addMetadata(action.key, row);
                    break;
                }
            }
            if (!match) {
                action.onError("No id match");
            }
            return match;
        };
    }
}
exports.MatchChecker = MatchChecker;
class UploadToBrowserAction extends PipelineActor {
    constructor() {
        super(...arguments);
        this.act = async (action) => {
            return new Promise((resolve, reject) => {
                const fileReader = new FileReader();
                fileReader.onprogress = (ev) => {
                    action.onProgress(ev.loaded, ev.total);
                };
                fileReader.onload = async (ev) => {
                    action.onComplete();
                    resolve(true);
                };
                fileReader.onerror = (ev) => {
                    action.onError(ev.target.error.message);
                };
                fileReader.onabort = (ev) => {
                    action.onError("Aborted");
                    reject("aborted");
                };
                fileReader.readAsDataURL(action.getFile());
            });
        };
    }
}
exports.UploadToBrowserAction = UploadToBrowserAction;
class UploadToServerAction extends PipelineActor {
    constructor() {
        super(...arguments);
        this.act = async (action) => {
            return new Promise((resolve, reject) => {
                Uploader_1.Uploader.getInstance().upload(action.getFile(), {
                    progress: (progress) => {
                        action.onProgress(progress, 100);
                    },
                    error: (message) => {
                        action.onError(message);
                        reject(message);
                    },
                    finish: () => {
                        action.onComplete();
                        resolve(true);
                    }
                });
            });
        };
    }
}
exports.UploadToServerAction = UploadToServerAction;
class UpdateFileLocatorInfoBlockAction extends PipelineActor {
    constructor() {
        super(...arguments);
        this.act = async (action) => {
            const fileLocator = {
                location: this.options?.uploadLocation ?? "selectorUpload",
                filename: action.getFile().name,
                path: ""
            };
            action.onProgress(10, 100);
            const stateStore = await subscribers_1.StateStoreSubscriber.getConnected(this.options?.stateStore ?? "stateStore");
            await stateStore.updateJobInfo({
                jobUuid: action.getMetadata().job?.job.uuid,
                infoType: "json",
                subType: this.options?.subType ?? "tempUploadFileLocation",
                info: {
                    content: fileLocator
                }
            });
            action.onProgress(100, 100);
            return true;
        };
    }
}
exports.UpdateFileLocatorInfoBlockAction = UpdateFileLocatorInfoBlockAction;
const pipelineActors = {
    MetadataChecker,
    MatchChecker,
    WavChecker,
    UploadToBrowserAction,
    UploadToServerAction,
    UpdateFileLocatorInfoBlockAction
};
class PipelineActorFactory {
    static createPipelineActor(action, options) {
        if (!pipelineActors[action]) {
            throw new Error(`Action type ${action} not registered`);
        }
        return new pipelineActors[action](options);
    }
}
exports.PipelineActorFactory = PipelineActorFactory;
