"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UploadManager = exports.PipelineConfig = void 0;
const _ = require("lodash");
const logging_1 = require("../logging");
const UploadActors_1 = require("./UploadActors");
class PipelineConfig {
}
exports.PipelineConfig = PipelineConfig;
class UploadManager {
    constructor(pipeline) {
        this.uploadStatus = [];
        this.onChangeCallbacks = [];
        this.onCompleteCallbacks = [];
        this.pipelineError = (file, pipelineConfig) => {
            return (message) => {
                const status = this.getUploadStatusForFile(file);
                status.error = {
                    message
                };
                this.sendUpdate();
            };
        };
        this.pipelineProgress = (file, pipelineConfig) => {
            return (loaded, total) => {
                const status = this.getUploadStatusForFile(file);
                if (pipelineConfig.progressLimit) {
                    const percentageWidth = pipelineConfig.progressLimit[1] - pipelineConfig.progressLimit[0];
                    const progress = pipelineConfig.progressLimit[0] + (loaded / total) * percentageWidth;
                    status.progress = progress;
                }
                this.log("progress", status, status.progress, file);
                this.sendUpdate();
            };
        };
        this.pipelineComplete = (file, pipelineConfig) => {
            return () => {
            };
        };
        this.addMetadata = (file) => {
            const status = this.getUploadStatusForFile(file);
            return (key, metadata) => {
                _.merge(status.metadata, { [key]: metadata });
                this.sendUpdate();
            };
        };
        this.getMetadata = (file) => {
            return () => {
                const status = this.getUploadStatusForFile(file);
                return status.metadata;
            };
        };
        this.log = logging_1.Log.getNamespace("Upload");
        this.pipeline = pipeline;
    }
    setPipeline(pipeline) {
        this.pipeline = pipeline;
    }
    addOnChangeHandler(callback) {
        this.onChangeCallbacks.push(callback);
    }
    addOnCompleteHandler(callback) {
        this.onCompleteCallbacks.push(callback);
    }
    sendUpdate() {
        const status = this.uploadStatus;
        for (const callback of this.onChangeCallbacks) {
            callback(status);
        }
    }
    sendComplete(success) {
        for (const callback of this.onCompleteCallbacks) {
            callback({
                success,
                uploadStatus: this.uploadStatus
            });
        }
    }
    getUploadStatusForFile(file) {
        const existingStatus = _.find(this.uploadStatus, { name: file.name });
        if (existingStatus) {
            return existingStatus;
        }
        else {
            const status = {
                file,
                name: file.name,
                metadata: {}
            };
            this.uploadStatus.unshift(status);
            return status;
        }
    }
    makePipelineActionFromConfig(pipelineConfig, file) {
        return {
            ...pipelineConfig,
            action: UploadActors_1.PipelineActorFactory.createPipelineActor(pipelineConfig.name, pipelineConfig.options),
            onError: this.pipelineError(file, pipelineConfig),
            onProgress: this.pipelineProgress(file, pipelineConfig),
            onComplete: this.pipelineComplete(file, pipelineConfig),
            addMetadata: this.addMetadata(file),
            getMetadata: this.getMetadata(file),
            getFile: () => {
                return file;
            }
        };
    }
    getUploadStatus() {
        return this.uploadStatus;
    }
    async run(files, data) {
        const fileArray = Array.from(files);
        let allSuccess = true;
        this.uploadStatus = [];
        for (const file of fileArray) {
            let success = true;
            const status = this.getUploadStatusForFile(file);
            this.log("Upload", file.name);
            for (const pipelineConfig of this.pipeline) {
                if (success) {
                    this.log("Start pipeline", pipelineConfig.label);
                    status.pipeline = pipelineConfig;
                    const pipelineAction = this.makePipelineActionFromConfig(pipelineConfig, file);
                    try {
                        this.log("Before upload action:", pipelineAction?.label + ",", pipelineAction.getFile().name, status);
                        await pipelineAction.action.act(pipelineAction, data);
                        if (status.error) {
                            this.log("Error upload action:", pipelineAction?.label + ",", pipelineAction.getFile().name, status, status.error.message);
                        }
                        else {
                            this.log("After upload action:", pipelineAction?.label + ",", pipelineAction.getFile().name, status);
                        }
                        success = success && !status.error;
                    }
                    catch (e) {
                        this.pipelineError(file, pipelineConfig)(e.message);
                        this.log("Error upload action:", pipelineAction?.label + ",", pipelineAction.getFile().name, status, status.error.message);
                        success = false;
                    }
                    this.log("End pipeline", pipelineConfig.label, success, file);
                    if (!success) {
                        allSuccess = false;
                        this.log("Break pipeline on error");
                    }
                }
            }
            this.log("Uploaded", file.name);
        }
        this.sendComplete(allSuccess);
    }
}
exports.UploadManager = UploadManager;
