"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onConfigChanged = exports.addLaunchConfiguration = exports.JavaDebugConfigurationProvider = void 0;
const esb_1 = require("@busby/esb");
const _ = require("lodash");
const vscode = require("vscode");
const logging_1 = require("./logging");
const extensionUpdater_1 = require("./updater/extensionUpdater");
const ignored = [];
/**
 * @TODO add this config provider to get the java and node configs for running agaist a busby system
 * vscode.debug.registerDebugConfigurationProvider("java", new JavaDebugConfigurationProvider()),
 */
class JavaDebugConfigurationProvider {
    // Returns an initial debug configurations based on contextual information.
    provideDebugConfigurations(folder, token) {
        esb_1.Log.logInfo(`provideDebugConfigurations ${JSON.stringify(folder)}`);
        const configs = [];
        return configs;
    }
    // Try to add all missing attributes to the debug configuration being launched.
    resolveDebugConfiguration(_folder, config, _token) {
        esb_1.Log.logInfo(`resolveDebugConfiguration ${JSON.stringify(config)}`);
        return config;
    }
    // Try to add all missing attributes to the debug configuration being launched.
    resolveDebugConfigurationWithSubstitutedVariables(folder, config, token) {
        esb_1.Log.logInfo(`resolveDebugConfigurationWithSubstitutedVariables ${JSON.stringify(folder)} ${JSON.stringify(config)}`);
        return this.resolveAndValidateDebugConfiguration(folder, config, token);
    }
    async resolveAndValidateDebugConfiguration(folder, config, token) {
        let configCopy;
        const isConfigFromInternal = config.__origin === "internal" /** in-memory configuration from debugger */
            || config.__configurationTarget /** configuration from launch.json */;
        if (config.request === "launch" && isConfigFromInternal) {
            configCopy = _.cloneDeep(config);
            delete configCopy.__progressId;
            delete configCopy.noDebug;
        }
        try {
            // If no debug configuration is provided, then generate one in memory.
            if (this.isEmptyConfig(config)) {
                config.type = "java";
                config.name = "Java Debug";
                config.request = "launch";
            }
            if (config.request === "launch") {
                // VS Code internal console uses UTF-8 to display output by default.
                if (config.console === "internalConsole" && !config.encoding) {
                    config.encoding = "UTF-8";
                }
            }
            else {
                throw new Error(`Request type "${config.request}" is not supported. Only "launch" is supported.`);
            }
            if (token?.isCancellationRequested) {
                return undefined;
            }
            return config;
        }
        catch (ex) {
        }
    }
    /**
     * When VS Code cannot find any available DebugConfiguration, it passes a { noDebug?: boolean } to resolve.
     * This function judges whether a DebugConfiguration is empty by filtering out the field "noDebug".
     */
    isEmptyConfig(config) {
        return Object.keys(config).filter((key) => key !== "noDebug").length === 0;
    }
}
exports.JavaDebugConfigurationProvider = JavaDebugConfigurationProvider;
async function addLaunchConfiguration(folder) {
    if (!folder) {
        if ((vscode.workspace.workspaceFolders?.length ?? 0) > 1) {
            const picked = await vscode.window.showWorkspaceFolderPick();
            if (picked === undefined) {
                return;
            }
            esb_1.Log.logInfo(`${picked.uri}`);
            folder = picked.uri;
        }
        else if (vscode.workspace.workspaceFolders) {
            folder = vscode.workspace.workspaceFolders[0].uri;
        }
        else {
            return;
        }
    }
    const launchConfigurations = vscode.workspace.getConfiguration("launch", folder);
    esb_1.Log.logInfo(`Launch ${folder.toString()}`);
    const rawConfigs = launchConfigurations.configurations;
    const busbyLaunchConfig = containsBusbyConfig(rawConfigs);
    for (const config of rawConfigs) {
        if (config.request === "launch") {
            if (config.type === "java" && !config.args) {
                if (config.mainClass && config.projectName && !busbyLaunchConfig) {
                    const newConfig = _.cloneDeep(config);
                    delete newConfig.__configurationTarget;
                    newConfig.name = "Launch Program Busby";
                    newConfig.args = [
                        "${command:squaredpaper.pickBusbyCore}",
                        "squared",
                        "Squared1",
                        "${command:squaredpaper.pickProvider}"
                    ];
                    newConfig.env ?? (newConfig.env = {});
                    newConfig.env.JAVA_ENV = "development";
                    await addLaunchConfig(folder, newConfig);
                }
            }
            else {
                if (config.name === "Launch Program" && !config.args && !busbyLaunchConfig) {
                    const newConfig = _.cloneDeep(config);
                    delete newConfig.__configurationTarget;
                    newConfig.name = "Launch Program Busby";
                    newConfig.args = [
                        "${command:squaredpaper.pickBusbyCore}",
                        "squared",
                        "Squared1",
                        "${command:squaredpaper.pickProvider}"
                    ];
                    newConfig.env ?? (newConfig.env = {});
                    newConfig.env.NODE_ENV = "development";
                    await addLaunchConfig(folder, newConfig);
                }
            }
        }
    }
}
exports.addLaunchConfiguration = addLaunchConfiguration;
async function persistLaunchConfig(folder, oldConfig, newConfig, add) {
    const launchConfigurations = vscode.workspace.getConfiguration("launch", folder);
    const rawConfigs = launchConfigurations.configurations;
    const targetIndex = _.findIndex(rawConfigs, (config) => _.isEqual(config, oldConfig));
    if (targetIndex >= 0) {
        esb_1.Log.logInfo(`Persisting Launch Config ${targetIndex} ${folder?.toString()}`);
        rawConfigs[targetIndex] = newConfig;
        await launchConfigurations.update("configurations", rawConfigs);
    }
    else if (add) {
        esb_1.Log.logInfo(`Adding Launch Config ${folder?.toString()}`);
        rawConfigs.push(newConfig);
        await launchConfigurations.update("configurations", rawConfigs);
    }
}
async function addLaunchConfig(folder, newConfig) {
    const launchConfigurations = vscode.workspace.getConfiguration("launch", folder);
    const rawConfigs = _.cloneDeep(launchConfigurations.configurations);
    esb_1.Log.logInfo(`Adding Launch Config ${folder?.toString()} ${JSON.stringify(newConfig)} to ${JSON.stringify(rawConfigs)}`);
    rawConfigs.push(newConfig);
    try {
        await launchConfigurations.update("configurations", rawConfigs);
    }
    catch (err) {
        esb_1.Log.logError(`Faield updating launch ${err}`, err);
    }
}
const onConfigChanged = (context) => async (e) => {
    if (e.affectsConfiguration("squaredpaper.logging")) {
        (0, logging_1.updateLogLevels)();
    }
    if (e.affectsConfiguration("squaredpaper.updater")) {
        (0, extensionUpdater_1.setupUpdater)(context);
    }
    const folders = vscode.workspace.workspaceFolders ?? [];
    for (const folder of folders) {
        const check = e.affectsConfiguration("launch", folder);
        if (check) {
            const launchConfigurations = vscode.workspace.getConfiguration("launch", folder);
            const rawConfigs = launchConfigurations.configurations;
            const busbyLaunchConfig = containsBusbyConfig(rawConfigs);
            const current = context.workspaceState.get(`busby.ignoredConfigs`, []);
            if (!busbyLaunchConfig && !current.includes(folder.uri.path)) {
                vscode.window.showInformationMessage(`Add a Busby Launch Config for ${folder.name}?`, "Add", "Ignore")
                    .then((choice) => {
                    if (choice === "Ignore") {
                        const current = context.workspaceState.get(`busby.ignoredConfigs`, []);
                        current.push(folder.uri.path);
                        context.workspaceState.update(`busby.ignoredConfigs`, current);
                    }
                    else if (choice === "Add") {
                        vscode.commands.executeCommand("squaredpaper.addLaunchConfig", folder.uri);
                    }
                });
            }
        }
    }
};
exports.onConfigChanged = onConfigChanged;
function containsBusbyConfig(configs) {
    return configs.find(c => c.name === "Launch Program Busby" || (typeof c.args === "string" && c.args.startsWith("${command:squaredpaper.pickBusbyCore}")) || c.args?.[0] === "${command:squaredpaper.pickBusbyCore}");
}
//# sourceMappingURL=configurations.js.map