"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("@busby/esb/node");
const commander_1 = require("commander");
const repl = require("repl");
const util = require("util");
const fs = require("fs-extra");
const usage = () => {
    commander_1.program.outputHelp();
    process.exit(1);
};
let defaultRabbit = "localhost";
if (process.env.RABBIT) {
    defaultRabbit = process.env.RABBIT;
}
else if (fs.statSync("/opt/squaredpaper/busby.conf")) {
    try {
        const conf = fs.readJSONSync("/opt/squaredpaper/busby.conf");
        defaultRabbit = conf.corehost;
    }
    catch { }
}
const readFile = util.promisify(require("fs").readFile);
commander_1.program
    .version('2.0.0')
    .showHelpAfterError(true)
    .option('-r, --rabbit <user>', 'Host of the RabbitMQ cluster', defaultRabbit)
    .option('-u, --user <user>', 'Username to log in to RabbitMQ cluster', 'squared')
    .option('-p, --password <password>', 'Password for the RabbitMQ cluster', 'Squared1')
    .option('-f, --file [file]', 'Optional file of commands to send')
    .argument('<provider>', 'The provider to connect to on the bus')
    .argument('[command]', 'Optional command to send');
commander_1.program.parse(process.argv);
const options = commander_1.program.opts();
const providerName = commander_1.program.args[0];
const command = commander_1.program.args[1];
if (!providerName) {
    usage();
}
main();
async function main() {
    node_1.Log.setFileName("esbCli");
    node_1.Log.setLogLevel(node_1.LogLevel.TRACE, true);
    new node_1.LoginDetails(options['rabbit'], options['user'], options['password'], "esbCli");
    console.log(`Connecting to ${options['rabbit']} - (${options['user']}:${options['password']}) ${providerName}`);
    if (options.file) {
        processCliFile(options.file);
        process.exit(0);
    }
    if (command) {
        await cliCommand(command);
        process.exit(0);
    }
    await cliCommand("?");
    await getDescription();
    console.log(`Connected`);
    startRepl();
}
function processCliFile(filename) {
    readFile(filename, 'utf-8')
        .then((fileContents) => {
        const lines = fileContents.split("\n");
        for (const line of lines) {
            console.log(line);
            cliCommand(line);
        }
    });
    return "done";
}
let bus;
async function customEval(evalCmd, context, file, cb) {
    const commandLine = evalCmd.replace(/[\r\n]*/g, "");
    cb(null, await cliCommand(commandLine));
}
function startRepl() {
    bus = repl.start({
        prompt: `cli (${providerName}) >> `,
        ignoreUndefined: true,
        preview: false,
        eval: customEval,
        completer: async (line, callback) => {
            const result = await autoComplete(line);
            // console.debug(result)
            callback(null, result);
        },
    });
}
async function autoComplete(line) {
    const toCursor = line.substring(0, bus.cursor);
    const toCursorSplit = toCursor.split(" ");
    const args = bus.line.split(" ").filter(Boolean);
    const cmd = args.shift();
    if (!cmd || (toCursorSplit.length === 1 && !bus.line.endsWith(" "))) {
        return [availableCommands.filter(c => c.startsWith(toCursor)), bus.line];
    }
    toCursorSplit.shift();
    if (!availableCommands.includes(cmd)) {
        return [[], bus.line];
    }
    if (!commands[cmd]) {
        try {
            const reply = await node_1.Subscriber.connectSendCloseDirect(node_1.LoginDetails.get(), providerName, "qqCliCommand", { commandLine: `?? -c ${cmd}` });
            if (!reply || reply.isError()) {
            }
            else {
                const { result } = reply.getParams();
                result.pop();
                const description = JSON.parse(result.join(""));
                commands[cmd] = description;
            }
        }
        catch (err) {
            console.error("error getting completion data", cmd, err);
        }
    }
    if (!commands[cmd]) {
        return [[], bus.line];
    }
    const currArg = args[toCursorSplit.length - 1];
    const prevArg = args[toCursorSplit.length - 2];
    let param = "";
    let soFar = "";
    if (currArg && currArg.startsWith("-")) {
        param = currArg.replace(/^-/, "");
    }
    else if (prevArg && prevArg.startsWith("-")) {
        param = prevArg.replace(/^-/, "");
        soFar = currArg || "";
    }
    else {
        return [commands[cmd].params.filter((p) => !args.includes(`-${p.paramName}`)).map((p) => `-${p.paramName}`), param];
    }
    // console.log("%%", cmd, "~", args, "~", toCursorSplit, "~", currArg, "~", prevArg, "~", param, "~", soFar, "**")
    const unusedParams = commands[cmd].params.filter((p) => !args.includes(`-${p.paramName}`));
    let filteredParams = commands[cmd].params.filter((p) => p.paramName.startsWith(param));
    if (filteredParams.length === 0) {
        return [[], line];
    }
    else if (filteredParams.length === 1 && filteredParams[0].paramName === param) {
        // Process lookup for command
        if (currArg === `-${param}`) {
            return [[currArg], currArg];
        }
        const curr = filteredParams[0];
        switch (curr.type) {
            case "autoComplete": {
                return [(await getAutoComplete({ command: cmd, index: toCursorSplit.length - 1, args, param, soFar })).filter((l) => l.startsWith(soFar)), soFar];
            }
            case "boolean":
            case "enum": {
                return [curr.options.filter((v) => v.startsWith(soFar)), soFar];
            }
            case "string":
            case "number": {
                return [[], soFar];
            }
            case "flag": {
                filteredParams = commands[cmd].params.filter(p => p.paramName !== curr.paramName && !args.includes(`-${p.paramName}`));
                break;
            }
        }
    }
    else {
        filteredParams = filteredParams.filter(p => unusedParams.find(u => u.paramName === p.paramName));
    }
    return [filteredParams.map((p) => `-${p.paramName}`), "-"];
}
async function cliCommand(commandLine) {
    if (commandLine === ".exit") {
        process.exit(0);
    }
    const reply = await node_1.Subscriber.connectSendCloseDirect(node_1.LoginDetails.get(), providerName, "qqCliCommand", { commandLine }).catch(err => { });
    if (!reply) {
        console.error(`ERROR NO RESPONSE`);
    }
    else if (reply.isError()) {
        console.error(`ERROR ${reply?.getParams().reason}`);
    }
    else {
        const lines = reply.getParams().result;
        for (const line of lines) {
            if (line !== ">") {
                console.log(line);
            }
        }
    }
    if (commandLine === "?") {
        console.log("");
        console.log(".exit        Exit to shell");
    }
    return;
}
let commands = {};
let availableCommands = [];
async function getDescription() {
    const reply = await node_1.Subscriber.connectSendCloseDirect(node_1.LoginDetails.get(), providerName, "qqCliCommand", { commandLine: "??" }).catch(err => { });
    if (!reply) {
        console.error(`ERROR DESCRIBE NO RESPONSE`);
    }
    else if (reply.isError()) {
        console.error(`ERROR DESCRIBE ${reply?.getParams().reason}`);
    }
    else {
        const lines = reply.getParams().result;
        lines.pop();
        availableCommands = lines;
    }
    return;
}
const getAutoComplete = async ({ command, index, args, param, soFar }) => {
    const reply = await node_1.Subscriber.connectSendCloseDirect(node_1.LoginDetails.get(), providerName, 'cliAutoComplete', { command, index, args, param, soFar });
    if (!reply || reply.isError()) {
    }
    else {
        const { result } = reply.getParams();
        return result;
    }
    return [];
};
//# sourceMappingURL=esbCli.js.map