"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pickBusbyNode = void 0;
const ProviderState_1 = require("@busby/esb/lib/esbInterface/ProviderState");
const node_1 = require("@busby/esb/node");
const _ = require("lodash");
const vscode = require("vscode");
const vscode_1 = require("vscode");
/**
 * Shows a quick select box with all test cases defined in a given folder.
 * @param context
 */
async function pickBusbyNode(context) {
    let result = undefined;
    const quickPick = vscode_1.window.createQuickPick();
    quickPick.ignoreFocusOut = true;
    const providers = [];
    const lastNodes = context.workspaceState.get("lastNodes", []);
    if (lastNodes.length > 0) {
        for (const node of lastNodes) {
            providers.push({
                label: node,
                buttons: [
                    {
                        iconPath: new vscode.ThemeIcon('close'),
                        tooltip: "Remove",
                    }
                ]
            });
        }
        quickPick.activeItems = [providers[0]];
        providers.push({ kind: vscode_1.QuickPickItemKind.Separator, label: "" });
    }
    quickPick.items = [..._.sortBy(providers, "label")];
    quickPick.activeItems = [providers[0]];
    quickPick.title = "Select a node";
    let manager = null;
    let subscriber = null;
    let customEntry = null;
    const disposables = [];
    result = await new Promise(async (resolve) => {
        const disposable = vscode.Disposable.from(quickPick, quickPick.onDidChangeValue((value) => {
            if (!providers.find(h => h.label === value)) {
                customEntry = { label: quickPick.value, description: "Custom Node", };
                quickPick.items = [customEntry, ..._.sortBy(providers, "label")];
            }
            else {
                customEntry = null;
                quickPick.items = providers;
            }
        }), quickPick.onDidTriggerItemButton((e) => {
            providers.splice(providers.indexOf(e.item), 1);
            const current = quickPick.activeItems[0];
            quickPick.items = (customEntry ? [customEntry] : []).concat(_.sortBy(providers, "label"));
            quickPick.activeItems = [current];
            context.workspaceState.update("lastNodes", lastNodes.filter(h => h !== e.item.label));
        }), quickPick.onDidHide(() => {
            resolve(undefined);
            disposable.dispose();
        }), quickPick.onDidAccept(() => {
            const selection = quickPick.selectedItems[0];
            resolve(selection);
            disposable.dispose();
        }), {
            dispose: () => {
                for (const { dispose } of disposables) {
                    dispose?.();
                }
                subscriber?.close();
                manager?.closeAll();
            }
        });
        quickPick.show();
        const host = context.workspaceState.get("lastBusbyHost", "");
        if (host) {
            const previous = context.globalState.get(`busby.nodes.${host.replaceAll(".", "-")}`, []);
            for (const provider of previous) {
                if (!providers.find(p => p.label === provider)) {
                    providers.push({
                        label: provider,
                    });
                }
            }
            const current = quickPick.activeItems[0];
            quickPick.items = (customEntry ? [customEntry] : []).concat(_.sortBy(providers, "label"));
            quickPick.activeItems = [current];
            try {
                manager = node_1.SubscriberManager.getManager(node_1.LoginDetails.makeFromJson({
                    username: "squared",
                    password: "Squared1",
                    host,
                    providerId: host,
                }));
                subscriber = manager.getSubscriber("directory", node_1.BusbyDirectorySubscriber);
                const state = await Promise.race([
                    subscriber.control.getDirectoryState(undefined, { timeoutSecs: 5 }),
                    new Promise((resolve) => setTimeout(resolve, 10000))
                ]);
                if (state) {
                    const nodes = [];
                    const services = state.getResult().services ?? state.getResult();
                    for (const service of services) {
                        for (const provider of service.providers) {
                            if (provider.providerId.startsWith("Monitor_")) {
                                const node = provider.providerId.replaceAll("Monitor_", "");
                                nodes.push(node);
                                const description = `$(${provider.providerState === ProviderState_1.ProviderState.online ? "check" : "warning"}) ${provider.providerState}`;
                                const current = providers.find(p => p.label === node);
                                if (current) {
                                    current.description = description;
                                }
                                else {
                                    providers.push({
                                        label: node,
                                        description
                                    });
                                }
                            }
                        }
                    }
                    context.globalState.update(`busby.nodes.${host.replaceAll(".", "-")}`, nodes);
                    disposables.push(subscriber.addStatusProcessor((msg) => {
                        console.log(msg.getParams());
                    }));
                    disposables.push(subscriber.status.providerStateUpdate((msg) => {
                        const { providerState, providerId } = msg.getParams();
                        if (providerId.startsWith("Monitor_")) {
                            const current = providers.find(p => p.label === providerId.replace("Monitor_", ""));
                            if (current) {
                                current.description = `$(${providerState === "online" ? "check" : "warning"}) ${providerState}`;
                            }
                            const active = quickPick.activeItems[0];
                            quickPick.items = (customEntry ? [customEntry] : []).concat(_.sortBy(providers, "label"));
                            quickPick.activeItems = [active];
                        }
                    }));
                }
                else {
                    node_1.Log.logWarn("State not received");
                    vscode_1.window.showWarningMessage(`Could not find providers for host ${host}.`);
                }
                const current = quickPick.activeItems[0];
                quickPick.items = (customEntry ? [customEntry] : []).concat(_.sortBy(providers, "label"));
                quickPick.activeItems = [current];
            }
            catch (err) {
                node_1.Log.logError("Error getting providers", err);
            }
            finally {
                // subscriber?.close();
                // manager?.closeAll();
            }
        }
    });
    const chosen = result?.label;
    if (chosen) {
        const lastNodes = context.workspaceState.get("lastNodes", []);
        if (lastNodes.includes(chosen)) {
            lastNodes.splice(lastNodes.indexOf(chosen), 1);
        }
        lastNodes.unshift(chosen);
        const config = vscode.workspace.getConfiguration("squaredpaper.history");
        const history = config.get("node") ?? 0;
        if (history > 0) {
            if (lastNodes.length > history) {
                lastNodes.splice(history - 1, lastNodes.length - history);
            }
        }
        node_1.Log.logInfo(`Last Nodes Changed to ${lastNodes.length} ${lastNodes.join(", ")}`);
        context.workspaceState.update("lastNodes", lastNodes);
    }
    return chosen;
}
exports.pickBusbyNode = pickBusbyNode;
//# sourceMappingURL=pickBusbyNode.js.map