"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pickProvider = void 0;
const ProviderState_1 = require("@busby/esb/lib/esbInterface/ProviderState");
const node_1 = require("@busby/esb/node");
const _ = require("lodash");
const vscode = require("vscode");
const vscode_1 = require("vscode");
/**
 * Shows a quick select box with all test cases defined in a given folder.
 * @param context
 */
async function pickProvider(context, connect = false) {
    const activeIcon = !connect ? "warning" : "check";
    const inactiveIcon = connect ? "warning" : "check";
    let result = undefined;
    const quickPick = vscode_1.window.createQuickPick();
    quickPick.ignoreFocusOut = true;
    const providers = [];
    const lastProviders = context.workspaceState.get("lastProviders", []);
    if (lastProviders.length > 0) {
        for (const provider of lastProviders) {
            providers.push({
                label: provider,
                picked: lastProviders.indexOf(provider) === 0,
                buttons: [
                    {
                        iconPath: new vscode.ThemeIcon('close'),
                        tooltip: "Remove",
                    }
                ]
            });
        }
        quickPick.activeItems = [providers[0]];
        providers.push({ kind: vscode_1.QuickPickItemKind.Separator, label: "" });
    }
    quickPick.items = [...providers];
    quickPick.activeItems = [providers[0]];
    quickPick.title = "Select a provider";
    let manager = null;
    let subscriber = null;
    let customEntry = null;
    const disposables = [];
    result = await new Promise(async (resolve) => {
        const disposable = vscode.Disposable.from(quickPick, quickPick.onDidChangeValue((value) => {
            if (!providers.find(h => h.label === value)) {
                customEntry = { label: quickPick.value, description: "Custom Provider", };
                quickPick.items = [customEntry, ...providers];
            }
            else {
                customEntry = null;
                quickPick.items = providers;
            }
        }), quickPick.onDidTriggerItemButton((e) => {
            providers.splice(providers.indexOf(e.item), 1);
            const current = quickPick.activeItems[0];
            quickPick.items = (customEntry ? [customEntry] : []).concat(providers);
            quickPick.activeItems = [current];
            context.workspaceState.update("lastProviders", lastProviders.filter(h => h !== e.item.label));
        }), quickPick.onDidHide(() => {
            resolve(undefined);
            disposable.dispose();
        }), quickPick.onDidAccept(() => {
            const selection = quickPick.selectedItems[0];
            resolve(selection);
            disposable.dispose();
        }), {
            dispose: () => {
                for (const { dispose } of disposables) {
                    dispose?.();
                }
                subscriber?.close();
                manager?.closeAll();
            }
        });
        quickPick.show();
        const host = context.workspaceState.get("lastBusbyHost", "");
        if (host) {
            const previous = context.globalState.get(`busby.providers.${host.replaceAll(".", "-")}`, []);
            const prevProviders = [];
            for (const provider of previous) {
                if (!providers.find(p => p.label === provider)) {
                    prevProviders.push({
                        label: provider,
                    });
                }
            }
            providers.push(..._.sortBy(prevProviders, "label"));
            const current = quickPick.activeItems[0];
            quickPick.items = (customEntry ? [customEntry] : []).concat(providers);
            quickPick.activeItems = [current];
            try {
                manager = node_1.SubscriberManager.getManager(node_1.LoginDetails.makeFromJson({
                    username: "squared",
                    password: "Squared1",
                    host,
                    providerId: host,
                }));
                subscriber = manager.getSubscriber("directory", node_1.BusbyDirectorySubscriber);
                const state = await Promise.race([
                    subscriber.control.getDirectoryState(undefined, { timeoutSecs: 5 }),
                    new Promise((resolve) => setTimeout(resolve, 10000))
                ]);
                if (state) {
                    const providerIds = [];
                    const newProviders = [];
                    const services = state.getResult().services ?? state.getResult();
                    for (const service of services) {
                        for (const provider of service.providers) {
                            providerIds.push(provider.providerId);
                            const description = `$(${provider.providerState !== ProviderState_1.ProviderState.nocomms ? activeIcon : inactiveIcon}) ${provider.providerState}`;
                            const current = providers.find(p => p.label === provider.providerId);
                            if (current) {
                                current.description = description;
                            }
                            else {
                                newProviders.push({
                                    label: provider.providerId,
                                    description
                                });
                            }
                        }
                    }
                    providers.push(..._.sortBy(newProviders, "label"));
                    context.globalState.update(`busby.providers.${host.replaceAll(".", "-")}`, providerIds);
                    disposables.push(subscriber.addStatusProcessor((msg) => {
                        console.log(msg.getParams());
                    }));
                    disposables.push(subscriber.status.providerStateUpdate((msg) => {
                        const { providerState, providerId } = msg.getParams();
                        const current = providers.find(p => p.label === providerId);
                        if (current) {
                            current.description = `$(${providerState !== "nocomms" ? activeIcon : inactiveIcon}) ${providerState}`;
                        }
                        const active = quickPick.activeItems[0];
                        quickPick.items = (customEntry ? [customEntry] : []).concat(providers);
                        quickPick.activeItems = [active];
                    }));
                }
                else {
                    console.warn("State not received");
                    vscode_1.window.showWarningMessage(`Could not find providers for host ${host}.`);
                }
                const current = quickPick.activeItems[0];
                quickPick.items = (customEntry ? [customEntry] : []).concat(providers);
                quickPick.activeItems = [current];
            }
            catch (err) {
                console.error("Error getting providers", err);
            }
            finally {
                // subscriber?.close();
                // manager?.closeAll();
            }
        }
    });
    const chosen = result?.label;
    if (chosen) {
        const lastProviders = context.workspaceState.get("lastProviders", []);
        if (lastProviders.includes(chosen)) {
            lastProviders.splice(lastProviders.indexOf(chosen), 1);
        }
        lastProviders.unshift(chosen);
        const config = vscode.workspace.getConfiguration("squaredpaper.history");
        const history = config.get("provider") ?? 0;
        if (history > 0) {
            if (lastProviders.length > history) {
                lastProviders.splice(history - 1, lastProviders.length - history);
            }
        }
        node_1.Log.logInfo(`Last Providers Changed to ${lastProviders.length} ${lastProviders.join(", ")}`);
        context.workspaceState.update("lastProviders", lastProviders);
    }
    return chosen;
}
exports.pickProvider = pickProvider;
//# sourceMappingURL=pickProvider.js.map