"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkNewVersion = exports.setupUpdater = exports.ExtensionUpdater = exports.sleep = void 0;
const https = require("https");
const esb_1 = require("@busby/esb");
const semver = require("semver");
const fs = require("fs");
const vscode_1 = require("vscode");
const asyncTmp = require("./asynctmp");
const pkg = require("../../package.json");
// eslint-disable-next-line @typescript-eslint/naming-convention
const Log = esb_1.Log.getSubFile("updater");
function sleep(ms) {
    return new Promise(resolve => {
        setTimeout(resolve, ms);
    });
}
exports.sleep = sleep;
class ExtensionUpdater {
    constructor(context, options) {
        this.context = context;
        this.options = options;
        this.extensionManifest = context.extension.packageJSON;
        this.extensionFullName = this.extensionManifest.publisher + '.' + this.extensionManifest.name;
        this.installedExtensionVersionKey = this.extensionFullName + '.lastInstalledUpdaterVersion';
    }
    getExtensionManifest() {
        return this.extensionManifest;
    }
    async getNewVersionAndInstall() {
        const newVersion = await this.showProgress(`Checking for updates for ${this.extensionManifest.displayName}`, () => this.getNewVersion());
        if (newVersion) {
            if (this.options?.reInstall || await this.consentToInstall(newVersion)) {
                const vsixUri = await this.showProgress(`Downloading ${this.extensionManifest.displayName}`, () => this.download(newVersion.downloadUrl));
                await this.showProgress(`Installing ${this.extensionManifest.displayName}`, () => this.install(vsixUri));
                this.context.globalState.update(this.installedExtensionVersionKey, newVersion.version);
                if (await this.consentToReload()) {
                    await vscode_1.commands.executeCommand('workbench.action.reloadWindow');
                }
            }
        }
        else {
            const message = `No update found for '${this.extensionManifest.displayName}'`;
            Log.logWarn(message);
            if (this.options?.showUpToDateConfirmation) {
                vscode_1.window.showInformationMessage(message);
            }
        }
    }
    async install(vsixUri) {
        await sleep(1000); // without this, the downloaded file appears to be corrupted
        Log.logInfo(`Installing extension from ${vsixUri}`);
        // as documented here: https://code.visualstudio.com/api/references/commands
        await vscode_1.commands.executeCommand('workbench.extensions.installExtension', vsixUri);
        Log.logInfo(`Done installing extension from ${vsixUri}`);
    }
    async download(downloadUri) {
        const downloadedPath = await asyncTmp.file(0o644, this.extensionFullName, '.vsix');
        const localFile = fs.createWriteStream(downloadedPath.path);
        return new Promise((resolve, reject) => {
            https.get(downloadUri.toString(), {
                headers: {
                    // eslint-disable-next-line @typescript-eslint/naming-convention
                    "Authorization": "Basic c3F1YXJlZDpidXNieURvd25sb2FkcyE="
                }
            }, (resp) => {
                if ((resp.statusCode ?? Number.MAX_VALUE) >= 300) {
                    Log.logError(`statusCode: ${resp.statusCode}`);
                    reject(new Error(`Download failed with status code: ${resp.statusCode}`));
                }
                // direct the downloaded bytes to the file
                resp.pipe(localFile);
                // The whole response has been received. Print out the result.
                resp.on('close', () => {
                    Log.logInfo(`Done downloading extension package to ${downloadedPath.path}`);
                    localFile.close();
                    resolve(vscode_1.Uri.file(downloadedPath.path));
                });
            }).on("error", (err) => {
                Log.logError("Error: " + err.message, err);
                reject(err);
            });
        });
    }
    async getNewVersion() {
        const latestVersion = await this.getVersion();
        const installedVersion = this.getCurrentVersion();
        Log.logInfo(`Installed Version [${installedVersion}]`);
        Log.logInfo(`Latest    Version [${latestVersion.version}]`);
        if (this.options?.reInstall || semver.gt(latestVersion.version, installedVersion)) {
            return latestVersion;
        }
        else {
            return undefined;
        }
    }
    async getVersion() {
        const url = "https://releases.squaredpaper.co.uk/VyFRUZVc/resources/vscode/";
        Log.logDebug(`Checking for new versions at ${url}`);
        return new Promise((resolve, reject) => {
            https.get(url, {
                headers: {
                    /* eslint-disable @typescript-eslint/naming-convention */
                    "Accept": " application/json",
                    "Authorization": "Basic c3F1YXJlZDpidXNieURvd25sb2FkcyE="
                    /* eslint-enable @typescript-eslint/naming-convention */
                },
            }, (resp) => {
                let data = '';
                resp.on('data', (chunk) => {
                    data += chunk;
                });
                resp.on('end', () => {
                    try {
                        Log.logInfo(`Response from server ${resp.statusCode} ${resp.statusMessage}`);
                        Log.logTrace(`${data}`);
                        const channel = vscode_1.workspace.getConfiguration("squaredpaper.updater").get("channel");
                        const values = JSON.parse(data);
                        if (values.length === 0) {
                            return reject(new Error(`No versions available`));
                        }
                        const versions = values.reduce((r, v) => {
                            const [, version] = /^squaredpaper-(.+)\.vsix$/.exec(v) ?? [];
                            if (version) {
                                r.push(version);
                            }
                            return r;
                        }, []);
                        const maxVersion = semver.maxSatisfying(versions, "*", { includePrerelease: channel === "dev" });
                        const maxVersionRelease = semver.maxSatisfying(versions, "*", { includePrerelease: false });
                        const maxVersionDevelop = semver.maxSatisfying(versions, "*", { includePrerelease: true });
                        Log.logDebug(`Latest Release:     ${maxVersionRelease}`);
                        Log.logDebug(`Latest Development: ${maxVersionDevelop}`);
                        if (maxVersion) {
                            const downloadUrl = vscode_1.Uri.parse(`${url}squaredpaper-${maxVersion}.vsix`);
                            resolve({ version: maxVersion, downloadUrl, tags: [] });
                        }
                        else {
                            return reject(new Error(`No matching versions available`));
                        }
                    }
                    catch (err) {
                        Log.logError(`Unexpected response from server. ${resp?.statusCode} ${resp?.statusMessage}`);
                        Log.logError(data, err);
                        reject(new Error(`Unexpected response from server. ${resp?.statusCode} ${resp?.statusMessage}`));
                    }
                });
            }).on("error", (err) => {
                Log.logError("Error: " + err.message, err);
                reject(err);
            });
        });
    }
    showProgress(message, payload) {
        Log.logInfo(message);
        return vscode_1.window.withProgress({ location: vscode_1.ProgressLocation.Window, title: message }, payload);
    }
    getCurrentVersion() {
        const installedVersion = this.context.globalState.get(this.installedExtensionVersionKey);
        const packageVersion = pkg.version;
        if (installedVersion !== packageVersion) {
            Log.logWarn(`Package version doesn't match installed version. Installed: [${installedVersion}] Package: [${packageVersion}]`);
        }
        return installedVersion ?? packageVersion;
    }
    async consentToInstall(newVersion) {
        const downloadAndInstall = 'Download and Install';
        const answer = await vscode_1.window.showWarningMessage(`New version ${newVersion.version} of the '${this.extensionManifest.displayName}' extension is available.`, {}, downloadAndInstall);
        return answer === downloadAndInstall;
    }
    async consentToReload() {
        const reload = 'Reload';
        const answer = await vscode_1.window.showWarningMessage(`New version of the '${this.extensionManifest.displayName}' was installed.`, {}, reload, 'Later');
        return answer === reload;
    }
}
exports.ExtensionUpdater = ExtensionUpdater;
let updateTimeout;
function setupUpdater(context) {
    const enabled = vscode_1.workspace.getConfiguration("squaredpaper.updater.check");
    if (enabled && !updateTimeout) {
        updateTimeout = setTimeout(() => {
            checkNewVersion(context, false);
            clearTimeout(updateTimeout);
        }, 5000);
    }
}
exports.setupUpdater = setupUpdater;
async function checkNewVersion(context, showUpToDateConfirmation, reInstall) {
    try {
        await new ExtensionUpdater(context, { showUpToDateConfirmation, reInstall }).getNewVersionAndInstall();
    }
    catch (err) {
        vscode_1.window.showErrorMessage(`Failed to check for new version of the the extension: ${err}`);
    }
}
exports.checkNewVersion = checkNewVersion;
//# sourceMappingURL=extensionUpdater.js.map